@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009-2018 Yelp and Contributors
+# Copyright 2009-2020 Yelp and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 """Write and run Hadoop Streaming jobs on
 Amazon Elastic MapReduce / Google Dataproc / your own Hadoop cluster."""
 
-__author__ = 'David Marin <dm@davidmarin.org>'
+__author__ = 'Coyote Codornices Marin <coyote@coyote.rocks>'
 
 __credits__ = [
     'Marc Abramowitz <msabramo@gmail.com>',
@@ -30,6 +30,7 @@ __credits__ = [
     'Hunter Blanks <hblanks@monetate.com>',
     'Jim Blomo <jblomo@yelp.com>',
     'Reno Bowen <renobowen@gmail.com>',
+    'Stephen Brennan <sbrennan@yelp.com>',
     'James Brown <jbrown@uber.com>',
     'Kevin Burke <kevin@twilio.com>',
     'Tim Chan <macrotim@gmail.com>',
@@ -37,14 +38,17 @@ __credits__ = [
     'Ben Chess <bchess@yelp.com>',
     'Jordan Christensen <jc@kobo.com>',
     'Jonathan Chu <milki@yelp.com>',
+    'Ben Dalling <b.dalling@locp.co.uk>',
     'David Dehghan <ddehghan@gmail.com>',
     'Adam Derewecki <derewecki@gmail.com>',
     'Nick Dimiduk <ndimiduk@gmail.com>',
     'Ciaran Doherty <cad106uk@gmail.com>',
     'Tom Dooner <tomdooner@gmail.com>',
     'Tomer Elmalem <telmalem@gmail.com>',
+    'Stuart Elston <selston@yelp.com>',
     'Pedro Emanuel de Castro Faria Salgado <steenzout@ymail.com>',
     'Buck Evan <buck@yelp.com>',
+    'Aaron Friedlander <aaron.friedlander@affirm.com>',
     'Cesar Gonzalez Fernandez <cesar.gon.fer@gmail.com>',
     'Dan Frank <danielhfrank@gmail.com>',
     'Sudarshan Gaikaiwari <sudarshan@acm.org>',
@@ -52,6 +56,7 @@ __credits__ = [
     'Brett Gibson <brett@swiftserve.com>',
     'Benjamin Goldenberg <benjamin@yelp.com',
     'Thomas Grainger <tagrain@gmail.com>',
+    'Jean Gressmann <jean.steven.gressmann@iav.de>',
     'Peter Harrington <peter.b.harrington@gmail.com>',
     'Brandon Haynes <bhaynes@fas.harvard.edu>',
     'Tim Henderson <tim.tadh@gmail.com>',
@@ -65,13 +70,16 @@ __credits__ = [
     'Matt Jones <matt@mhjones.org>',
     'Mike Kaplinskiy <mkaplinskiy@twitter.com>',
     'Kamil Kisiel <kamil@kamilkisiel.net>',
+    'Susannah Klaneček <suzil@protonmail.ch>',
     'Evan Klitzke <evan@eklitzke.org>',
     'Alex Konradi <alexkonradi@gmail.com>',
     'Nikolaos Koutsopoulos <nhk@mochimedia.com>',
     'Julian Krause <juliank@yelp.com>',
     'Pai-Wei Lai <paiwei@yelp.com>',
     'Boris Lau <boris.w.lau@gmail.com>',
+    'Laucia <laucia@users.noreply.github.com>',
     'Jerome Leclanche <jerome@leclan.ch>',
+    'Lydian Lee <tingyenl@yelp.com>',
     'Robert Leftwich <rl.0x0@eml.cc>',
     'Andrew Lenards <andrew.lenards@gmail.com>',
     'Ronny Lindner <ronny.lindner@invia.de>',
@@ -93,6 +101,7 @@ __credits__ = [
     'Spencer Nelson <s@spenczar.com>',
     'Dávid Nemeskey <david@cliqz.com>',
     'Daniel Nephin <dnephin@yelp.com>',
+    'Qui Nguyen <qui@yelp.com>',
     'Oliver Nicholas <bigo@wonlove.net>',
     'Tyler Nisonoff <tyler@asana.com>',
     'Santeri Paavolainen <santtu@iki.fi>',
@@ -114,6 +123,7 @@ __credits__ = [
     'David Selassie <selassid@gmail.com>',
     'Alex Shkop <a.v.shkop@gmail.com>',
     'Jesse Shieh <jesse@adku.com>',
+    'Manpreet Singh <manpreet@yelp.com>',
     'Isaac Slavitt <isaac.slavitt@gmail.com>',
     'Anthony Sottile <asottile@umich.edu>',
     'Steve Spencer <steve@bigfrog.net>',
@@ -128,6 +138,7 @@ __credits__ = [
     'Derek Wilson <jderekwilson@gmail.com>',
     'Tao Yu <taoyu@yelp.com>',
     'Andrea Zonca <andrea.zonca@gmail.com>',
+    'drulludanni <drulludanni5@gmail.com>',
 ]
 
-__version__ = '0.6.3'
+__version__ = '0.7.4'
