@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2009-2017 Yelp and Contributors
-# Copyright 2018 Yelp
+# Copyright 2018-2019 Yelp
+# Copyright 2020 Affirm, Inc. and Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,25 +17,43 @@
 """Abstract base class for all runners that execute binaries/scripts
 (that is, everything but inline mode).
 """
-import json
 import logging
 import os
+import os.path
 import pipes
 import re
 import sys
+from mrjob.py2 import PY2
+from platform import python_implementation
 from subprocess import Popen
 from subprocess import PIPE
 
+try:
+    import pty
+    pty  # quiet "redefinition of unused ..." warning from pyflakes
+except ImportError:
+    pty = None
+
+try:
+    import pyspark
+    pyspark  # quiet "redefinition of unused ..." warning from pyflakes
+except ImportError:
+    pyspark = None
+
 import mrjob.step
 from mrjob.compat import translate_jobconf
+from mrjob.conf import combine_cmds
 from mrjob.conf import combine_dicts
-from mrjob.conf import combine_local_envs
-from mrjob.py2 import PY2
+from mrjob.logs.log4j import _parse_hadoop_log4j_records
+from mrjob.logs.spark import _parse_spark_log
+from mrjob.logs.step import _eio_to_eof
 from mrjob.py2 import string_types
 from mrjob.runner import MRJobRunner
-from mrjob.step import _is_spark_step_type
+from mrjob.setup import parse_setup_cmd
 from mrjob.util import cmd_line
 from mrjob.util import shlex_split
+from mrjob.util import unique
+from mrjob.util import which
 from mrjob.util import zip_dir
 
 log = logging.getLogger(__name__)
@@ -45,17 +64,33 @@ _HADOOP_SAFE_ARG_RE = re.compile(r'^[\w\./=-]*$')
 # used to handle manifest files
 _MANIFEST_INPUT_FORMAT = 'org.apache.hadoop.mapred.lib.NLineInputFormat'
 
+# map archive file extensions to the command used to unarchive them
+_EXT_TO_UNARCHIVE_CMD = {
+    '.zip': 'unzip -o %(file)s -d %(dir)s',
+    '.tar': 'mkdir %(dir)s; tar xf %(file)s -C %(dir)s',
+    '.tar.gz': 'mkdir %(dir)s; tar xfz %(file)s -C %(dir)s',
+    '.tgz': 'mkdir %(dir)s; tar xfz %(file)s -C %(dir)s',
+}
+
+
+def _unarchive_cmd(path):
+    """Look up the unarchive command to use with the given file extension,
+    or raise KeyError if there is no matching command."""
+    for ext, unarchive_cmd in sorted(_EXT_TO_UNARCHIVE_CMD.items()):
+        # use this so we can match e.g. mrjob-0.7.0.tar.gz
+        if path.endswith(ext):
+            return unarchive_cmd
+
+    raise KeyError('unknown archive type: %s' % path)
+
 
 class MRJobBinRunner(MRJobRunner):
 
     OPT_NAMES = MRJobRunner.OPT_NAMES | {
-        'interpreter',
         'python_bin',
         'sh_bin',
         'spark_args',
         'spark_submit_bin',
-        'steps_interpreter',
-        'steps_python_bin',
         'task_python_bin',
     }
 
@@ -68,74 +103,77 @@ class MRJobBinRunner(MRJobRunner):
         # we'll create the setup wrapper scripts later
         self._setup_wrapper_script_path = None
         self._manifest_setup_script_path = None
+        self._spark_python_wrapper_path = None
 
-    def _default_opts(self):
+        # self._setup is a list of shell commands with path dicts
+        # interleaved; see mrjob.setup.parse_setup_cmd() for details
+        self._setup = [parse_setup_cmd(cmd) for cmd in self._opts['setup']]
+
+        for cmd in self._setup:
+            for token in cmd:
+                if isinstance(token, dict):
+                    # convert dir archives tokens to archives
+                    if token['type'] == 'dir':
+                        # feed the archive's path to self._working_dir_mgr
+                        token['path'] = self._dir_archive_path(token['path'])
+                        token['type'] = 'archive'
+
+                    self._working_dir_mgr.add(**token)
+
+        # warning: no setup scripts on Spark when no working dir
+        if self._setup and self._has_pyspark_steps() and not(
+                self._spark_executors_have_own_wd()):
+            log.warning("setup commands aren't supported on Spark master %r" %
+                        self._spark_master())
+
+        # --py-files on Spark doesn't allow '#' (see #1375)
+        if any('#' in path for path in self._opts['py_files']):
+            raise ValueError("py_files cannot contain '#'")
+
+        # Keep track of where the spark-submit binary is
+        self._spark_submit_bin = self._opts['spark_submit_bin']
+
+    @classmethod
+    def _default_opts(cls):
         return combine_dicts(
-            super(MRJobBinRunner, self)._default_opts(),
+            super(MRJobBinRunner, cls)._default_opts(),
             dict(
-                sh_bin=['sh', '-ex'],
+                read_logs=True,
             )
         )
 
-    def _load_steps(self):
-        if not self._script_path:
-            return []
+    def _fix_opt(self, opt_key, opt_value, source):
+        """Check sh_bin"""
+        opt_value = super(MRJobBinRunner, self)._fix_opt(
+            opt_key, opt_value, source)
 
-        args = (self._executable(True) + ['--steps'] +
-                self._mr_job_extra_args(local=True))
-        log.debug('> %s' % cmd_line(args))
+        # check that sh_bin doesn't have too many args
+        if opt_key == 'sh_bin':
+            # opt_value is usually a string, combiner makes it a list of args
+            sh_bin = combine_cmds(opt_value)
 
-        # add . to PYTHONPATH (in case mrjob isn't actually installed)
-        env = combine_local_envs(os.environ,
-                                 {'PYTHONPATH': os.path.abspath('.')})
-        steps_proc = Popen(args, stdout=PIPE, stderr=PIPE, env=env)
-        stdout, stderr = steps_proc.communicate()
+            # empty sh_bin just means to use the default, see #1926
 
-        if steps_proc.returncode != 0:
-            raise Exception(
-                'error getting step information: \n%s' % stderr)
+            # make these hard requirements in v0.7.0?
+            if len(sh_bin) > 1 and not os.path.isabs(sh_bin[0]):
+                log.warning('sh_bin (from %s) should use an absolute path'
+                            ' if you want it to take arguments' % source)
+            elif len(sh_bin) > 2:
+                log.warning('sh_bin (from %s) should not take more than one'
+                            ' argument' % source)
 
-        # on Python 3, convert stdout to str so we can json.loads() it
-        if not isinstance(stdout, str):
-            stdout = stdout.decode('utf_8')
+        return opt_value
 
-        try:
-            steps = json.loads(stdout)
-        except ValueError:
-            raise ValueError("Bad --steps response: \n%s" % stdout)
-
-        # verify that this is a proper step description
-        if not steps or not stdout:
-            raise ValueError('step description is empty!')
-
-        return steps
-
-    ### interpreter/python binary ###
-
-    def _interpreter(self, steps=False):
-        if steps:
-            return (self._opts['steps_interpreter'] or
-                    self._opts['interpreter'] or
-                    self._steps_python_bin())
-        else:
-            return (self._opts['interpreter'] or
-                    self._task_python_bin())
-
-    def _executable(self, steps=False):
-        if steps:
-            return self._interpreter(steps=True) + [self._script_path]
-        else:
-            return self._interpreter() + [
-                self._working_dir_mgr.name('file', self._script_path)]
+    ### python binary ###
 
     def _python_bin(self):
         """Python binary used for everything other than invoking the job.
-        For invoking jobs with ``--steps``, see :py:meth:`_steps_python_bin`,
-        and for everything else (e.g. ``--mapper``, ``--spark``), see
-        :py:meth:`_task_python_bin`, which defaults to this method if
-        :mrjob-opt:`task_python_bin` isn't set.
 
-        Other ways mrjob uses Python:
+        For running job tasks (e.g. ``--mapper``, ``--spark``), we use
+        :py:meth:`_task_python_bin`, which can be set to a different value
+        by setting :mrjob-opt:`task_python_bin`.
+
+        Ways mrjob uses Python other than running tasks:
          * file locking in setup wrapper scripts
          * finding site-packages dir to bootstrap mrjob on clusters
          * invoking ``cat.py`` in local mode
@@ -144,11 +182,6 @@ class MRJobBinRunner(MRJobRunner):
         # python_bin isn't an option for inline runners
         return self._opts['python_bin'] or self._default_python_bin()
 
-    def _steps_python_bin(self):
-        """Python binary used to invoke job with ``--steps``"""
-        return (self._opts['steps_python_bin'] or
-                self._default_python_bin(local=True))
-
     def _task_python_bin(self):
         """Python binary used to invoke job with ``--mapper``,
         ``--reducer``, ``--spark``, etc."""
@@ -157,22 +190,26 @@ class MRJobBinRunner(MRJobRunner):
 
     def _default_python_bin(self, local=False):
         """The default python command. If local is true, try to use
-        sys.executable. Otherwise use 'python' or 'python3' as appropriate.
+        sys.executable. Otherwise use 'python2.7' or 'python3' as appropriate.
 
         This returns a single-item list (because it's a command).
         """
+        is_pypy = (python_implementation() == 'PyPy')
+
         if local and sys.executable:
             return [sys.executable]
-        elif PY2:
-            return ['python']
         else:
-            # e.g. python3
-            return ['python%d' % sys.version_info[0]]
+            if PY2:
+                return ['pypy'] if is_pypy else ['python2.7']
+            else:
+                return ['pypy3'] if is_pypy else ['python3']
 
     ### running MRJob scripts ###
 
     def _script_args_for_step(self, step_num, mrc, input_manifest=False):
-        args = self._executable() + self._args_for_task(step_num, mrc)
+        args = (self._task_python_bin() +
+                [self._working_dir_mgr.name('file', self._script_path)] +
+                self._args_for_task(step_num, mrc))
 
         if input_manifest and mrc == 'mapper':
             wrapper = self._manifest_setup_script_path
@@ -250,7 +287,7 @@ class MRJobBinRunner(MRJobRunner):
             args.extend(['-partitioner', partitioner])
 
         # cmdenv
-        for key, value in sorted(self._opts['cmdenv'].items()):
+        for key, value in sorted(self._cmdenv().items()):
             args.append('-cmdenv')
             args.append('%s=%s' % (key, value))
 
@@ -335,8 +372,7 @@ class MRJobBinRunner(MRJobRunner):
         jobconf = self._jobconf_for_step(step_num)
 
         for key, value in sorted(jobconf.items()):
-            if value is not None:
-                args.extend(['-D', '%s=%s' % (key, value)])
+            args.extend(['-D', '%s=%s' % (key, value)])
 
         return args
 
@@ -347,9 +383,37 @@ class MRJobBinRunner(MRJobRunner):
         """
         return self._opts['libjars']
 
+    def _interpolate_jar_step_args(self, args, step_num):
+        """Like :py:meth:`_interpolate_step_args` except it
+        also replaces `~mrjob.step.GENERIC_ARGS` with
+        :py:meth:`_hadoop_generic_args_for_step`. This only
+        makes sense for jar steps; Spark should raise an error
+        if `~mrjob.step.GENERIC_ARGS` is encountered.
+        """
+        result = []
+
+        for arg in args:
+            if arg == mrjob.step.GENERIC_ARGS:
+                result.extend(
+                    self._hadoop_generic_args_for_step(step_num))
+            else:
+                result.append(arg)
+
+        return self._interpolate_step_args(result, step_num)
+
     ### setup scripts ###
 
-    # TODO: rename to _setup_wrapper_scripts()
+    def _py_files(self):
+        """Everything in the *py_files* opt, plus a .zip of the mrjob
+        library if needed.
+        """
+        py_files = list(self._opts['py_files'])
+
+        if self._bootstrap_mrjob():
+            py_files.append(self._create_mrjob_zip())
+
+        return py_files
+
     def _create_setup_wrapper_scripts(self):
         """Create the setup wrapper script, and write it into our local temp
         directory (by default, to a file named setup-wrapper.sh).
@@ -363,37 +427,78 @@ class MRJobBinRunner(MRJobRunner):
         If *local* is true, use local line endings (e.g. Windows). Otherwise,
         use UNIX line endings (see #1071).
         """
-        setup = self._setup
+        if self._has_hadoop_streaming_steps():
+            streaming_setup = self._py_files_setup() + self._setup
 
-        if self._bootstrap_mrjob() and self._BOOTSTRAP_MRJOB_IN_SETUP:
-            # patch setup to add mrjob.zip to PYTHONPATH
-            mrjob_zip = self._create_mrjob_zip()
-            # this is a file, not an archive, since Python can import directly
-            # from .zip files
-            path_dict = {'type': 'file', 'name': None, 'path': mrjob_zip}
+            if streaming_setup and not self._setup_wrapper_script_path:
+
+                self._setup_wrapper_script_path = self._write_setup_script(
+                    streaming_setup, 'setup-wrapper.sh',
+                    'streaming setup wrapper script')
+
+            if (self._uses_input_manifest() and not
+                    self._manifest_setup_script_path):
+
+                self._manifest_setup_script_path = self._write_setup_script(
+                    streaming_setup, 'manifest-setup.sh',
+                    'manifest setup wrapper script',
+                    manifest=True)
+
+        if (self._has_pyspark_steps() and
+                self._spark_executors_have_own_wd() and
+                not self._spark_python_wrapper_path):
+
+            pyspark_setup = self._pyspark_setup()
+            if pyspark_setup:
+                self._spark_python_wrapper_path = self._write_setup_script(
+                    pyspark_setup,
+                    'python-wrapper.sh', 'Spark Python wrapper script',
+                    wrap_python=True)
+
+    def _pyspark_setup(self):
+        """Like ``self._setup``, but prepends commands for archive
+        emulation if needed."""
+        setup = []
+
+        if self._emulate_archives_on_spark():
+            for name, path in sorted(
+                    self._working_dir_mgr.name_to_path('archive').items()):
+
+                archive_file_name = self._working_dir_mgr.name(
+                    'archive_file', path)
+
+                setup.append(_unarchive_cmd(path) % dict(
+                    file=pipes.quote(archive_file_name),
+                    dir=pipes.quote(name)))
+
+        setup.extend(self._setup)
+
+        return setup
+
+    def _py_files_setup(self):
+        """A list of additional setup commands to emulate Spark's
+        --py-files option on Hadoop Streaming."""
+        result = []
+
+        for py_file in self._py_files():
+            path_dict = {'type': 'file', 'name': None, 'path': py_file}
             self._working_dir_mgr.add(**path_dict)
-            setup = [['export PYTHONPATH=', path_dict, ':$PYTHONPATH']] + setup
+            result.append(['export PYTHONPATH=', path_dict, ':$PYTHONPATH'])
 
-        if setup and not self._setup_wrapper_script_path:
+        return result
 
-            contents = self._setup_wrapper_script_content(setup)
-            path = os.path.join(self._get_local_tmp_dir(), 'setup-wrapper.sh')
+    def _write_setup_script(self, setup, filename, desc,
+                            manifest=False, wrap_python=False):
+        """Write a setup script and return its path."""
+        contents = self._setup_wrapper_script_content(
+            setup, manifest=manifest, wrap_python=wrap_python)
 
-            self._write_script(contents, path, 'setup wrapper script')
+        path = os.path.join(self._get_local_tmp_dir(), filename)
+        self._write_script(contents, path, desc)
 
-            self._setup_wrapper_script_path = path
-            self._working_dir_mgr.add('file', self._setup_wrapper_script_path)
+        self._working_dir_mgr.add('file', path)
 
-        if (self._uses_input_manifest() and not
-                self._manifest_setup_script_path):
-
-            contents = self._setup_wrapper_script_content(setup, manifest=True)
-            path = os.path.join(self._get_local_tmp_dir(), 'manifest-setup.sh')
-
-            self._write_script(contents, path, 'manifest setup script')
-
-            self._manifest_setup_script_path = path
-            self._working_dir_mgr.add('file', self._manifest_setup_script_path)
+        return path
 
     def _create_mrjob_zip(self):
         """Make a zip of the mrjob library, without .pyc or .pyo files,
@@ -437,7 +542,8 @@ class MRJobBinRunner(MRJobRunner):
 
         return self._mrjob_zip_path
 
-    def _setup_wrapper_script_content(self, setup, manifest=False):
+    def _setup_wrapper_script_content(
+            self, setup, manifest=False, wrap_python=False):
         """Return a (Bourne) shell script that runs the setup commands and then
         executes whatever is passed to it (this will be our mapper/reducer),
         as a list of strings (one for each line, including newlines).
@@ -448,6 +554,25 @@ class MRJobBinRunner(MRJobRunner):
         """
         lines = []
 
+        # TODO: this is very similar to _start_of_sh_script() in cloud.py
+
+        if wrap_python:
+            # start with shebang
+            sh_bin = self._sh_bin()
+
+            if os.path.isabs(sh_bin[0]):
+                shebang_bin = sh_bin
+            else:
+                shebang_bin = ['/usr/bin/env'] + list(sh_bin)
+
+            if len(shebang_bin) > 2:
+                # Linux limits shebang to one binary and one arg
+                shebang_bin = shebang_bin[:2]
+                log.warning('Limiting shebang to two arguments:'
+                            '#!%s' % cmd_line(shebang_bin))
+
+            lines.append('#!%s' % cmd_line(shebang_bin))
+
         # hook for 'set -e', etc.
         pre_commands = self._sh_pre_commands()
         if pre_commands:
@@ -458,11 +583,17 @@ class MRJobBinRunner(MRJobRunner):
         if setup:
             lines.extend(self._setup_cmd_content(setup))
 
-        # we're always going to execute this script as an argument to
-        # sh, so there's no need to add a shebang (e.g. #!/bin/sh)
-        if manifest:
+        # handle arguments to the script
+        if wrap_python:
+            # pretend to be python ($@ is arguments to the python binary)
+            python_bin = self._task_python_bin()
+            lines.append('%s "$@"' % cmd_line(python_bin))
+        elif manifest:
+            # arguments ($@) are a command
+            # eventually runs: "$@" $INPUT_PATH $INPUT_URI
             lines.extend(self._manifest_download_content())
         else:
+            # arguments ($@) are a command, just run it
             lines.append('"$@"')
 
         return lines
@@ -564,7 +695,7 @@ class MRJobBinRunner(MRJobRunner):
             lines.append('    *.%s)' % ext)
             lines.append('      %s $INPUT_PATH' % cmd)
             lines.append("      INPUT_PATH="
-                         "$(echo $INPUT_PATH | sed -e 's/\.%s$//')" % ext)
+                         r"$(echo $INPUT_PATH | sed -e 's/\.%s$//')" % ext)
             lines.append('      ;;')
         lines.append('  esac')
         lines.append('} 1>&2')
@@ -579,7 +710,7 @@ class MRJobBinRunner(MRJobRunner):
 
         # save return code, turn off echo
         lines.append('# if script fails, print input URI before exiting')
-        lines.append('{ RETURNCODE=$?; set +x; } &> /dev/null')
+        lines.append('{ RETURNCODE=$?; set +x; } 1>&2 2>/dev/null')
         lines.append('')
 
         lines.append('{')
@@ -626,7 +757,11 @@ class MRJobBinRunner(MRJobRunner):
         if, for example, a runner needs different default values
         depending on circumstances (see :py:class:`~mrjob.emr.EMRJobRunner`).
         """
-        return self._opts['sh_bin']
+        return self._opts['sh_bin'] or self._default_sh_bin()
+
+    def _default_sh_bin(self):
+        """The default sh binary, if :mrjob-opt:`sh_bin` isn't set."""
+        return ['/bin/sh', '-ex']
 
     def _sh_pre_commands(self):
         """A list of lines to put at the very start of any sh script
@@ -646,63 +781,156 @@ class MRJobBinRunner(MRJobRunner):
 
     ### spark ###
 
-    def _args_for_spark_step(self, step_num):
+    def _args_for_spark_step(self, step_num, last_step_num=None):
         """The actual arguments used to run the spark-submit command.
 
         This handles both all Spark step types (``spark``, ``spark_jar``,
         and ``spark_script``).
+
+        *last_step_num* is only used by the Spark runner, where multiple
+        streaming steps are run in a single Spark job
         """
         return (
             self.get_spark_submit_bin() +
             self._spark_submit_args(step_num) +
             [self._spark_script_path(step_num)] +
-            self._spark_script_args(step_num)
+            self._spark_script_args(step_num, last_step_num)
         )
 
-    def _spark_script_args(self, step_num):
-        """A list of args to the spark script/jar, used by
-        _args_for_spark_step()."""
-        step = self._get_step(step_num)
+    def _run_spark_submit(self, spark_submit_args, env, record_callback):
+        """Run the spark submit binary in a subprocess, using a PTY if possible
 
-        if step['type'] == 'spark':
-            args = (
-                [
-                    '--step-num=%d' % step_num,
-                    '--spark',
-                ] + self._mr_job_extra_args() + [
-                    mrjob.step.INPUT,
-                    mrjob.step.OUTPUT,
-                ]
-            )
-        elif step['type'] in ('spark_jar', 'spark_script'):
-            args = step['args']
+        :param spark_submit_args: spark-submit binary and arguments, as as list
+        :param env: environment variables, as a dict
+        :param record_callback: a function that takes a single log4j record
+                                as its argument (see
+                                :py:func:`~mrjob.logs.log4j\
+                                ._parse_hadoop_log4j_records)
+
+        :return: tuple of the subprocess's return code and a
+                 step interpretation dictionary
+        """
+        log.debug('> %s' % cmd_line(spark_submit_args))
+        log.debug('  with environment: %r' % sorted(env.items()))
+
+        # these should always be set, but just in case
+        returncode = 0
+        step_interpretation = {}
+
+        # try to use a PTY if it's available
+        try:
+            pid, master_fd = pty.fork()
+        except (AttributeError, OSError):
+            # no PTYs, just use Popen
+
+            # user won't get much feedback for a while, so tell them
+            # spark-submit is running
+            log.debug('No PTY available, using Popen() to invoke spark-submit')
+
+            step_proc = Popen(
+                spark_submit_args, stdout=PIPE, stderr=PIPE, env=env)
+
+            # parse driver output
+            step_interpretation = _parse_spark_log(
+                step_proc.stderr, record_callback=record_callback)
+
+            # there shouldn't be much output on STDOUT, just echo it
+            for record in _parse_hadoop_log4j_records(step_proc.stdout):
+                record_callback(record)
+
+            step_proc.stdout.close()
+            step_proc.stderr.close()
+
+            returncode = step_proc.wait()
         else:
-            raise TypeError('Bad step type: %r' % step['type'])
+            # we have PTYs
+            if pid == 0:  # we are the child process
+                try:
+                    os.execvpe(spark_submit_args[0], spark_submit_args, env)
+                    # now this process is no longer Python
+                except OSError as ex:
+                    # use _exit() so we don't do cleanup, etc. that's
+                    # the parent process's job
+                    os._exit(ex.errno)
+                finally:
+                    # if we get some other exception, still exit hard
+                    os._exit(-1)
+            else:
+                log.debug('Invoking spark-submit via PTY')
 
-        return self._interpolate_input_and_output(args, step_num)
+                with os.fdopen(master_fd, 'rb') as master:
+                    step_interpretation = (
+                        _parse_spark_log(
+                            _eio_to_eof(master),
+                            record_callback=record_callback))
+
+                    _, returncode = os.waitpid(pid, 0)
+
+        return (returncode, step_interpretation)
 
     def get_spark_submit_bin(self):
-        """The spark-submit command, as a list of args. Re-define
-        this in your subclass for runner-specific behavior.
-        """
-        return self._opts['spark_submit_bin'] or ['spark-submit']
+        """Return the location of the ``spark-submit`` binary, searching for it
+        if necessary."""
+        if not self._spark_submit_bin:
+            self._spark_submit_bin = self._find_spark_submit_bin()
+        return self._spark_submit_bin
 
-    def _spark_submit_arg_prefix(self):
-        """Runner-specific args to spark submit (e.g. ['--master', 'yarn'])"""
-        return []
+    def _find_spark_submit_bin(self):
+        """Attempt to find the spark binary. Returns a list of arguments.
+        Defaults to ``['spark-submit']``.
+
+        Re-define this in your subclass if you already know where
+        to find spark-submit (e.g. on cloud services).
+        """
+        for path in unique(self._spark_submit_bin_dirs()):
+            log.info('Looking for spark-submit binary in %s...' % (
+                path or '$PATH'))
+
+            spark_submit_bin = which('spark-submit', path=path)
+
+            if spark_submit_bin:
+                log.info('Found spark-submit binary: %s' % spark_submit_bin)
+                return [spark_submit_bin]
+        else:
+            log.info("Falling back to 'spark-submit'")
+            return ['spark-submit']
+
+    def _spark_submit_bin_dirs(self):
+        # $SPARK_HOME
+        spark_home = os.environ.get('SPARK_HOME')
+        if spark_home:
+            yield os.path.join(spark_home, 'bin')
+
+        yield None  # use $PATH
+
+        # look for pyspark installation (see #1984)
+        if pyspark:
+            yield os.path.join(os.path.dirname(pyspark.__file__), 'bin')
+
+        # some other places recommended by install docs (see #1366)
+        yield '/usr/lib/spark/bin'
+        yield '/usr/local/spark/bin'
+        yield '/usr/local/lib/spark/bin'
 
     def _spark_submit_args(self, step_num):
         """Build a list of extra args to the spark-submit binary for
         the given spark or spark_script step."""
         step = self._get_step(step_num)
 
-        if not _is_spark_step_type(step['type']):
-            raise TypeError('non-Spark step: %r' % step)
-
         args = []
 
-        # add runner-specific args
-        args.extend(self._spark_submit_arg_prefix())
+        # --conf arguments include python bin, cmdenv, jobconf. Make sure
+        # that we can always override these manually
+        jobconf = {}
+        for key, value in self._spark_cmdenv(step_num).items():
+            jobconf['spark.executorEnv.%s' % key] = value
+            if self._spark_master() == 'yarn':  # YARN only, see #1919
+                jobconf['spark.yarn.appMasterEnv.%s' % key] = value
+
+        jobconf.update(self._jobconf_for_step(step_num))
+
+        for key, value in sorted(jobconf.items()):
+            args.extend(['--conf', '%s=%s' % (key, value)])
 
         # add --class (JAR steps)
         if step.get('main_class'):
@@ -713,42 +941,60 @@ class MRJobBinRunner(MRJobRunner):
         if libjar_paths:
             args.extend(['--jars', ','.join(libjar_paths)])
 
-        # --conf arguments include python bin, cmdenv, jobconf. Make sure
-        # that we can always override these manually
-        jobconf = {}
-        for key, value in self._spark_cmdenv(step_num).items():
-            jobconf['spark.executorEnv.%s' % key] = value
-            jobconf['spark.yarn.appMasterEnv.%s' % key] = value
+        # spark-submit treats --master and --deploy-mode as aliases for
+        # --conf spark.master=... and --conf spark.deploy-mode=... (see #2032).
+        #
+        # we never want jobconf to override spark master or deploy mode, so put
+        # these switches after --conf
 
-        jobconf.update(self._jobconf_for_step(step_num))
+        # add --master
+        if self._spark_master():
+            args.extend(['--master', self._spark_master()])
 
-        for key, value in sorted(jobconf.items()):
-            if value is not None:
-                args.extend(['--conf', '%s=%s' % (key, value)])
+        # add --deploy-mode
+        if self._spark_deploy_mode():
+            args.extend(['--deploy-mode', self._spark_deploy_mode()])
 
         # --files and --archives
         args.extend(self._spark_upload_args())
 
         # --py-files (Python only)
-        if step['type'] in ('spark', 'spark_script'):
-            py_files_arg = ','.join(self._spark_py_files())
-            if py_files_arg:
-                args.extend(['--py-files', py_files_arg])
+        # spark runner can run 'streaming' steps, so just exclude
+        # non-Python steps
+        if 'jar' not in step['type']:
+            py_file_uris = self._py_files()
+
+            if self._upload_mgr:
+                # don't assume py_files are in _upload_mgr; for example,
+                # spark-submit doesn't need to upload them
+                path_to_uri = self._upload_mgr.path_to_uri()
+                py_file_uris = [path_to_uri.get(p, p) for p in py_file_uris]
+
+            if py_file_uris:
+                args.extend(['--py-files', ','.join(py_file_uris)])
 
         # spark_args option
         args.extend(self._opts['spark_args'])
 
         # step spark_args
-        args.extend(step['spark_args'])
+        if step.get('spark_args'):
+            args.extend(step['spark_args'])
 
         return args
 
     def _spark_upload_args(self):
-        return self._upload_args_helper('--files', self._spark_files,
-                                        '--archives', self._spark_archives)
+        if not self._spark_executors_have_own_wd():
+            # don't bother, there's no working dir to upload to
+            return []
+
+        return self._upload_args_helper(
+            '--files', None,
+            '--archives', None,
+            always_use_hash=False,
+            emulate_archives=self._emulate_archives_on_spark())
 
     def _spark_script_path(self, step_num):
-        """The path of the spark script or har, used by
+        """The path of the spark script or JAR, used by
         _args_for_spark_step()."""
         step = self._get_step(step_num)
 
@@ -776,28 +1022,29 @@ class MRJobBinRunner(MRJobRunner):
 
         cmdenv = {}
 
-        if step['type'] in ('spark', 'spark_script'):  # not spark_jar
-            cmdenv = dict(PYSPARK_PYTHON=cmd_line(self._python_bin()))
-        cmdenv.update(self._opts['cmdenv'])
+        if self._step_type_uses_pyspark(step['type']):
+            driver_python = cmd_line(self._python_bin())
+
+            if self._spark_python_wrapper_path:
+                executor_python = './%s' % self._working_dir_mgr.name(
+                    'file', self._spark_python_wrapper_path)
+            else:
+                executor_python = cmd_line(self._task_python_bin())
+
+            if self._spark_deploy_mode() == 'cluster':
+                # treat driver like executors (they run in same environment)
+                cmdenv['PYSPARK_PYTHON'] = executor_python
+            elif driver_python == executor_python:
+                # no difference, just set $PYSPARK_PYTHON
+                cmdenv['PYSPARK_PYTHON'] = driver_python
+            else:
+                # set different pythons for driver and executor
+                cmdenv['PYSPARK_PYTHON'] = executor_python
+                cmdenv['PYSPARK_DRIVER_PYTHON'] = driver_python
+
+        cmdenv.update(self._cmdenv())
         return cmdenv
 
-    def _spark_py_files(self):
-        """The list of files to pass to spark-submit with --py-files.
-
-        By default (cluster mode), Spark only accepts local files, so
-        we pass these as-is.
-        """
-        py_files = []
-
-        py_files.extend(self._opts['py_files'])
-
-        # Spark doesn't have setup scripts; instead, we need to add
-        # mrjob to py_files
-        if self._bootstrap_mrjob() and self._BOOTSTRAP_MRJOB_IN_SETUP:
-            py_files.append(self._create_mrjob_zip())
-
-        return py_files
-
 
 # these don't need to be methods
 
