@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
-# Copyright 2017 Yelp
-# Copyright 2018 Yelp
+# Copyright 2017-2018 Yelp
+# Copyright 2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,38 +19,32 @@ import os
 import pipes
 import socket
 import random
+import signal
 import time
-from os.path import basename
-from signal import SIGKILL
+from copy import deepcopy
 from subprocess import Popen
 from subprocess import PIPE
 
 from mrjob.bin import MRJobBinRunner
+from mrjob.bin import _unarchive_cmd
 from mrjob.conf import combine_dicts
 from mrjob.py2 import integer_types
 from mrjob.py2 import xrange
 from mrjob.setup import WorkingDirManager
 from mrjob.setup import parse_setup_cmd
 from mrjob.util import cmd_line
-from mrjob.util import file_ext
 
 log = logging.getLogger(__name__)
 
 # don't try to bind SSH tunnel to more than this many local ports
 _MAX_SSH_RETRIES = 20
 
-
-# map archive file extensions to the command used to unarchive them
-_EXT_TO_UNARCHIVE_CMD = {
-    '.zip': 'unzip -o %(file)s -d %(dir)s',
-    '.tar': 'mkdir %(dir)s; tar xf %(file)s -C %(dir)s',
-    '.tar.gz': 'mkdir %(dir)s; tar xfz %(file)s -C %(dir)s',
-    '.tgz': 'mkdir %(dir)s; tar xfz %(file)s -C %(dir)s',
-}
-
 # issue a warning if max_mins_idle is set to less than this
 _DEFAULT_MAX_MINS_IDLE = 10.0
 
+# default part size (so we can share with Spark runner)
+_DEFAULT_CLOUD_PART_SIZE_MB = 100
+
 
 class HadoopInTheCloudJobRunner(MRJobBinRunner):
     """Abstract base class for all Hadoop-in-the-cloud services."""
@@ -67,11 +62,11 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
         'core_instance_type',
         'extra_cluster_params',
         'hadoop_streaming_jar',
+        'image_id',
         'image_version',
         'instance_type',
         'master_instance_type',
         'max_mins_idle',
-        'max_hours_idle',
         'num_core_instances',
         'num_task_instances',
         'region',
@@ -82,9 +77,6 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
         'zone',
     }
 
-    # so far, every service provides the ability to run bootstrap scripts
-    _BOOTSTRAP_MRJOB_IN_SETUP = False
-
     def __init__(self, **kwargs):
         super(HadoopInTheCloudJobRunner, self).__init__(**kwargs)
 
@@ -124,11 +116,12 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
 
     ### Options ###
 
-    def _default_opts(self):
+    @classmethod
+    def _default_opts(cls):
         return combine_dicts(
-            super(HadoopInTheCloudJobRunner, self)._default_opts(),
+            super(HadoopInTheCloudJobRunner, cls)._default_opts(),
             dict(
-                cloud_part_size_mb=100,  # 100 MB
+                cloud_part_size_mb=_DEFAULT_CLOUD_PART_SIZE_MB,
                 max_mins_idle=_DEFAULT_MAX_MINS_IDLE,
                 # don't use a list because it makes it hard to read option
                 # values when running in verbose mode. See #1284
@@ -150,16 +143,6 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
                               (integer_types, float)):
                 raise TypeError('cloud_part_size_mb must be a number')
 
-        # patch max_hours_idle into max_mins_idle (see #1663)
-        if opts.get('max_hours_idle') is not None:
-            log.warning(
-                'max_hours_idle is deprecated and will be removed in v0.7.0.' +
-                (' Please use max_mins_idle instead'
-                 if opts.get('max_mins_idle') is None else ''))
-
-            if opts.get('max_mins_idle') is None:
-                opts['max_mins_idle'] = opts['max_hours_idle'] * 60
-
         return opts
 
     def _combine_opts(self, opt_list):
@@ -239,45 +222,14 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
             return
 
         # Also don't bother if we're not bootstrapping
-        if not (self._bootstrap or self._bootstrap_mrjob()):
+        if not self._bootstrap:
             return
 
-        # create mrjob.zip if we need it, and add commands to install it
-        mrjob_bootstrap = []
-        if self._bootstrap_mrjob():
-            assert self._mrjob_zip_path
-            path_dict = {
-                'type': 'file', 'name': None, 'path': self._mrjob_zip_path}
-            self._bootstrap_dir_mgr.add(**path_dict)
-
-            # find out where python keeps its libraries
-            mrjob_bootstrap.append([
-                "__mrjob_PYTHON_LIB=$(%s -c "
-                "'from distutils.sysconfig import get_python_lib;"
-                " print(get_python_lib())')" %
-                cmd_line(self._python_bin())])
-
-            # remove anything that might be in the way (see #1567)
-            mrjob_bootstrap.append(['sudo rm -rf $__mrjob_PYTHON_LIB/mrjob'])
-
-            # unzip mrjob.zip
-            mrjob_bootstrap.append(
-                ['sudo unzip ', path_dict, ' -d $__mrjob_PYTHON_LIB'])
-
-            # re-compile pyc files now, since mappers/reducers can't
-            # write to this directory. Don't fail if there is extra
-            # un-compileable crud in the tarball (this would matter if
-            # sh_bin were 'sh -e')
-            mrjob_bootstrap.append(
-                ['sudo %s -m compileall -q'
-                 ' -f $__mrjob_PYTHON_LIB/mrjob && true' %
-                 cmd_line(self._python_bin())])
-
         path = os.path.join(self._get_local_tmp_dir(), 'b.sh')
         log.info('writing master bootstrap script to %s' % path)
 
         contents = self._master_bootstrap_script_content(
-            self._bootstrap + mrjob_bootstrap)
+            self._bootstrap)
 
         self._write_script(contents, path, 'master bootstrap script')
 
@@ -340,20 +292,18 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
 
             for name, path in archive_names_and_paths:
                 uri = self._upload_mgr.uri(path)
-                ext = file_ext(basename(path))
+
+                archive_file_name = self._bootstrap_dir_mgr.name(
+                    'archive_file', path)
 
                 # copy file to tmp dir
-                quoted_archive_path = '$__mrjob_TMP/%s' % pipes.quote(name)
+                quoted_archive_path = '$__mrjob_TMP/%s' % pipes.quote(
+                    archive_file_name)
 
                 out.append('  %s %s %s' % (
                     cp_to_local, pipes.quote(uri), quoted_archive_path))
 
-                # unarchive file
-                if ext not in _EXT_TO_UNARCHIVE_CMD:
-                    raise KeyError('unknown archive file extension: %s' % path)
-                unarchive_cmd = _EXT_TO_UNARCHIVE_CMD[ext]
-
-                out.append('  ' + unarchive_cmd % dict(
+                out.append('  ' + _unarchive_cmd(path) % dict(
                     file=quoted_archive_path,
                     dir='$__mrjob_PWD/' + pipes.quote(name)))
 
@@ -409,9 +359,10 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
     def _add_extra_cluster_params(self, params):
         """Return a dict with the *extra_cluster_params* opt patched into
         *params*, and ``None`` values removed."""
-        params = params.copy()
-        params.update(self._opts['extra_cluster_params'])
-        params = {k: v for k, v in params.items() if v is not None}
+        params = deepcopy(params)
+
+        for k, v in sorted(self._opts['extra_cluster_params'].items()):
+            _patch_params(params, k, v)
 
         return params
 
@@ -546,7 +497,11 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
             self._ssh_proc.stderr.close()
 
             try:
-                os.kill(self._ssh_proc.pid, SIGKILL)
+                if hasattr(signal, 'SIGKILL'):
+                    os.kill(self._ssh_proc.pid, signal.SIGKILL)
+                else:
+                    # Windows doesn't have SIGKILL, see #1892
+                    os.kill(self._ssh_proc.pid, signal.SIGABRT)
             except Exception as e:
                 log.exception(e)
 
@@ -592,3 +547,33 @@ class HadoopInTheCloudJobRunner(MRJobBinRunner):
             return random.sample(self._opts['ssh_bind_ports'], num_picks)
         finally:
             random.setstate(random_state)
+
+
+def _patch_params(params, name, value):
+    """Helper method for _add_extra_cluster_params().
+
+    Set *name* in *params* to *value*
+
+    If *name* has one or more dots in it, recursively set the value
+    in successive nested dictionaries, creating them if necessary.
+    For example, if *name* is ``Instances.EmrManagedMasterSecurityGroup``,
+    set ``params['Instances']['EmrManagedMasterSecurityGroup']``
+
+    If *value* is ``None``, delete the value (if it exists), rather than
+    setting it to ``None``.
+    """
+    if not isinstance(params, dict):
+        raise TypeError('must be a dictionary')
+
+    if '.' in name:
+        head, rest = name.split('.', 1)
+        _patch_params(params.setdefault(head, {}), rest, value)
+    elif value is None:
+        if name in params:
+            del params[name]
+    elif isinstance(value, dict) and isinstance(params.get(name), dict):
+        # recursively patch dicts rather than clobbering them (see #2154)
+        for k, v in value.items():
+            _patch_params(params[name], k, v)
+    else:
+        params[name] = value
