@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 # Copyright 2016 Google Inc. and Yelp
 # Copyright 2017 Yelp
-# Copyright 2018 Google Inc.
+# Copyright 2018 Google Inc. and Yelp
+# Copyright 2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,8 +25,8 @@ from os import environ
 
 try:
     import google.auth
-    import mrjob._vendor.dataproc_v1beta2
-    import mrjob._vendor.dataproc_v1beta2.types
+    import google.cloud.dataproc_v1beta2
+    import google.cloud.dataproc_v1beta2.types
     import google.cloud.logging
     import google.api_core.exceptions
     import google.api_core.grpc_helpers
@@ -38,10 +40,9 @@ from mrjob.conf import combine_dicts
 from mrjob.fs.composite import CompositeFilesystem
 from mrjob.fs.gcs import GCSFilesystem
 from mrjob.fs.gcs import is_gcs_uri
-from mrjob.fs.gcs import parse_gcs_uri
 from mrjob.fs.local import LocalFilesystem
 from mrjob.logs.counters import _pick_counters
-from mrjob.logs.errors import _format_error
+from mrjob.logs.errors import _log_probable_cause_of_failure
 from mrjob.logs.mixin import LogInterpretationMixin
 from mrjob.logs.task import _parse_task_stderr
 from mrjob.logs.task import _parse_task_syslog_records
@@ -68,7 +69,7 @@ _DEFAULT_INSTANCE_TYPE = 'n1-standard-1'
 
 # default imageVersion to use on Dataproc. This may be updated with each
 # version of mrjob
-_DEFAULT_IMAGE_VERSION = '1.0'
+_DEFAULT_IMAGE_VERSION = '1.3'
 _DEFAULT_CHECK_CLUSTER_EVERY = 10.0
 _DEFAULT_CLOUD_FS_SYNC_SECS = 5.0
 _DEFAULT_CLOUD_TMP_DIR_OBJECT_TTL_DAYS = 90
@@ -87,7 +88,10 @@ _STATE_MATCHER_ACTIVE = 1
 # for the full list.
 _DATAPROC_IMAGE_TO_HADOOP_VERSION = {
     '0.1': '2.7.1',
-    '1.0': '2.7.2'
+    '1.0': '2.7.2',
+    '1.1': '2.7.7',
+    '1.2': '2.8.5',
+    '1.3': '2.9.2',
 }
 
 _HADOOP_STREAMING_JAR_URI = (
@@ -116,7 +120,7 @@ _CONTAINER_EXIT_RE = re.compile(
     r'Exit code from container (?P<container_id>\w+)'
     r' is ?: (?P<returncode>\d+)')
 
-_TRACEBACK_EXCEPTION_RE = re.compile('\w+: .*$')
+_TRACEBACK_EXCEPTION_RE = re.compile(r'\w+: .*$')
 
 _STDERR_LOG4J_WARNING = re.compile(
     r'.*(No appenders could be found for logger'
@@ -130,12 +134,12 @@ _FULL_SCOPE = 'https://www.googleapis.com/auth/cloud-platform'
 # convert enum values to strings (e.g. 'RUNNING')
 
 def _cluster_state_name(state_value):
-    return mrjob._vendor.dataproc_v1beta2.types.ClusterStatus.State.Name(
+    return google.cloud.dataproc_v1beta2.types.ClusterStatus.State.Name(
         state_value)
 
 
 def _job_state_name(state_value):
-    return mrjob._vendor.dataproc_v1beta2.types.JobStatus.State.Name(
+    return google.cloud.dataproc_v1beta2.types.JobStatus.State.Name(
         state_value)
 
 
@@ -170,7 +174,7 @@ def _wait_for(msg, sleep_secs):
 
 
 def _cleanse_gcp_job_id(job_id):
-    return re.sub('[^a-zA-Z0-9_\-]', '-', job_id)
+    return re.sub(r'[^a-zA-Z0-9_\-]', '-', job_id)
 
 
 def _check_and_fix_fs_dir(gcs_uri):
@@ -226,6 +230,9 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         'task_instance_config',
     }
 
+    # no Spark support yet (see #1765)
+    _STEP_TYPES = {'jar', 'streaming'}
+
     def __init__(self, **kwargs):
         """:py:class:`~mrjob.dataproc.DataprocJobRunner` takes the same
         arguments as
@@ -254,6 +261,11 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
             raise DataprocException(
                 'Dataproc v1 expects core/task instance types to be identical')
 
+        # see #1820
+        if self._opts['image_id']:
+            log.warning('mrjob does not yet support custom machine images'
+                        ' on Dataproc')
+
         # load credentials and project ID
         self._credentials, auth_project_id = google.auth.default(
             scopes=[_FULL_SCOPE])  # needed for $GOOGLE_APPLICATION_CREDENTIALS
@@ -333,21 +345,20 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
             else:
                 self._opts['region'] = _DEFAULT_GCE_REGION
 
-    def _default_opts(self):
+    @classmethod
+    def _default_opts(cls):
         return combine_dicts(
-            super(DataprocJobRunner, self)._default_opts(),
+            super(DataprocJobRunner, cls)._default_opts(),
             dict(
                 bootstrap_python=True,
                 check_cluster_every=_DEFAULT_CHECK_CLUSTER_EVERY,
                 cleanup=['CLUSTER', 'JOB', 'LOCAL_TMP'],
                 cloud_fs_sync_secs=_DEFAULT_CLOUD_FS_SYNC_SECS,
-                gcloud_bin=['gcloud'],
                 image_version=_DEFAULT_IMAGE_VERSION,
                 instance_type=_DEFAULT_INSTANCE_TYPE,
                 master_instance_type=_DEFAULT_INSTANCE_TYPE,
                 num_core_instances=_DATAPROC_MIN_WORKERS,
                 num_task_instances=0,
-                sh_bin=['/bin/sh', '-ex'],
             )
         )
 
@@ -362,12 +373,12 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
 
     @property
     def cluster_client(self):
-        return mrjob._vendor.dataproc_v1beta2.ClusterControllerClient(
+        return google.cloud.dataproc_v1beta2.ClusterControllerClient(
             **self._client_create_kwargs())
 
     @property
     def job_client(self):
-        return mrjob._vendor.dataproc_v1beta2.JobControllerClient(
+        return google.cloud.dataproc_v1beta2.JobControllerClient(
             **self._client_create_kwargs())
 
     @property
@@ -395,26 +406,24 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         """:py:class:`~mrjob.fs.base.Filesystem` object for SSH, S3, GCS, and
         the local filesystem.
         """
-        if self._fs is not None:
-            return self._fs
+        if self._fs is None:
+            self._fs = CompositeFilesystem()
 
-        self._gcs_fs = GCSFilesystem(
-            credentials=self._credentials,
-            local_tmp_dir=self._get_local_tmp_dir(),
-            project_id=self._project_id,
-        )
+            location = self._opts['region'] or _zone_to_region(
+                self._opts['zone'])
+
+            self._fs.add_fs('gcs', GCSFilesystem(
+                credentials=self._credentials,
+                project_id=self._project_id,
+                part_size=self._upload_part_size(),
+                location=location,
+                object_ttl_days=_DEFAULT_CLOUD_TMP_DIR_OBJECT_TTL_DAYS,
+            ))
+
+            self._fs.add_fs('local', LocalFilesystem())
 
-        self._fs = CompositeFilesystem(self._gcs_fs, LocalFilesystem())
         return self._fs
 
-    def _fs_chunk_size(self):
-        """Chunk size for cloud storage Blob objects. Currently
-        only used for uploading."""
-        if self._opts['cloud_part_size_mb']:
-            return int(self._opts['cloud_part_size_mb'] * 1024 * 1024)
-        else:
-            return None
-
     def _get_tmpdir(self, given_tmpdir):
         """Helper for _fix_tmpdir"""
         if given_tmpdir:
@@ -428,8 +437,9 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         # determine region for bucket
         region = self._region()
 
-        for tmp_bucket_name in self.fs.get_all_bucket_names(prefix='mrjob-'):
-            tmp_bucket = self.fs.get_bucket(tmp_bucket_name)
+        for tmp_bucket_name in self.fs.gcs.get_all_bucket_names(
+                prefix='mrjob-'):
+            tmp_bucket = self.fs.gcs.get_bucket(tmp_bucket_name)
 
             # NOTE - GCP ambiguous Behavior - Bucket location is being
             # returned as UPPERCASE, ticket filed as of Apr 23, 2016 as docs
@@ -467,7 +477,8 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         self._create_setup_wrapper_scripts()
         self._add_bootstrap_files_for_upload()
         self._add_job_files_for_upload()
-        self._upload_local_files_to_fs()
+        self._upload_local_files()
+        self._wait_for_fs_sync()
 
     def _check_output_not_exists(self):
         """Verify the output path does not already exist. This avoids
@@ -480,16 +491,9 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _add_bootstrap_files_for_upload(self):
         """Add files needed by the bootstrap script to self._upload_mgr.
 
-        Tar up mrjob if bootstrap_mrjob is True.
-
         Create the master bootstrap script if necessary.
 
         """
-        # lazily create mrjob.zip
-        if self._bootstrap_mrjob():
-            self._create_mrjob_zip()
-            self._bootstrap_dir_mgr.add('file', self._mrjob_zip_path)
-
         # all other files needed by the script are already in
         # _bootstrap_dir_mgr
         for path in self._bootstrap_dir_mgr.paths():
@@ -504,9 +508,6 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _add_job_files_for_upload(self):
         """Add files needed for running the job (setup and input)
         to self._upload_mgr."""
-        for path in self._working_dir_mgr.paths():
-            self._upload_mgr.add(path)
-
         if self._opts['hadoop_streaming_jar']:
             self._upload_mgr.add(self._opts['hadoop_streaming_jar'])
 
@@ -514,47 +515,6 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
             if step.get('jar'):
                 self._upload_mgr.add(step['jar'])
 
-    def _upload_local_files_to_fs(self):
-        """Copy local files tracked by self._upload_mgr to FS."""
-        bucket_name, _ = parse_gcs_uri(self._job_tmpdir)
-        self._create_fs_tmp_bucket(bucket_name)
-
-        log.info('Copying non-input files into %s' % self._upload_mgr.prefix)
-
-        for path, gcs_uri in self._upload_mgr.path_to_uri().items():
-            log.debug('uploading %s -> %s' % (path, gcs_uri))
-
-            self.fs.put(path, gcs_uri, chunk_size=self._fs_chunk_size())
-
-        self._wait_for_fs_sync()
-
-    def _create_fs_tmp_bucket(self, bucket_name, location=None):
-        """Create a temp bucket if missing
-
-        Tie the temporary bucket to the same region as the GCE job and set a
-        28-day TTL
-        """
-        # Return early if our bucket already exists
-        try:
-            self.fs.get_bucket(bucket_name)
-            return
-        except google.api_core.exceptions.NotFound:
-            pass
-
-        log.info('creating FS bucket %r' % bucket_name)
-
-        location = location or self._opts['region'] or _zone_to_region(
-            self._opts['zone'])
-
-        # NOTE - By default, we create a bucket in the same GCE region as our
-        # job (tmp buckets ONLY)
-        # https://cloud.google.com/storage/docs/bucket-locations
-        self.fs.create_bucket(
-            bucket_name, location=location,
-            object_ttl_days=_DEFAULT_CLOUD_TMP_DIR_OBJECT_TTL_DAYS)
-
-        self._wait_for_fs_sync()
-
     ### Running the job ###
 
     def cleanup(self, mode=None):
@@ -639,8 +599,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         hadoop_job = {}
 
         hadoop_job['args'] = (
-            self._hadoop_generic_args_for_step(step_num) +
-            self._interpolate_input_and_output(step['args'], step_num))
+            self._interpolate_jar_step_args(step['args'], step_num))
 
         jar_uri = self._upload_mgr.uri(step['jar'])
 
@@ -663,8 +622,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _launch_cluster(self):
         """Create an empty cluster on Dataproc, and set self._cluster_id to
         its ID."""
-        bucket_name, _ = parse_gcs_uri(self._job_tmpdir)
-        self._create_fs_tmp_bucket(bucket_name)
+        self.fs.mkdir(self._job_tmpdir)
 
         # clusterName must be a match of
         # regex '(?:[a-z](?:[-a-z0-9]{0,53}[a-z0-9])?).'
@@ -803,8 +761,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
             if job_state == 'ERROR':
                 error = self._pick_error(log_interpretation, step_type)
                 if error:
-                    log.error('Probable cause of failure:\n\n%s\n\n' %
-                              _format_error(error))
+                    _log_probable_cause_of_failure(log, error)
 
             # we're done, will return at the end of this
             if job_state == 'DONE':
@@ -827,6 +784,9 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         Unlike with most runners, you may call this multiple times and it
         will continue to parse the step log incrementally, which is useful
         for getting job progress."""
+        # don't turn this off even if read_logs opt is false; it's
+        # the only way this runner can track job progress
+
         driver_output_uri = log_interpretation.get(
             'step', {}).get('driver_output_uri')
 
@@ -867,7 +827,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
                 state['pos'] = 0
                 state['log_uri'] = log_uri
 
-            log_blob = self.fs._get_blob(log_uri)
+            log_blob = self.fs.gcs._get_blob(log_uri)
 
             try:
                 new_data = log_blob.download_as_string(start=state['pos'])
@@ -899,6 +859,9 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _interpret_history_log(self, log_interpretation):
         """Does nothing. We can't get the history logs, and we don't need
         them."""
+        if not self._read_logs():
+            return
+
         log_interpretation.setdefault('history', {})
 
     # task
@@ -911,6 +874,9 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
                 partial or not log_interpretation['task'].get('partial')):
             return   # already interpreted
 
+        if not self._read_logs():
+            return
+
         step_interpretation = log_interpretation.get('step') or {}
 
         application_id = step_interpretation.get('application_id')
@@ -925,6 +891,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _task_log_interpretation(
             self, application_id, step_type, partial=True):
         """Helper for :py:meth:`_interpret_task_logs`"""
+        # not bothering with _read_logs() since this is a helper method
         result = {}
 
         for container_id in self._failed_task_container_ids(application_id):
@@ -1072,7 +1039,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _store_cluster_info(self):
         """Set self._image_version and self._hadoop_version."""
         if not self._cluster_id:
-            raise AssertionError('cluster has not yet been created')
+            raise ValueError('cluster has not yet been created')
 
         cluster = self._get_cluster(self._cluster_id)
         self._image_version = (
@@ -1290,7 +1257,7 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
     def _manifest_download_commands(self):
         return [
             # TODO: SSH in and figure out how to use gsutil or similar
-            #('gs://*', 'gsutil cp'),
+            # ('gs://*', 'gsutil cp'),
             ('*://*', 'hadoop fs -copyToLocal'),
         ]
 
@@ -1316,17 +1283,15 @@ class DataprocJobRunner(HadoopInTheCloudJobRunner, LogInterpretationMixin):
         return min(20.0, self._opts['check_cluster_every'])
 
     def _ssh_tunnel_args(self, bind_port):
-        if not self._opts['gcloud_bin']:
-            self._give_up_on_ssh_tunnel = True
-            return None
-
         if not self._cluster_id:
             return
 
+        gcloud_bin = self._opts['gcloud_bin'] or ['gcloud']
+
         cluster = self._get_cluster(self._cluster_id)
         zone = cluster.config.gce_cluster_config.zone_uri.split('/')[-1]
 
-        return self._opts['gcloud_bin'] + [
+        return gcloud_bin + [
             'compute', 'ssh',
             '--zone', zone,
             self._job_tracker_host(),
