@@ -1,4 +1,6 @@
 # Copyright 2012 Yelp
+# Copyright 2017 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,19 +20,21 @@ from mrjob.util import cmd_line
 
 class MRGrepJob(MRJob):
 
-    def configure_options(self):
-        super(MRGrepJob, self).configure_options()
+    def configure_args(self):
+        super(MRGrepJob, self).configure_args()
 
-        self.add_passthrough_option(
-            '-e', '--expression', type='str', default=None,
-            help=( 'Expression to search for. Required.'))
+        self.add_passthru_arg(
+            '-e', '--expression',
+            required=True,
+            help=('Expression to search for. Required.'))
 
     def mapper_cmd(self):
-        if self.options.expression is None:
-            raise ValueError("Must specify --expression")
-        return cmd_line(['grep', '-e', self.options.expression])
+        # grep will return exit status 1 if no matching lines are found
+        return cmd_line([
+            'sh', '-c',
+            'grep -e %s || RC=$?; if [ $RC -ne 1 ]; then (exit $RC); fi' % (
+                cmd_line([self.options.expression]))])
 
 
 if __name__ == '__main__':
-    MRGrepJob().run()
-
+    MRGrepJob.run()
