@@ -1,7 +1,7 @@
 # Copyright 2009-2012 Yelp and Contributors
 # Copyright 2013 David Marin
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -60,21 +60,31 @@ class HadoopFilesystem(Filesystem):
     def __init__(self, hadoop_bin=None):
         """Create a Hadoop filesystem
 
-        :param hadoop_bin: ``hadoop`` binary, as a list of args
+        :param hadoop_bin: ``hadoop`` binary, as a list of args. If set to
+                           ``None``, we'll auto-detect the Hadoop binary.
+                           If set to ``[]``, this FS will be disabled
+                           until you call :py:meth:`set_hadoop_bin`.
         """
         super(HadoopFilesystem, self).__init__()
         self._hadoop_bin = hadoop_bin
         self._hadoop_version = None  # cache for get_hadoop_version()
 
     def can_handle_path(self, path):
+        if not (self._hadoop_bin or self._hadoop_bin is None):
+            return False
+
         return is_uri(path)
 
     def get_hadoop_bin(self):
         """Return the hadoop binary, searching for it if need be."""
-        if not self._hadoop_bin:
+        if self._hadoop_bin is None:
             self._hadoop_bin = self._find_hadoop_bin()
         return self._hadoop_bin
 
+    def set_hadoop_bin(self, hadoop_bin):
+        """Manually set the hadoop binary, as a list of args."""
+        self._hadoop_bin = hadoop_bin
+
     def _find_hadoop_bin(self):
         """Look for the hadoop binary in any plausible place. If all
         else fails, return ``['hadoop']``.
@@ -251,14 +261,14 @@ class HadoopFilesystem(Filesystem):
             else:
                 yield hdfs_prefix + path
 
-    def _cat_file(self, filename):
+    def _cat_file(self, path):
         # stream from HDFS
-        cat_args = self.get_hadoop_bin() + ['fs', '-cat', filename]
+        cat_args = self.get_hadoop_bin() + ['fs', '-cat', path]
         log.debug('> %s' % cmd_line(cat_args))
 
         cat_proc = Popen(cat_args, stdout=PIPE, stderr=PIPE)
 
-        for chunk in decompress(cat_proc.stdout, filename):
+        for chunk in decompress(cat_proc.stdout, path):
             yield chunk
 
         # this does someties happen; see #1396
@@ -271,7 +281,7 @@ class HadoopFilesystem(Filesystem):
         returncode = cat_proc.wait()
 
         if returncode != 0:
-            raise IOError("Could not stream %s" % filename)
+            raise IOError("Could not stream %s" % path)
 
     def mkdir(self, path):
         version = self.get_hadoop_version()
@@ -303,12 +313,12 @@ class HadoopFilesystem(Filesystem):
         except CalledProcessError:
             raise IOError("Could not check path %s" % path_glob)
 
-    def _put(self, local_path, target):
-        # used by HadoopMRJobRunner._upload_to_hdfs()
+    def put(self, src, path):
+        # don't inadvertently support cp syntax
+        if path.endswith('/'):
+            raise ValueError('put() destination may not be a directory')
 
-        # not exposing this method because it's not part of the general FS
-        # interface. Probably want to add cp() at some point
-        self.invoke_hadoop(['fs', '-put', local_path, target])
+        self.invoke_hadoop(['fs', '-put', src, path])
 
     def rm(self, path_glob):
         if not is_uri(path_glob):
@@ -327,8 +337,8 @@ class HadoopFilesystem(Filesystem):
         except CalledProcessError:
             raise IOError("Could not rm %s" % path_glob)
 
-    def touchz(self, dest):
+    def touchz(self, path):
         try:
-            self.invoke_hadoop(['fs', '-touchz', dest])
+            self.invoke_hadoop(['fs', '-touchz', path])
         except CalledProcessError:
-            raise IOError("Could not touchz %s" % dest)
+            raise IOError("Could not touchz %s" % path)
