@@ -2,6 +2,8 @@
 # Copyright 2009-2016 Yelp and Contributors
 # Copyright 2017 Yelp
 # Copyright 2018 Yelp, Google, Inc., and Contributors
+# Copyright 2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,10 +31,12 @@ from logging import getLogger
 from mrjob.conf import combine_cmds
 from mrjob.conf import combine_dicts
 from mrjob.conf import combine_envs
+from mrjob.conf import combine_jobconfs
 from mrjob.conf import combine_lists
 from mrjob.conf import combine_paths
 from mrjob.conf import combine_path_lists
 from mrjob.parse import _parse_port_range_list
+from mrjob.util import shlex_split
 
 log = getLogger(__name__)
 
@@ -49,13 +53,6 @@ log = getLogger(__name__)
 #: * ``'LOGS'``: delete logs only
 #: * ``'NONE'``: delete nothing
 #: * ``'TMP'``: delete local, HDFS, and cloud storage temp files, but not logs
-#:
-#: .. versionchanged:: 0.5.0
-#:
-#:     - ``LOCAL_TMP`` used to be ``LOCAL_SCRATCH``
-#:     - ``HADOOP_TMP`` is new (and used to be covered by ``LOCAL_SCRATCH``)
-#:     - ``CLOUD_TMP`` used to be ``REMOTE_SCRATCH``
-#:
 CLEANUP_CHOICES = [
     'ALL',
     'CLOUD_TMP',
@@ -68,20 +65,19 @@ CLEANUP_CHOICES = [
     'TMP',
 ]
 
-
-# map from optparse types to Python types
-_OPTPARSE_TYPES = dict(
-    choice=str,  # optparse only allows strings as choices
-    complex=complex,
-    float=float,
-    int=int,
-    long=int,
-    string=str,
-)
-
 # use to identify malformed JSON
 _PROBABLY_JSON_RE = re.compile(r'^\s*[\{\[\"].*$')
 
+# names of runners
+_RUNNER_ALIASES = {
+    'dataproc',
+    'emr',
+    'hadoop',
+    'inline',
+    'local',
+    'spark',
+}
+
 
 ### custom actions ###
 
@@ -96,7 +92,7 @@ def _default_to(namespace, dest, value):
 
 class _KeyValueAction(Action):
     """action for KEY=VALUE pairs"""
-    # used for --cmdenv, --emr-api-param, and more
+    # used for --cmdenv, --jobconf, and more
 
     def __call__(self, parser, namespace, value, option_string=None):
         try:
@@ -149,6 +145,30 @@ class _CommaSeparatedListAction(Action):
         setattr(namespace, self.dest, items)
 
 
+class _AppendCommaSeparatedItemsAction(Action):
+    """action to parse a comma-separated list and append
+    each of them to an existing list.
+
+    This eliminates whitespace
+    """
+    def __call__(self, parser, namespace, value, option_string=None):
+        _default_to(namespace, self.dest, [])
+
+        items = [s.strip() for s in value.split(',') if s]
+
+        getattr(namespace, self.dest).extend(items)
+
+
+class _AppendArgsAction(Action):
+    """action to parse one or more arguments and append them to a list."""
+    def __call__(self, parser, namespace, value, option_string=None):
+        _default_to(namespace, self.dest, [])
+
+        args = shlex_split(value)
+
+        getattr(namespace, self.dest).extend(args)
+
+
 class _AppendJSONAction(Action):
     """action to parse JSON and append it to a list."""
     def __call__(self, parser, namespace, value, option_string=None):
@@ -247,14 +267,6 @@ _STEP_OPTS = dict(
             help='run Spark code',
         ),
     ),
-    show_steps=(
-        ['--steps'],
-        dict(
-            action='store_true',
-            help=('print the mappers, combiners, and reducers that this job'
-                  ' defines'),
-        ),
-    ),
     step_num=(
         ['--step-num'],
         dict(
@@ -265,6 +277,9 @@ _STEP_OPTS = dict(
     ),
 )
 
+# don't show these unless someone types --help -v --deprecated
+_DEPRECATED_STEP_OPTS = set()  # none at the moment
+
 # don't show these unless someone types --help --deprecated
 _DEPRECATED_NON_RUNNER_OPTS = {'deprecated'}
 
@@ -282,16 +297,30 @@ _DEPRECATED_NON_RUNNER_OPTS = {'deprecated'}
 # switches: list of switches to add to ArgumentParser for this option. Items
 #   have the format (['--switch-names', ...], dict(**kwargs)), where kwargs
 #   can be:
-#     action: action to pass to add_argument() (e.g. 'store_true')
-#     deprecated_aliases: list of old '--switch-names' slated for removal
-#     help: help string to pass to add_argument()
-#     type: option type for add_argument() to enforce (e.g. float).
+#     action -- action to pass to add_argument() (e.g. 'store_true')
+#     deprecated -- if True, this switch is deprecated and slated for removal
+#     deprecated_aliases -- list of old '--switch-names' slated for removal
+#     help -- help string to pass to add_argument()
+#     type -- option type for add_argument() to enforce (e.g. float).
 #   You can't set the ArgumentParser's default; we use [] if *action* is
 #   'append' and None otherwise.
 #
 # the list of which options apply to which runner is in the runner class
 # itself (e.g. EMRJobRunner.OPT_NAMES)
 _RUNNER_OPTS = dict(
+    add_steps_in_batch=dict(
+        switches=[
+            (['--add-steps-in-batch'], dict(
+                action='store_true',
+                help='For multi-step jobs, submit all steps at once',
+            )),
+            (['--no-add-steps-in-batch'], dict(
+                action='store_false',
+                help=('For multi-step jobs, submit steps successively after'
+                      ' the previous one completes'),
+            )),
+        ]
+    ),
     additional_emr_info=dict(
         cloud_role='launch',
         switches=[
@@ -304,10 +333,11 @@ _RUNNER_OPTS = dict(
         cloud_role='launch',
         combiner=combine_lists,
         switches=[
-            (['--application'], dict(
-                action='append',
-                help=('Additional applications to run on 4.x AMIs (e.g.'
-                      ' Ganglia, Mahout, Spark)'),
+            (['--applications', '--application'], dict(
+                action=_AppendCommaSeparatedItemsAction,
+                help=('Additional applications to run on 4.x and 5.x'
+                      ' AMIs, separated by commas (e.g.'
+                      ' "Ganglia,Spark")'),
             )),
         ],
     ),
@@ -467,8 +497,8 @@ _RUNNER_OPTS = dict(
         ],
     ),
     cloud_part_size_mb=dict(
-        deprecated_aliases=['cloud_upload_part_size'],
         cloud_role='launch',
+        deprecated_aliases=['cloud_upload_part_size'],
         switches=[
             (['--cloud-part-size-mb'], dict(
                 deprecated_aliases=['--cloud-upload-part-size'],
@@ -487,6 +517,7 @@ _RUNNER_OPTS = dict(
         ],
     ),
     cluster_properties=dict(
+        cloud_role='launch',
         combiner=combine_dicts,
         switches=[
             (['--cluster-property'], dict(
@@ -506,8 +537,8 @@ _RUNNER_OPTS = dict(
             (['--cmdenv'], dict(
                 action=_KeyValueAction,
                 help=('Set an environment variable for your job inside Hadoop '
-                      'streaming. Must take the form KEY=VALUE. You can use'
-                      ' --cmdenv multiple times.'),
+                      'streaming/Spark. Must take the form KEY=VALUE.'
+                      ' You can use --cmdenv multiple times.'),
             )),
         ],
     ),
@@ -544,6 +575,56 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    docker_client_config=dict(
+        switches=[
+            (['--docker-client-config'], dict(
+                help='Patch of docker client config, used connect to ECR',
+            )),
+        ],
+    ),
+    docker_image=dict(
+        switches=[
+            (['--docker-image'], dict(
+                help='ID of Docker image to run tasks inside',
+            )),
+            (['--no-docker'], dict(
+                action='store_const',
+                const='',
+                help="Don't run tasks inside Docker",
+            )),
+        ],
+    ),
+    docker_mounts=dict(
+        combiner=combine_lists,
+        switches=[
+            (['--docker-mount'], dict(
+                action='append',
+                help=('Volume to mount into docker, e.g. '
+                      ' "/etc/passwd:/etc/passwd:ro". May be used multiple'
+                      ' times'),
+            )),
+        ]
+    ),
+    ebs_root_volume_gb=dict(
+        cloud_role='launch',
+        switches=[
+            (['--ebs-root-volume-gb'], dict(
+                help=('Size of root EBS volume, in GiB. Must be an integer.'
+                      'Set to 0 to use the default'),
+                type=int,
+            )),
+        ],
+    ),
+    ec2_endpoint=dict(
+        cloud_role='connect',
+        switches=[
+            (['--ec2-endpoint'], dict(
+                help=('Force mrjob to connect to EC2 on this endpoint'
+                      ' (e.g. ec2.us-west-1.amazonaws.com).'
+                      ' Default is to infer this from region.'),
+            )),
+        ],
+    ),
     ec2_key_pair=dict(
         cloud_role='launch',
         switches=[
@@ -561,6 +642,7 @@ _RUNNER_OPTS = dict(
         ],
     ),
     emr_action_on_failure=dict(
+        cloud_role='launch',
         switches=[
             (['--emr-action-on-failure'], dict(
                 help=('Action to take when a step fails'
@@ -568,18 +650,6 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
-    emr_api_params=dict(
-        cloud_role='launch',
-        deprecated=True,
-        switches=[
-            (['--emr-api-param'], dict(
-                help=('deprecated. Use --extra-cluster-param instead'),
-            )),
-            (['--no-emr-api-param'], dict(
-                help=('deprecated. Use --extra-cluster-param instead'),
-            )),
-        ],
-    ),
     emr_configurations=dict(
         cloud_role='launch',
         combiner=combine_lists,
@@ -603,6 +673,22 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    emulate_map_input_file=dict(
+        switches=[
+            (['--emulate-map-input-file'], dict(
+                action='store_true',
+                help=("In the first mapper, set $mapreduce_map_input_file to"
+                      " the input file path, like Hadoop would, to support"
+                      " jobs that use"
+                      " jobconf_from_env('mapreduce.map.input.file')."
+                      " Ignored if job sets HADOOP_INPUT_FORMAT."),
+            )),
+            (['--no-emulate-map-input-file'], dict(
+                action='store_false',
+                help=("Disables setting $mapreduce_map_input_file"),
+            )),
+        ],
+    ),
     enable_emr_debugging=dict(
         cloud_role='launch',
         switches=[
@@ -637,6 +723,14 @@ _RUNNER_OPTS = dict(
             (['--gcloud-bin'], dict(help='path to gcloud binary')),
         ],
     ),
+    gcs_region=dict(
+        cloud_role='connect',
+        switches=[
+            (['--gcs-region'], dict(
+                help='region to create Google Cloud Storage buckets in',
+            )),
+        ],
+    ),
     hadoop_bin=dict(
         combiner=combine_cmds,
         switches=[
@@ -646,12 +740,10 @@ _RUNNER_OPTS = dict(
     hadoop_extra_args=dict(
         combiner=combine_lists,
         switches=[
-            (['--hadoop-arg'], dict(
-                action='append',
-                help=('Argument of any type to pass to hadoop '
-                      'streaming. Use an equals sign to avoid confusing the'
-                      ' parser (e.g. --hadoop-arg=-verbose).'
-                      ' You can use --hadoop-arg multiple times.'),
+            (['--hadoop-args'], dict(
+                action=_AppendArgsAction,
+                help=('One or more arguments to pass to the hadoop binary.'
+                      ' (e.g. --hadoop-args="-fs file:///").'),
             )),
         ],
     ),
@@ -717,11 +809,19 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    image_id=dict(
+        cloud_role='launch',
+        switches=[
+            (['--image-id'], dict(
+                help='ID of custom AWS machine image (AMI) to use',
+            )),
+        ],
+    ),
     image_version=dict(
         cloud_role='launch',
         switches=[
             (['--image-version'], dict(
-                help='EMR/Dataproc machine image to launch clusters with',
+                help='version of EMR/Dataproc machine image to run',
             )),
         ],
     ),
@@ -762,22 +862,13 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
-    interpreter=dict(
-        combiner=combine_cmds,
-        switches=[
-            (['--interpreter'], dict(
-                help='Non-python command to run your script, e.g. "ruby".',
-            )),
-        ],
-    ),
     jobconf=dict(
-        combiner=combine_dicts,
+        combiner=combine_jobconfs,
         switches=[
-            (['--jobconf'], dict(
+            (['-D', '--jobconf'], dict(
                 action=_KeyValueAction,
-                help=('-D arg to pass through to hadoop streaming; should'
-                      ' take the form KEY=VALUE. You can use --jobconf'
-                      ' multiple times.'),
+                help=('passed through to hadoop streaming as -D and to Spark'
+                      ' as --conf. Should take the form KEY=VALUE'),
             )),
         ],
     ),
@@ -792,17 +883,22 @@ _RUNNER_OPTS = dict(
     libjars=dict(
         combiner=combine_path_lists,
         switches=[
-            (['--libjar'], dict(
-                action='append',
-                help=('Path of a JAR to pass to Hadoop with -libjar. On EMR,'
-                      ' this can also be a URI; use file:/// to reference JARs'
-                      ' already on the EMR cluster'),
+            (['--libjars'], dict(
+                action=_AppendCommaSeparatedItemsAction,
+                help=('Paths of JARs to pass to Hadoop with -libjars,'
+                      ' separated by commas. On EMR,'
+                      ' these can also be URIs; use file:/// to'
+                      ' reference JARs already on the EMR cluster.')
             )),
         ],
     ),
     local_tmp_dir=dict(
         combiner=combine_paths,
-        # no switches, use $TMPDIR etc.
+        switches=[
+            (['--local-tmp-dir'], dict(
+                help='temp directory on local filesystem',
+            )),
+        ],
     ),
     master_instance_bid_price=dict(
         cloud_role='launch',
@@ -836,6 +932,26 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    max_clusters_in_pool=dict(
+        switches=[
+            (['--max-clusters-in-pool'], dict(
+                help=("If attempting to join a pooled cluster, don't bail"
+                      " out and create a new one if there are at least"
+                      " this many clusters already in the pool."),
+                type=int,
+            )),
+        ],
+    ),
+    max_concurrent_steps=dict(
+        cloud_role='launch',
+        switches=[
+            (['--max-concurrent-steps'], dict(
+                help=("Maximum number of steps that may run on the cluster"
+                      " at one time. Default is 1"),
+                type=int,
+            )),
+        ],
+    ),
     max_mins_idle=dict(
         cloud_role='launch',
         switches=[
@@ -847,27 +963,38 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
-    max_hours_idle=dict(
-        cloud_role='launch',
-        deprecated=True,
+    # Spark runner only, only passed in on the command line (see #2040)
+    max_output_files=dict(
         switches=[
-            (['--max-hours-idle'], dict(
-                help='Please use --max-mins-idle instead',
-                type=float,
+            (['--max-output-files'], dict(
+                help=('Maximum number of output files when running a'
+                      ' streaming job on Spark; just runs rdd.coalesce()'
+                      ' before outputting files'),
+                type=int,
             )),
         ],
     ),
-    mins_to_end_of_hour=dict(
-        cloud_role='launch',
-        deprecated=True,
+    # min_available_* options aren't in the "launch" group because they
+    # are only used when joining a cluster, not creating one
+    min_available_mb=dict(
         switches=[
-            (['--mins-to-end-of-hour'], dict(
-                help=("If --max-mins-idle is set, control how close to the"
-                      " end of an hour the cluster can automatically"
-                      " terminate itself (default is 5 minutes)"),
+            (['--min-available-mb'], dict(
+                help=('When attempting to join a pooled cluster, check'
+                      ' the YARN resource manager to ensure at least'
+                      ' this much memory is available.'),
+                type=int,
+            )),
+        ]
+    ),
+    min_available_virtual_cores=dict(
+        switches=[
+            (['--min-available-virtual-cores'], dict(
+                help=('When attempting to join a pooled cluster, check'
+                      ' the YARN resource manager to ensure at least'
+                      ' this much CPU capacity is available.'),
                 type=float,
             )),
-        ],
+        ]
     ),
     network=dict(
         cloud_role='launch',
@@ -927,6 +1054,17 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    pool_jitter_seconds=dict(
+        switches=[
+            (['--pool-jitter-seconds'], dict(
+                help=('If --max-pools-in-cluster is set, before launching a'
+                      ' cluster, wait a random amount of time between 0 and'
+                      ' this many seconds and then double-check the number'
+                      ' of clusters in the pool before launching'),
+                type=int,
+            )),
+        ],
+    ),
     pool_name=dict(
         cloud_role='launch',
         switches=[
@@ -935,6 +1073,16 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    pool_timeout_minutes=dict(
+        switches=[
+            (['--pool-timeout-minutes'], dict(
+                help=("If pooling can't join or create a cluster within this"
+                      " many minutes, raise an exception. (0 means don't"
+                      " timeout"),
+                type=int,
+            )),
+        ],
+    ),
     pool_wait_minutes=dict(
         switches=[
             (['--pool-wait-minutes'], dict(
@@ -946,20 +1094,23 @@ _RUNNER_OPTS = dict(
         ],
     ),
     project_id=dict(
+        cloud_role='connect',
         deprecated_aliases=['gcp_project'],
         switches=[
             (['--project-id'], dict(
                 deprecated_aliases=['--gcp-project'],
-                help='Project to run Dataproc jobs in'
+                help=('Project to use when connecting to Google Cloud Services'
+                      ' and to run Cloud Dataproc jobs in')
             )),
         ],
     ),
     py_files=dict(
         combiner=combine_path_lists,
         switches=[
-            (['--py-file'], dict(
-                action='append',
-                help='.zip or .egg file to add to PYTHONPATH'
+            (['--py-files'], dict(
+                action=_AppendCommaSeparatedItemsAction,
+                help=('.zip or .egg files to add to PYTHONPATH,'
+                      ' separated by commas'),
             )),
         ],
     ),
@@ -967,9 +1118,21 @@ _RUNNER_OPTS = dict(
         combiner=combine_cmds,
         switches=[
             (['--python-bin'], dict(
-                help=('Alternate python command for Python mappers/reducers.'
-                      ' You can include arguments, e.g. --python-bin "python'
-                      ' -v"'),
+                help=('Alternate python command. You can include arguments,'
+                      ' e.g. --python-bin "python -v"'),
+            )),
+        ],
+    ),
+    read_logs=dict(
+        switches=[
+            (['--read-logs'], dict(
+                action='store_true',
+                help=('Parse logs generated by the job to get counters and'
+                      ' cause of error (the default).')
+            )),
+            (['--no-read-logs'], dict(
+                action='store_false',
+                help="Don't list or read logs generated by the job."
             )),
         ],
     ),
@@ -1001,6 +1164,14 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    s3_region=dict(
+        cloud_role='connect',
+        switches=[
+            (['--s3-region'], dict(
+                help='AWS region to create s3 buckets in',
+            )),
+        ],
+    ),
     service_account=dict(
         cloud_role='launch',
         switches=[
@@ -1046,6 +1217,20 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    skip_internal_protocol=dict(
+        switches=[
+            (['--skip-internal-protocol'], dict(
+                action='store_true',
+                help=("Don't use the job's internal protocol to communicate"
+                      " between tasks internal to the job, instead relying"
+                      " on Spark to encode and decode raw data structures.")
+            )),
+            (['--no-skip-internal-protocol'], dict(
+                action='store_false',
+                help='Use internal protocols as usual',
+            )),
+        ],
+    ),
     sort_bin=dict(
         combiner=combine_cmds,
         switches=[
@@ -1058,20 +1243,26 @@ _RUNNER_OPTS = dict(
     spark_args=dict(
         combiner=combine_lists,
         switches=[
-            (['--spark-arg'], dict(
-                action='append',
-                help=('Argument of any type to pass to spark-submit.'
-                      ' Use an equals sign to avoid confusing the parser'
-                      ' (e.g. --spark-arg=--verbose).'
-                      ' You can use --spark-arg multiple times.'),
+            (['--spark-args'], dict(
+                action=_AppendArgsAction,
+                help=('One or more arguments to pass to spark-submit'
+                      ' (e.g. --spark-args="--properties-file my.conf").'),
             )),
         ],
     ),
+    spark_deploy_mode=dict(
+        switches=[
+            (['--spark-deploy-mode'], dict(
+                help=('--deploy-mode argument to spark-submit (e.g.'
+                      ' "cluster". Default is "client"'),
+            )),
+        ]
+    ),
     spark_master=dict(
         switches=[
             (['--spark-master'], dict(
                 help=('--master argument to spark-submit (e.g. '
-                      'spark://host:port, local. Default is yarn'),
+                      'spark://host:port, local). Default is "yarn"'),
             )),
         ],
     ),
@@ -1083,6 +1274,25 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
+    spark_tmp_dir=dict(
+        cloud_role='launch',
+        combiner=combine_paths,
+        switches=[
+            (['--spark-tmp-dir'], dict(
+                help=('optional URI visible to Spark executors to use as our'
+                      ' temp directory.'),
+            )),
+        ],
+    ),
+    ssh_add_bin=dict(
+        combiner=combine_cmds,
+        switches=[
+            (['--ssh-add-bin'], dict(
+                help=("Name/path of ssh-add binary. Arguments are allowed"
+                      " (e.g. --ssh-bin 'ssh-add -v')"),
+            )),
+        ],
+    ),
     ssh_bin=dict(
         combiner=combine_cmds,
         switches=[
@@ -1131,25 +1341,6 @@ _RUNNER_OPTS = dict(
             )),
         ],
     ),
-    steps_interpreter=dict(
-        combiner=combine_cmds,
-        switches=[
-            (['--steps-interpreter'], dict(
-                help=("Non-Python command to use to query the job about its"
-                      " steps, if different from --interpreter."),
-            )),
-        ],
-    ),
-    steps_python_bin=dict(
-        combiner=combine_cmds,
-        switches=[
-            (['--steps-python-bin'], dict(
-                help=('Name/path of alternate python command to use to'
-                      ' query the job about its steps, if different from the'
-                      ' current Python interpreter.'),
-            )),
-        ],
-    ),
     subnet=dict(
         cloud_role='launch',
         switches=[
@@ -1223,47 +1414,36 @@ _RUNNER_OPTS = dict(
     upload_archives=dict(
         combiner=combine_path_lists,
         switches=[
-            (['--archive'], dict(
-                action='append',
-                help=('Unpack archive in the working directory of this script.'
-                      ' You can use --archive multiple times.'),
+            (['--archives'], dict(
+                action=_AppendCommaSeparatedItemsAction,
+                help=('Archives to unpack in the working directory of the'
+                      ' script, separated by commas. Use "#" to assign a'
+                      ' different name to each directory (e.g. '
+                      '"foo-libs.zip#lib,bar.tar.gz#bar")'),
             )),
         ],
     ),
     upload_dirs=dict(
         combiner=combine_path_lists,
         switches=[
-            (['--dir'], dict(
-                action='append',
-                help=('Tarball the given directory and unpack the resulting'
-                      ' archive in the working directory of this script.'
-                      ' You can use --dir multiple times'),
+            (['--dirs'], dict(
+                action=_AppendCommaSeparatedItemsAction,
+                help=('Directories to tarball and unpack in the working'
+                      ' directory of the script, separated by commas. Append'
+                      '#<name> to each directory to assign a different name'
+                      ' (e.g. "foo#lib,bar#local-bar")'),
             )),
         ],
     ),
     upload_files=dict(
         combiner=combine_path_lists,
         switches=[
-            (['--file'], dict(
-                action='append',
-                help=('Copy file to the working directory of this script. You'
-                      ' can use --file multiple times.'),
-            )),
-        ],
-    ),
-    visible_to_all_users=dict(
-        cloud_role='launch',
-        deprecated=True,
-        switches=[
-            (['--visible-to-all-users'], dict(
-                action='store_true',
-                help=('Make your cluster is visible to all IAM users on the'
-                      ' same AWS account (the default)'),
-            )),
-            (['--no-visible-to-all-users'], dict(
-                action='store_false',
-                help=('Hide your cluster from other IAM users on the same AWS'
-                      ' account'),
+            (['--files'], dict(
+                action=_AppendCommaSeparatedItemsAction,
+                help=('Files to copy to the working directory of the script,'
+                      ' separated by commas. Use "#"'
+                      ' to assign a different name to each file (e.g. '
+                      '"foo.db#bar.db")'),
             )),
         ],
     ),
@@ -1309,7 +1489,8 @@ def _filter_by_role(opt_names, *cloud_roles):
     }
 
 
-def _add_runner_args(parser, opt_names=None, include_deprecated=True):
+def _add_runner_args(parser, opt_names=None, include_deprecated=True,
+                     customize_switches=None, suppress_switches=None):
     """add switches for the given runner opts to the given
     ArgumentParser, alphabetically by destination. If *opt_names* is
     None, include all runner opts."""
@@ -1318,11 +1499,22 @@ def _add_runner_args(parser, opt_names=None, include_deprecated=True):
 
     for opt_name in sorted(opt_names):
         _add_runner_args_for_opt(
-            parser, opt_name, include_deprecated=include_deprecated)
+            parser, opt_name,
+            include_deprecated=include_deprecated,
+            customize_switches=customize_switches,
+            suppress_switches=suppress_switches
+        )
 
 
-def _add_runner_args_for_opt(parser, opt_name, include_deprecated=True):
+def _add_runner_args_for_opt(parser, opt_name, include_deprecated=True,
+                             customize_switches=None, suppress_switches=None):
     """Add switches for a single option (*opt_name*) to the given parser."""
+    if customize_switches is None:
+        customize_switches = {}
+
+    if suppress_switches is None:
+        suppress_switches = set()
+
     conf = _RUNNER_OPTS[opt_name]
 
     if conf.get('deprecated') and not include_deprecated:
@@ -1330,22 +1522,34 @@ def _add_runner_args_for_opt(parser, opt_name, include_deprecated=True):
 
     switches = conf.get('switches') or []
 
+    def suppressed(switches):
+        return any(sw in suppress_switches for sw in switches)
+
     for args, kwargs in switches:
         kwargs = dict(kwargs)
 
+        # allow customization
+        for switch in args:
+            if switch in customize_switches:
+                kwargs.update(customize_switches[switch])
+
         deprecated_aliases = kwargs.pop('deprecated_aliases', None)
+        deprecated = kwargs.pop('deprecated', False)
 
-        kwargs['dest'] = opt_name
+        # add this switch
+        if (include_deprecated or not deprecated) and not suppressed(args):
+            kwargs['dest'] = opt_name
 
-        if kwargs.get('action') == 'append':
-            kwargs['default'] = []
-        else:
-            kwargs['default'] = None
+            if kwargs.get('action') == 'append':
+                kwargs['default'] = []
+            else:
+                kwargs['default'] = None
 
-        parser.add_argument(*args, **kwargs)
+            parser.add_argument(*args, **kwargs)
 
         # add a switch for deprecated aliases
-        if deprecated_aliases and include_deprecated:
+        if (deprecated_aliases and include_deprecated and
+                not suppressed(deprecated_aliases)):
             help = 'Deprecated alias%s for %s' % (
                 ('es' if len(deprecated_aliases) > 1 else ''),
                 args[-1])
@@ -1378,7 +1582,7 @@ def _add_basic_args(parser):
         action='store_true', help='print more messages to stderr')
 
 
-def _add_job_args(parser, include_deprecated=True):
+def _add_job_args(parser, include_deprecated=True, include_steps=True):
 
     parser.add_argument(
         '--cat-output', dest='cat_output',
@@ -1403,10 +1607,10 @@ def _add_job_args(parser, include_deprecated=True):
         'and must be empty')
 
     parser.add_argument(
-        '-r', '--runner', dest='runner', default=None,
-        choices=('local', 'hadoop', 'emr', 'inline', 'dataproc'),
-        help=('Where to run the job; one of dataproc, emr, hadoop, inline,'
-              ' or local'))
+        '-r', '--runner', dest='runner',
+        choices=sorted(_RUNNER_ALIASES),
+        help=('Where to run the job; one of: %s' % ', '.join(
+            sorted(_RUNNER_ALIASES))))
 
     parser.add_argument(
         '--step-output-dir', dest='step_output_dir', default=None,
@@ -1424,9 +1628,11 @@ def _add_job_args(parser, include_deprecated=True):
         help='show this message and exit')
 
 
-def _add_step_args(parser):
+def _add_step_args(parser, include_deprecated=False):
     """Add switches that determine what part of the job a MRJob runs."""
     for dest, (args, kwargs) in _STEP_OPTS.items():
+        if dest in _DEPRECATED_STEP_OPTS and not include_deprecated:
+            continue
         kwargs = dict(dest=dest, **kwargs)
         parser.add_argument(*args, **kwargs)
 
@@ -1442,15 +1648,8 @@ def _print_help_for_runner(opt_names, include_deprecated=False):
     help_parser.print_help()
 
 
-def _print_help_for_steps():
-    help_parser = ArgumentParser(usage=SUPPRESS, add_help=False)
-
-    _add_step_args(help_parser)
-
-    help_parser.print_help()
-
-
-def _print_basic_help(option_parser, usage, include_deprecated=False):
+def _print_basic_help(option_parser, usage, include_deprecated=False,
+                      include_steps=False):
     """Print all help for the parser. Unlike similar functions, this needs a
     parser so that it can include custom options added by a
     :py:class:`~mrjob.job.MRJob`.
@@ -1458,7 +1657,8 @@ def _print_basic_help(option_parser, usage, include_deprecated=False):
     help_parser = ArgumentParser(usage=usage, add_help=False)
 
     _add_basic_args(help_parser)
-    _add_job_args(help_parser, include_deprecated=include_deprecated)
+    _add_job_args(help_parser, include_deprecated=include_deprecated,
+                  include_steps=include_steps)
 
     basic_dests = {action.dest for action in help_parser._actions}
 
@@ -1478,8 +1678,8 @@ def _print_basic_help(option_parser, usage, include_deprecated=False):
         if action.dest in _RUNNER_OPTS:
             continue
 
-        # this excludes options that are show with --help --steps
-        if action.dest in _STEP_OPTS:
+        # don't include steps if *include_steps* isn't set
+        if action.dest in _STEP_OPTS and not include_steps:
             continue
 
         # this excludes the ARGS option, which is already covered by usage
@@ -1494,9 +1694,10 @@ def _print_basic_help(option_parser, usage, include_deprecated=False):
     print()
     print('To see help for a specific runner, use --help -r <runner name>')
     print()
-    print('To see help for options that control what part of a job runs,'
-          ' use --help --steps')
-    print()
+    if not include_steps:
+        print('To include switches that control what part of a job runs,'
+              ' use --help -v')
+        print()
     if not include_deprecated:
         print('To include help for deprecated options, add --deprecated')
         print()
@@ -1537,41 +1738,6 @@ def _parse_raw_args(parser, args):
     return results
 
 
-def _optparse_kwargs_to_argparse(**kwargs):
-    """Translate old keyword args to OptionParser.add_option() so they can be
-    passed to ArgumentParser.add_argument().
-
-    The two methods take almost identical arguments, so this is mostly a
-    matter of filtering.
-    """
-    if any(k.startswith('callback') for k in kwargs):
-        raise ValueError(
-            'mrjob does not emulate callback arguments to add_option(); please'
-            ' use argparse actions instead.')
-
-    # translate type from string (optparse) to type (argparse)
-    if kwargs.get('type') is not None:
-        if kwargs['type'] not in _OPTPARSE_TYPES:
-            raise ValueError('invalid option type: %r' % kwargs['type'])
-        kwargs['type'] = _OPTPARSE_TYPES[kwargs['type']]
-
-    # opt_group was a mrjob-specific feature that we've abandoned
-    if 'opt_group' in kwargs:
-        log.warning(
-            'ignoring opt_group keyword arg (mrjob no longer supports'
-            ' opt groups')
-        kwargs.pop('opt_group')
-
-    # convert %default -> %(default)s
-    if kwargs.get('help'):
-        kwargs['help'] = kwargs['help'].replace('%default', '%(default)s')
-
-    # pretty much everything else is the same. if people want to pass argparse
-    # kwargs through the old optparse interface (e.g. *action* or *required*)
-    # more power to 'em.
-    return kwargs
-
-
 def _alphabetize_actions(arg_parser):
     """Alphabetize arg parser actions for the sake of nicer help printouts."""
     # based on https://stackoverflow.com/questions/12268602/sort-argparse-help-alphabetically  # noqa
