@@ -1,8 +1,8 @@
 # Copyright 2009-2012 Yelp
 # Copyright 2013 Steve Johnson and David Marin
 # Copyright 2014 Yelp and Contributors
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,9 +20,12 @@ import logging
 import re
 from functools import wraps
 from io import BytesIO
+from os.path import abspath
 
 from mrjob.py2 import ParseResult
+from mrjob.py2 import pathname2url
 from mrjob.py2 import to_unicode
+from mrjob.py2 import urljoin
 from mrjob.py2 import urlparse as urlparse_buggy
 
 log = logging.getLogger(__name__)
@@ -30,20 +33,9 @@ log = logging.getLogger(__name__)
 
 ### URI PARSING ###
 
-
-# Used to parse the real netloc out of a malformed path from early Python 2.6
-# urlparse()
-_NETLOC_RE = re.compile(r'//(.*?)((/.*?)?)$')
-
-
 def is_uri(uri):
-    """Return True if *uri* is a URI and contains ``://``
+    r"""Return True if *uri* is a URI and contains ``://``
     (we only care about URIs that can describe files)
-
-    .. versionchanged:: 0.5.7
-
-       used to recognize anything containing a colon as a URI
-       unless it was a Windows path (``C:\...``).
     """
     return '://' in uri and bool(urlparse(uri).scheme)
 
@@ -74,6 +66,15 @@ def parse_s3_uri(uri):
     return components.netloc, components.path[1:]
 
 
+def to_uri(path_or_uri):
+    """If *path_or_uri* is not a URI already, convert it to a ``file:///``
+    URI."""
+    if is_uri(path_or_uri):
+        return path_or_uri
+    else:
+        return urljoin('file:', pathname2url(abspath(path_or_uri)))
+
+
 @wraps(urlparse_buggy)
 def urlparse(urlstring, scheme='', allow_fragments=True, *args, **kwargs):
     """A wrapper for :py:func:`urlparse.urlparse` that splits the fragment
