@@ -1,6 +1,8 @@
 # Copyright 2009-2013 Yelp and Contributors
 # Copyright 2015-2017 Yelp
 # Copyright 2018 Yelp and Contributors
+# Copyright 2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +20,8 @@ import logging
 import os
 import shutil
 import stat
+import platform
+from copy import deepcopy
 from functools import partial
 from multiprocessing import cpu_count
 from os.path import dirname
@@ -33,10 +37,12 @@ from mrjob.compat import translate_jobconf
 from mrjob.compat import translate_jobconf_for_all_versions
 from mrjob.conf import combine_dicts
 from mrjob.conf import combine_local_envs
+from mrjob.fs.local import _from_file_uri
 from mrjob.logs.counters import _format_counters
 from mrjob.parse import parse_mr_job_stderr
 from mrjob.runner import MRJobRunner
 from mrjob.runner import _fix_env
+from mrjob.step import _is_spark_step_type
 from mrjob.util import unarchive
 
 log = logging.getLogger(__name__)
@@ -66,6 +72,8 @@ class SimMRJobRunner(MRJobRunner):
         'num_cores'
     }
 
+    _STEP_TYPES = {'spark', 'streaming'}
+
     def __init__(self, **kwargs):
         super(SimMRJobRunner, self).__init__(**kwargs)
 
@@ -125,10 +133,24 @@ class SimMRJobRunner(MRJobRunner):
         """Log why the job failed."""
         pass
 
+    def _run_step_on_spark(self, step, step_num):
+        """Run a Step on Spark. Override this in your subclass. You can
+        assume that setup wrapper scripts are created (if relevant)
+        and that self._counters has a dictionary for that step already"""
+        raise NotImplementedError
+
+    # other implementation
+
     def _run(self):
+        if not self._output_dir:
+            self._output_dir = join(self._get_local_tmp_dir(), 'output')
+
         if hasattr(self, '_create_setup_wrapper_scripts'):  # inline doesn't
             self._create_setup_wrapper_scripts()
 
+        # this does nothing in inline mode, since there's no _spark_master()
+        self._copy_files_to_wd_mirror()
+
         # run mapper, combiner, sort, reducer for each step
         for step_num, step in enumerate(self._get_steps()):
             log.info('Running step %d of %d...' % (
@@ -136,28 +158,41 @@ class SimMRJobRunner(MRJobRunner):
 
             self._counters.append({})
 
-            try:
-                self._create_dist_cache_dir(step_num)
-                self.fs.mkdir(self._output_dir_for_step(step_num))
+            self._run_step(step, step_num)
 
-                map_splits = self._split_mapper_input(
-                    self._input_paths_for_step(step_num), step_num)
+    def _run_step(self, step, step_num):
+        """Run an individual step. You can assume that setup wrapper scripts
+        are created and self._counters has a dictionary for that step already.
+        """
+        if _is_spark_step_type(step['type']):
+            self._run_step_on_spark(step, step_num)
+        else:
+            self._run_streaming_step(step, step_num)
 
-                self._run_mappers_and_combiners(step_num, map_splits)
+    def _run_streaming_step(self, step, step_num):
+        """Run a Hadoop streaming step on simulated Hadoop."""
+        try:
+            self._create_dist_cache_dir(step_num)
+            self.fs.mkdir(self._output_dir_for_step(step_num))
 
-                if 'reducer' in step:
-                    self._sort_reducer_input(step_num, len(map_splits))
-                    num_reducer_tasks = self._split_reducer_input(step_num)
+            map_splits = self._split_mapper_input(
+                self._input_paths_for_step(step_num), step_num)
 
-                    self._run_reducers(step_num, num_reducer_tasks)
+            self._run_mappers_and_combiners(step_num, map_splits)
 
-                self._log_counters(step_num)
+            if 'reducer' in step:
+                self._sort_reducer_input(step_num, len(map_splits))
+                num_reducer_tasks = self._split_reducer_input(step_num)
 
-            except Exception as ex:
-                self._log_counters(step_num)
-                self._log_cause_of_error(ex)
+                self._run_reducers(step_num, num_reducer_tasks)
 
-                raise
+            self._log_counters(step_num)
+
+        except Exception as ex:
+            self._log_counters(step_num)
+            self._log_cause_of_error(ex)
+
+            raise
 
     def _run_task_func(self, task_type, step_num, task_num, map_split=None):
         """Returns a no-args function that runs one mapper, reducer, or
@@ -192,6 +227,8 @@ class SimMRJobRunner(MRJobRunner):
 
     def _parse_task_counters(self, task_type, step_num):
         """Parse all stderr files from the given task (if any)."""
+        # don't disable if read_logs=False; parsing counters is
+        # internal to Hadoop, not something that happens in log files
         stderr_paths = self.fs.ls(self._task_stderr_paths_glob(
             task_type, step_num))
 
@@ -200,7 +237,7 @@ class SimMRJobRunner(MRJobRunner):
                 parse_mr_job_stderr(stderr, counters=self._counters[step_num])
 
     def counters(self):
-        return self._counters
+        return deepcopy(self._counters)
 
     def get_hadoop_version(self):
         return self._opts['hadoop_version']
@@ -265,7 +302,7 @@ class SimMRJobRunner(MRJobRunner):
         self.fs.mkdir(cache_dir)
 
         for name, path in self._working_dir_mgr.name_to_path('file').items():
-
+            path = _from_file_uri(path)  # might start with file://
             dest = self._path_in_dist_cache_dir(name, step_num)
             log.debug('copying %s -> %s' % (path, dest))
             shutil.copy(path, dest)
@@ -273,8 +310,9 @@ class SimMRJobRunner(MRJobRunner):
 
         for name, path in self._working_dir_mgr.name_to_path(
                 'archive').items():
-
+            path = _from_file_uri(path)  # might start with file://
             dest = self._path_in_dist_cache_dir(name, step_num)
+
             log.debug('unarchiving %s -> %s' % (path, dest))
             unarchive(path, dest)
             _chmod_u_rx(dest, recursive=True)
@@ -309,7 +347,7 @@ class SimMRJobRunner(MRJobRunner):
         return combine_local_envs(os.environ,
                                   to_env(user_jobconf),
                                   to_env(simulated_jobconf),
-                                  self._opts['cmdenv'])
+                                  self._cmdenv())
 
     def _simulate_jobconf_for_step(
             self, task_type, step_num, task_num, map_split=None):
@@ -346,11 +384,9 @@ class SimMRJobRunner(MRJobRunner):
                 join(working_dir, name) for name, path in named_paths)
 
         if map_split:
-            # mapreduce.map.input.file
-            # mapreduce.map.input.start
-            # mapreduce.map.input.length
-            for key, value in map_split.items():
-                j['mapreduce.map.input.' + key] = str(value)
+            j['mapreduce.map.input.file'] = 'file://' + map_split['file']
+            j['mapreduce.map.input.length'] = str(map_split['length'])
+            j['mapreduce.map.input.start'] = str(map_split['start'])
 
         # translate to correct version
 
@@ -365,13 +401,16 @@ class SimMRJobRunner(MRJobRunner):
             return {tk: v for k, v in j.items()
                     for tk in translate_jobconf_for_all_versions(k)}
 
+    def _num_cores(self):
+        return self._opts['num_cores'] or cpu_count()
+
     def _num_mappers(self, step_num):
         # TODO: look up mapred.job.maps (convert to int) in _jobconf_for_step()
-        return self._opts['num_cores'] or cpu_count()
+        return self._num_cores()
 
     def _num_reducers(self, step_num):
         # TODO: look up mapred.job.reduces in _jobconf_for_step()
-        return self._opts['num_cores'] or cpu_count()
+        return self._num_cores()
 
     def _split_mapper_input(self, input_paths, step_num):
         """Take one or more input paths (which may be compressed) and split
@@ -550,19 +589,20 @@ class SimMRJobRunner(MRJobRunner):
 
     def _input_paths_for_step(self, step_num):
         if step_num == 0:
-            return [path for input_path_glob in self._get_input_paths()
-                    for path in self.fs.ls(input_path_glob)]
+            return [
+                _from_file_uri(path)  # *path* could be a file:// URI
+                for input_path_glob in self._get_input_paths()
+                for path in self.fs.ls(input_path_glob)
+            ]
         else:
             return self.fs.ls(
                 join(self._output_dir_for_step(step_num - 1), 'part-*'))
 
     def _output_dir_for_step(self, step_num):
         if step_num == self._num_steps() - 1:
-            if not self._output_dir:
-                self._output_dir = join(self._get_local_tmp_dir(), 'output')
             return self._output_dir
         else:
-            return self._intermediate_output_uri(step_num, local=True)
+            return self._intermediate_output_dir(step_num, local=True)
 
     def _default_step_output_dir(self):
         return join(self._get_local_tmp_dir(), 'step-output')
@@ -645,7 +685,8 @@ def _chmod_u_rx(path, recursive=False):
             for file_name in file_names:
                 _chmod_u_rx(join(dir_name, file_name))
     else:
-        if hasattr(os, 'chmod'):  # only available on Unix, Windows
+        # only available on Unix, causes rmtree() to fail on Windows; see #1847
+        if hasattr(os, 'chmod') and platform.system() != "Windows":
             os.chmod(path, stat.S_IRUSR | stat.S_IXUSR)
 
 
