@@ -1,7 +1,7 @@
 # Copyright 2012 Yelp and Contributors
 # Copyright 2013 David Marin and Contributors
-# Copyright 2015-2017 Yelp
-# Copyright 2018 Yelp
+# Copyright 2015-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Representations of job steps, to use in your :py:class:`~mrjob.job.MRJob`'s
+:py:meth:`~mrjob.job.MRJob.steps` method.
+
+Because :py:class:`the runner <mrjob.runner.MRJobRunner>` just needs to know
+how to invoke your MRJob script, not how it works insternally, each step
+instance's ``description()`` method produces a simplified, JSON-able
+description of the step, to pass to the runner.
+"""
 import logging
 
 from mrjob.py2 import string_types
@@ -49,16 +57,21 @@ _SPARK_JAR_STEP_KWARGS = ['args', 'jar', 'main_class', 'spark_args']
 _SPARK_SCRIPT_STEP_KWARGS = ['args', 'script', 'spark_args']
 
 
-#: If passed as an argument to :py:class:`JarStep` or
-#: py:class:`SparkScriptStep`, it'll be replaced with the step's input path(s)
-#: (if there are multiple paths, they'll be joined with commas)
+#: If passed as an argument to :py:class:`JarStep`, :py:class:`SparkJarStep`,
+#: or :py:class:`SparkScriptStep`, it'll be replaced with the step's input
+#: path(s). If there are multiple paths, they'll be joined with commas.
 INPUT = '<input>'
 
-#: If this is passed as passed as an argument to :py:class:`JarStep` or
-#: py:class:`SparkScriptStep`, it'll be replaced
+#: If this is passed as an argument to :py:class:`JarStep`,
+#: :py:class:`SparkJarStep`, or :py:class:`SparkScriptStep`, it'll be replaced
 #: with the step's output path
 OUTPUT = '<output>'
 
+#: If this is passed as an argument to :py:class:`JarStep`,
+#: it'll be replaced with generic hadoop args (-D and -libjars)
+GENERIC_ARGS = '<generic args>'
+
+
 log = logging.getLogger(__name__)
 
 
@@ -84,7 +97,8 @@ class StepFailedException(Exception):
     _FIELDS = ('reason', 'step_num', 'num_steps', 'step_desc')
 
     def __init__(
-            self, reason=None, step_num=None, num_steps=None, step_desc=None):
+            self, reason=None, step_num=None, num_steps=None,
+            step_desc=None, last_step_num=None):
         """Initialize a reason for step failure.
 
         :param string reason: brief explanation of which step failed
@@ -92,6 +106,9 @@ class StepFailedException(Exception):
         :param int num_steps: number of steps in the job
         :param string step_desc: description of step (if we don't like the
                                  default "Step X of Y")
+        :param int last_step_num: if one of a range of steps failed, the
+                                  (0-indexed) last step in that range. If this
+                                  equals *step_num*, will be ignored.
 
         *reason* should not be several lines long; use ``log.error(...)``
         for that.
@@ -101,15 +118,38 @@ class StepFailedException(Exception):
         self.num_steps = num_steps
         self.step_desc = step_desc
 
+        # we only need this for streaming steps run by the Spark harness,
+        # so don't create noise
+        if last_step_num is None or last_step_num == step_num:
+            self.last_step_num = None
+        else:
+            self.last_step_num = last_step_num
+
     def __str__(self):
         """Human-readable version of the exception. Note that this 1-indexes
         *step_num*."""
-        return '%s failed%s' % (
-            (self.step_desc or 'Step%s%s' % (
-                '' if self.step_num is None else ' %d' % (self.step_num + 1),
-                '' if (self.step_num is None or self.num_steps is None) else (
-                    ' of %d' % self.num_steps))),
-            '' if self.reason is None else ': %s' % self.reason)
+        if self.step_desc:
+            step_desc = self.step_desc
+        else:
+            if self.step_num is not None:
+                # 1-index step numbers
+                if self.last_step_num is not None:
+                    step_name = 'Steps %d-%d' % (
+                        self.step_num + 1, self.last_step_num + 1)
+                else:
+                    step_name = 'Step %d' % (self.step_num + 1)
+
+                if self.num_steps:
+                    step_desc = '%s of %d' % (step_name, self.num_steps)
+                else:
+                    step_desc = step_name
+            else:
+                step_desc = 'Step'
+
+        if self.reason:
+            return '%s failed: %s' % (step_desc, self.reason)
+        else:
+            return '%s failed' % step_desc
 
     def __repr__(self):
         return '%s(%s)' % (
@@ -136,6 +176,34 @@ class MRStep(object):
 
     Also accepts `jobconf`, a dictionary with custom jobconf arguments to pass
     to hadoop.
+
+    A MRStep's description looks like::
+
+        {
+            'type': 'streaming',
+            'mapper': { ... },
+            'combiner': { ... },
+            'reducer': { ... },
+            'jobconf': { ... },  # dict of Hadoop configuration properties
+        }
+
+    At least one of ``mapper``, ``combiner`` and ``reducer`` need be included.
+    ``jobconf`` is completely optional.
+
+    ``mapper``, ``combiner``, and ``reducer`` are either handled by
+    the script containing your job definition, in which case they look like::
+
+        {
+            'type': 'script',
+            'pre_filter': 'grep -v bad', # optional cmd to filter input
+        }
+
+    or they simply run a command, which looks like::
+
+        {
+            'type': 'command',
+            'command': 'cut -f 1-2', # command to run, as a string
+        }
     """
     def __init__(self, **kwargs):
         # limit which keyword args can be specified
@@ -230,43 +298,7 @@ class MRStep(object):
     def render_reducer(self):
         return self._render_substep('reducer_cmd', 'reducer_pre_filter')
 
-    def description(self, step_num):
-        """Returns a dictionary representation of this step:
-
-        .. code-block:: js
-
-            {
-                'type': 'streaming',
-                'mapper': { ... },
-                'combiner': { ... },
-                'reducer': { ... },
-                'jobconf': dictionary of Hadoop configuration properties
-            }
-
-        ``jobconf`` is optional, and only one of ``mapper``, ``combiner``,
-        and ``reducer`` need be included.
-
-        ``mapper``, ``combiner``, and ``reducer`` are either handled by
-        the script containing your job definition:
-
-        .. code-block:: js
-
-           {
-               'type': 'script',
-               'pre_filter': (optional) cmd to pass input through, as a string
-           }
-
-        or they simply run a command:
-
-        .. code-block:: js
-
-            {
-                'type': 'command',
-                'command': command to run, as a string
-            }
-
-        See :ref:`steps-format` for examples.
-        """
+    def description(self, step_num=0):
         desc = {'type': 'streaming'}
         # Use a mapper if:
         #   - the user writes one
@@ -373,7 +405,7 @@ class _Step(object):
         else:
             return None
 
-    def description(self, other):
+    def description(self, step_num=0):
         """Return a dictionary representation of this step. See
         :ref:`steps-format` for examples."""
         result = dict(
@@ -405,6 +437,20 @@ class JarStep(_Step):
     *jar* can also be passed as a positional argument
 
     See :ref:`non-hadoop-streaming-jar-steps` for sample usage.
+
+    Sample description of a JarStep::
+
+        {
+            'type': 'jar',
+            'jar': 'binks.jar.jar',
+            'main_class': 'MyMainMan',  # optional
+            'args': ['argh', 'argh']  # optional
+            'jobconf': { ... }  # optional
+        }
+
+    To give your jar access to input files, an empty output directory,
+    configuration properties, and libjars managed by mrjob, you may include
+    :py:data:`INPUT`, :py:data:`OUTPUT`, and :py:data:`GENERIC_ARGS` in *args*.
     """
     _STEP_TYPE = 'jar'
 
@@ -424,6 +470,14 @@ class SparkStep(_Step):
     :param jobconf: (optional) A dictionary of Hadoop properties
     :param spark_args: (optional) an array of arguments to pass to spark-submit
                        (e.g. ``['--executor-memory', '2G']``).
+
+    Sample description of a SparkStep::
+
+        {
+            'type': 'spark',
+            'jobconf': { ... },  # optional
+            'spark_args': ['--executor-memory', '2G'],  # optional
+        }
     """
     _STEP_TYPE = 'spark'
 
@@ -453,6 +507,21 @@ class SparkJarStep(_Step):
                        (e.g. ``['--executor-memory', '2G']``).
 
     *jar* and *main_class* can also be passed as positional arguments
+
+    Sample description of a SparkJarStep::
+
+        {
+            'type': 'spark_jar',
+            'jar': 'binks.jar.jar',
+            'main_class': 'MyMainMan',  # optional
+            'args': ['argh', 'argh'],  # optional
+            'jobconf': { ... },  # optional
+            'spark_args': ['--executor-memory', '2G'],  # optional
+        }
+
+    To give your Spark JAR access to input files and an empty output directory
+    managed by mrjob, you may include :py:data:`INPUT` and :py:data:`OUTPUT`
+    in *args*.
     """
     _STEP_TYPE = 'spark_jar'
 
@@ -479,6 +548,20 @@ class SparkScriptStep(_Step):
                        (e.g. ``['--executor-memory', '2G']``).
 
     *script* can also be passed as a positional argument
+
+    Sample description of a ScriptStep::
+
+       {
+            'type': 'spark_script',
+            'script': 'my_spark_script.py',
+            'args': ['script_arg1', 'script_arg2'],
+            'jobconf': { ... },  # optional
+            'spark_args': ['--executor-memory', '2G'],  # optional
+        }
+
+    To give your Spark script access to input files and an empty output
+    directory managed by mrjob, you may include :py:data:`INPUT` and
+    :py:data:`OUTPUT` in *args*.
     """
     _STEP_TYPE = 'spark_script'
 
@@ -491,3 +574,8 @@ class SparkScriptStep(_Step):
 def _is_spark_step_type(step_type):
     """Does the given step type indicate that it uses Spark?"""
     return step_type.split('_')[0] == 'spark'
+
+
+def _is_pyspark_step_type(step_type):
+    """Does the given step type indicate that it uses Spark and Python?"""
+    return step_type in ('spark', 'spark_script')
