@@ -1,5 +1,6 @@
 # Copyright 2009-2010 Yelp
-# Copyright 2015-2017 Yelp
+# Copyright 2015-2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,6 +24,10 @@ Options::
   -c CONF_PATHS, --conf-path CONF_PATHS
                         Path to alternate mrjob.conf file to read from
   --no-conf             Don't load mrjob.conf even if it's available
+  --ec2-endpoint EC2_ENDPOINT
+                        Force mrjob to connect to EC2 on this endpoint (e.g.
+                        ec2.us-west-1.amazonaws.com). Default is to infer this
+                        from region.
   --emr-endpoint EMR_ENDPOINT
                         Force mrjob to connect to EMR on this endpoint (e.g.
                         us-west-1.elasticmapreduce.amazonaws.com). Default is
@@ -62,8 +67,7 @@ from mrjob.options import _add_basic_args
 from mrjob.options import _add_runner_args
 from mrjob.options import _alphabetize_actions
 from mrjob.options import _filter_by_role
-from mrjob.pool import _legacy_pool_hash_and_name
-from mrjob.pool import _pool_hash_and_name
+from mrjob.pool import _pool_name
 from mrjob.util import strip_microseconds
 
 # match an mrjob job key (used to uniquely identify the job)
@@ -100,7 +104,7 @@ def main(args=None):
 
 
 def _make_arg_parser():
-    usage = '%(prog)s [options]'
+    usage = '%(prog)s audit-emr-usage [options]'
     description = 'Print a giant report on EMR usage.'
 
     arg_parser = ArgumentParser(usage=usage, description=description)
@@ -129,10 +133,9 @@ def _runner_kwargs(options):
 
 
 def _clusters_to_stats(clusters, now=None):
-    """Aggregate statistics for several clusters into a dictionary.
+    r"""Aggregate statistics for several clusters into a dictionary.
 
-    :param clusters: a sequence of dicts with the keys ``bootstrap_actions``,
-                     ``cluster``, ``steps``.
+    :param clusters: a sequence of dicts with the keys ``cluster``, ``steps``.
     :param now: the current UTC time, as a :py:class:`datetime.datetime`.
                 Defaults to the current time.
 
@@ -337,10 +340,7 @@ def _cluster_to_basic_summary(cluster, now=None):
 
     bcs['num_steps'] = len(cluster['Steps'])
 
-    _, bcs['pool'] = _pool_hash_and_name(cluster)
-    if not bcs['pool']:
-        _, bcs['pool'] = _legacy_pool_hash_and_name(
-            cluster['BootstrapActions'])
+    bcs['pool'] = _pool_name(cluster)
 
     m = _JOB_KEY_RE.match(bcs['name'] or '')
     if m:
@@ -354,7 +354,7 @@ def _cluster_to_basic_summary(cluster, now=None):
 
 
 def _cluster_to_usage_data(cluster, basic_summary=None, now=None):
-    """Break billing/usage information for a cluster down by job.
+    r"""Break billing/usage information for a cluster down by job.
 
     :param cluster: a :py:mod:`boto3` cluster data structure
     :param basic_summary: a basic summary of the cluster, returned by
@@ -619,10 +619,6 @@ def _yield_clusters(max_days_ago=None, now=None, **runner_kwargs):
             'Steps', emr_client, 'list_steps',
             ClusterId=cluster_id, _delay=_DELAY))))
 
-        cluster['BootstrapActions'] = list(_boto3_paginate(
-            'BootstrapActions', emr_client, 'list_bootstrap_actions',
-            ClusterId=cluster_id, _delay=_DELAY))
-
         yield cluster
 
 
