@@ -1,7 +1,7 @@
 # Copyright 2010-2012 Yelp
 # Copyright 2013 David Marin and Steve Johnson
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -85,7 +85,7 @@ def _s3_cleanup(glob_path, time_old, dry_run=False, **runner_kwargs):
     log.info('Deleting all files in %s that are older than %s' %
              (glob_path, time_old))
 
-    for path, key in runner.fs._ls(glob_path):
+    for path, key in runner.fs.s3._ls(glob_path):
         age = _boto3_now() - key.last_modified
         if age > time_old:
             # Delete it
@@ -115,11 +115,12 @@ def _process_time(time):
 
 
 def _make_arg_parser():
+    usage = '%(prog)s s3-tmpwatch [options] TIME_UNTOUCHED URI [URI ...]'
     description = (
         'Delete all files at one or more URIs that are older than a'
         ' specified time.')
 
-    arg_parser = ArgumentParser(description=description)
+    arg_parser = ArgumentParser(usage=usage, description=description)
 
     arg_parser.add_argument(
         '-t', '--test', dest='test', default=False,
@@ -131,7 +132,7 @@ def _make_arg_parser():
         help='The time threshold for removing'
         ' files. A number with an optional'
         ' single-character suffix specifying the units: m for minutes, h for'
-        ' hours, d for days.  If no suffix is specified, time is in hours.')
+        ' hours, d for days. If no suffix is specified, time is in hours.')
 
     arg_parser.add_argument(
         dest='uris', nargs='+',
