@@ -2,8 +2,8 @@
 # Copyright 2009-2012 Yelp
 # Copyright 2013 Yelp and Contributors
 # Copyright 2014 Brett Gibson
-# Copyright 2015-2017 Yelp
-# Copyright 2018 Yelp
+# Copyright 2015-2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,19 +23,26 @@ Suggested usage: run this as a cron job with the ``-q`` option::
 
     */30 * * * * mrjob terminate-idle-clusters -q
 
+.. versionchanged:: 0.6.4
+
+   Skips termination-protected idle clusters, rather than crashing. (This was
+   also backported to mrjob v0.5.12.)
+
 Options::
 
   -c CONF_PATHS, --conf-path CONF_PATHS
                         Path to alternate mrjob.conf file to read from
   --no-conf             Don't load mrjob.conf even if it's available
   --dry-run             Don't actually kill idle jobs; just log that we would
+  --ec2-endpoint EC2_ENDPOINT
+                        Force mrjob to connect to EC2 on this endpoint (e.g.
+                        ec2.us-west-1.amazonaws.com). Default is to infer this
+                        from region.
   --emr-endpoint EMR_ENDPOINT
                         Force mrjob to connect to EMR on this endpoint (e.g.
                         us-west-1.elasticmapreduce.amazonaws.com). Default is
                         to infer this from region.
   -h, --help            show this help message and exit
-  --max-hours-idle MAX_HOURS_IDLE
-                        Please use --max-mins-idle instead.
   --max-mins-idle MAX_MINS_IDLE
                         Max number of minutes a cluster can go without
                         bootstrapping, running a step, or having a new step
@@ -44,18 +51,15 @@ Options::
                         this higher than the amount of time your jobs can take
                         to start instances and bootstrap.
   --max-mins-locked MAX_MINS_LOCKED
-                        Max number of minutes a cluster can be locked while
-                        idle.
-  --mins-to-end-of-hour MINS_TO_END_OF_HOUR
-                        Deprecated, does nothing.
+                        Deprecated, does nothing
   --pool-name POOL_NAME
                         Only terminate clusters in the given named pool.
   --pooled-only         Only terminate pooled clusters
   -q, --quiet           Don't print anything to stderr
   --region REGION       GCE/AWS region to run Dataproc/EMR jobs in.
   --s3-endpoint S3_ENDPOINT
-                        Force mrjob to connect to S3 on this endpoint (e.g. s3
-                        -us-west-1.amazonaws.com). You usually shouldn't set
+                        Force mrjob to connect to S3 on this endpoint (e.g.
+                        s3-us-west-1.amazonaws.com). You usually shouldn't set
                         this; by default mrjob will choose the correct
                         endpoint for each S3 bucket based on its location.
   --unpooled-only       Only terminate un-pooled clusters
@@ -69,20 +73,18 @@ from datetime import timedelta
 
 from mrjob.aws import _boto3_now
 from mrjob.aws import _boto3_paginate
-from mrjob.emr import _attempt_to_acquire_lock
 from mrjob.emr import EMRJobRunner
 from mrjob.job import MRJob
 from mrjob.options import _add_basic_args
 from mrjob.options import _add_runner_args
 from mrjob.options import _alphabetize_actions
 from mrjob.options import _filter_by_role
-from mrjob.pool import _pool_hash_and_name
+from mrjob.pool import _pool_name
 from mrjob.util import strip_microseconds
 
 log = logging.getLogger(__name__)
 
 _DEFAULT_MAX_MINS_IDLE = 60
-_DEFAULT_MAX_MINUTES_LOCKED = 1
 
 
 def main(cl_args=None):
@@ -92,16 +94,7 @@ def main(cl_args=None):
     MRJob.set_up_logging(quiet=options.quiet,
                          verbose=options.verbose)
 
-    # max_hours_idle -> max_mins_idle
     max_mins_idle = options.max_mins_idle
-    if max_mins_idle is None and options.max_hours_idle is not None:
-        log.warning('--max-hours-idle is deprecated and will be removed'
-                    ' in v0.7.0. Please use --max-mins-idle instead.')
-        max_mins_idle = options.max_hours_idle * 60
-
-    if options.mins_to_end_of_hour is not None:
-        log.warning('--mins-to-end-of-hour is deprecated as of v0.6.0'
-                    ' and does nothing')
 
     _maybe_terminate_clusters(
         dry_run=options.dry_run,
@@ -110,7 +103,6 @@ def main(cl_args=None):
         now=_boto3_now(),
         pool_name=options.pool_name,
         pooled_only=options.pooled_only,
-        max_mins_locked=options.max_mins_locked,
         quiet=options.quiet,
         **_runner_kwargs(options)
     )
@@ -118,10 +110,8 @@ def main(cl_args=None):
 
 def _runner_kwargs(options):
     kwargs = options.__dict__.copy()
-    for unused_arg in ('quiet', 'verbose', 'max_mins_idle', 'max_hours_idle',
-                       'mins_to_end_of_hour',
-                       'max_mins_locked', 'unpooled_only',
-                       'pooled_only', 'pool_name', 'dry_run'):
+    for unused_arg in ('quiet', 'verbose', 'max_mins_idle',
+                       'unpooled_only', 'pooled_only', 'pool_name', 'dry_run'):
         del kwargs[unused_arg]
 
     return kwargs
@@ -133,7 +123,6 @@ def _maybe_terminate_clusters(dry_run=False,
                               pool_name=None,
                               pooled_only=False,
                               unpooled_only=False,
-                              max_mins_locked=None,
                               quiet=False,
                               **kwargs):
     if now is None:
@@ -192,7 +181,7 @@ def _maybe_terminate_clusters(dry_run=False,
         # need to get actual cluster to see tags
         cluster = emr_client.describe_cluster(ClusterId=cluster_id)['Cluster']
 
-        _, pool = _pool_hash_and_name(cluster)
+        pool = _pool_name(cluster)
 
         if is_pending:
             num_pending += 1
@@ -200,12 +189,14 @@ def _maybe_terminate_clusters(dry_run=False,
             num_idle += 1
 
         log.debug(
-            'cluster %s %s for %s, %s (%s)' %
+            'cluster %s %s for %s, %s (%s) - %s' %
             (cluster_id,
              'pending' if is_pending else 'idle',
              strip_microseconds(time_idle),
              ('unpooled' if pool is None else 'in %s pool' % pool),
-             cluster_summary['Name']))
+             cluster_summary['Name'],
+             'protected' if cluster['TerminationProtected'] else 'unprotected',
+             ))
 
         # filter out clusters that don't meet our criteria
         if (max_mins_idle is not None and
@@ -221,6 +212,9 @@ def _maybe_terminate_clusters(dry_run=False,
         if (pool_name is not None and pool != pool_name):
             continue
 
+        if cluster['TerminationProtected']:
+            continue
+
         # terminate idle cluster
         _terminate_and_notify(
             runner=runner,
@@ -230,7 +224,6 @@ def _maybe_terminate_clusters(dry_run=False,
             is_pending=is_pending,
             time_idle=time_idle,
             dry_run=dry_run,
-            max_mins_locked=max_mins_locked,
             quiet=quiet)
 
     log.info(
@@ -305,50 +298,30 @@ def _time_last_active(cluster_summary, steps):
 
 def _terminate_and_notify(runner, cluster_id, cluster_name, num_steps,
                           is_pending, time_idle,
-                          dry_run=False, max_mins_locked=None, quiet=False):
+                          dry_run=False, quiet=False):
+    emr_client = runner.make_emr_client()
 
-    fmt = ('Terminated cluster %s (%s); was %s for %s')
-    msg = fmt % (
-        cluster_id, cluster_name,
-        'pending' if is_pending else 'idle',
-        strip_microseconds(time_idle))
+    if not dry_run:
+        emr_client.terminate_job_flows(JobFlowIds=[cluster_id])
 
-    did_terminate = False
-    if dry_run:
-        did_terminate = True
-    else:
-        status = _attempt_to_acquire_lock(
-            runner.fs,
-            runner._lock_uri(cluster_id, num_steps),
-            runner._opts['cloud_fs_sync_secs'],
-            '%s (%s)' % (msg,
-                         runner._make_unique_job_key(label='terminate')),
-            mins_to_expiration=max_mins_locked,
-        )
-        if status:
-            runner.make_emr_client().terminate_job_flows(
-                JobFlowIds=[cluster_id])
-            did_terminate = True
-        elif not quiet:
-            log.info('%s was locked between getting cluster info and'
-                     ' trying to terminate it; skipping' % cluster_id)
+    if not quiet:
+        fmt = ('Terminated cluster %s (%s); was %s for %s')
+        msg = fmt % (
+            cluster_id, cluster_name,
+            'pending' if is_pending else 'idle',
+            strip_microseconds(time_idle))
 
-    if did_terminate and not quiet:
         print(msg)
 
 
 def _make_arg_parser():
-    usage = '%(prog)s [options]'
+    usage = '%(prog)s terminate-idle-clusters [options]'
     description = ('Terminate idle EMR clusters that meet the criteria'
                    ' passed in on the command line (or, by default,'
                    ' clusters that have been idle for one hour).')
 
     arg_parser = ArgumentParser(usage=usage, description=description)
 
-    arg_parser.add_argument(
-        '--max-hours-idle', dest='max_hours_idle',
-        default=None, type=float,
-        help=('Please use --max-mins-idle instead.'))
     arg_parser.add_argument(
         '--max-mins-idle', dest='max_mins_idle',
         default=None, type=float,
@@ -359,12 +332,8 @@ def _make_arg_parser():
               ' your jobs can take to start instances and bootstrap.'))
     arg_parser.add_argument(
         '--max-mins-locked', dest='max_mins_locked',
-        default=_DEFAULT_MAX_MINUTES_LOCKED, type=float,
-        help='Max number of minutes a cluster can be locked while idle.')
-    arg_parser.add_argument(
-        '--mins-to-end-of-hour', dest='mins_to_end_of_hour',
-        default=None, type=float,
-        help=('Deprecated, does nothing.'))
+        type=float,
+        help='Deprecated, does nothing')
     arg_parser.add_argument(
         '--unpooled-only', dest='unpooled_only', action='store_true',
         default=False,
