@@ -1,5 +1,6 @@
 # Copyright 2015 Yelp
-# Copyright 2017 Yelp
+# Copyright 2017-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +13,16 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from unittest import TestCase
-
 from mrjob.examples.mr_word_freq_count import MRWordFreqCount
 
 from tests.job import run_job
+from tests.sandbox import BasicTestCase
 
 
-class MRWordFreqCountTestCase(TestCase):
+class MRWordFreqCountTestCase(BasicTestCase):
 
     def test_empty(self):
-        self.assertEqual(run_job(MRWordFreqCount()), {})
+        self.assertEqual(run_job(MRWordFreqCount([])), {})
 
     def test_the_wheels_on_the_bus(self):
         RAW_INPUT = b"""
@@ -45,5 +45,5 @@ class MRWordFreqCountTestCase(TestCase):
             u'wheels': 2,
         }
 
-        self.assertEqual(run_job(MRWordFreqCount(), RAW_INPUT),
+        self.assertEqual(run_job(MRWordFreqCount([]), RAW_INPUT),
                          EXPECTED_OUTPUT)
