@@ -1,5 +1,6 @@
 # Copyright 2015-2017 Yelp
 # Copyright 2018 Yelp and Google Inc.
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,9 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import errno
-from unittest import TestCase
-
 from mrjob.logs.step import _interpret_emr_step_syslog
 from mrjob.logs.step import _interpret_emr_step_stderr
 from mrjob.logs.step import _interpret_hadoop_jar_command_stderr
@@ -24,13 +22,10 @@ from mrjob.logs.step import _match_emr_step_syslog_path
 from mrjob.logs.step import _match_emr_step_stderr_path
 from mrjob.logs.step import _parse_indented_counters
 from mrjob.logs.step import _parse_step_syslog
-from mrjob.py2 import StringIO
-from mrjob.util import log_to_stream
 
 from tests.py2 import Mock
 from tests.py2 import patch
-from tests.quiet import no_handlers_for_logger
-from tests.sandbox import PatcherTestCase
+from tests.sandbox import BasicTestCase
 
 
 # abbreviated version of real output from Hadoop 2.7.0.
@@ -116,7 +111,7 @@ PARSED_NOT_A_VALID_JAR_LOG_LINES = dict(
 )
 
 
-class ParseStepSyslogTestCase(TestCase):
+class ParseStepSyslogTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_parse_step_syslog([]), {})
@@ -138,7 +133,7 @@ class ParseStepSyslogTestCase(TestCase):
         )
 
 
-class InterpretHadoopJarCommandStderrTestCase(TestCase):
+class InterpretHadoopJarCommandStderrTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_interpret_hadoop_jar_command_stderr([]), {})
@@ -295,30 +290,13 @@ class InterpretHadoopJarCommandStderrTestCase(TestCase):
                 ),
             ])
 
-    def test_treat_eio_as_eof(self):
-        def yield_lines():
-            yield ('15/12/11 13:33:11 INFO mapreduce.Job:'
-                   ' Running job: job_1449857544442_0002\n')
-            e = IOError()
-            e.errno = errno.EIO
-            raise e
 
-        self.assertEqual(
-            _interpret_hadoop_jar_command_stderr(yield_lines()),
-            dict(job_id='job_1449857544442_0002'))
+class ParseIndentedCountersTestCase(BasicTestCase):
 
-    def test_raise_other_io_errors(self):
-        def yield_lines():
-            yield ('15/12/11 13:33:11 INFO mapreduce.Job:'
-                   ' Running job: job_1449857544442_0002\n')
-            raise IOError
+    def setUp(self):
+        super(ParseIndentedCountersTestCase, self).setUp()
 
-        self.assertRaises(
-            IOError,
-            _interpret_hadoop_jar_command_stderr, yield_lines())
-
-
-class ParseIndentedCountersTestCase(TestCase):
+        self.log = self.start(patch('mrjob.logs.step.log'))
 
     def test_empty(self):
         self.assertEqual(_parse_indented_counters([]), {})
@@ -349,18 +327,14 @@ class ParseIndentedCountersTestCase(TestCase):
             '    FILE: Number of bytes read=86',
         ]
 
-        with no_handlers_for_logger('mrjob.logs.step'):
-            stderr = StringIO()
-            log_to_stream('mrjob.logs.step', stderr)
+        self.assertEqual(_parse_indented_counters(lines), {
+            'File System Counters': {
+                'FILE: Number of bytes read': 86,
+            },
+        })
 
-            self.assertEqual(_parse_indented_counters(lines), {
-                'File System Counters': {
-                    'FILE: Number of bytes read': 86,
-                },
-            })
-
-            # header shouldn't freak it out
-            self.assertEqual(stderr.getvalue(), '')
+        # header shouldn't freak it out
+        self.assertFalse(self.log.warning.called)
 
     def test_indentation_is_required(self):
         lines = [
@@ -368,15 +342,11 @@ class ParseIndentedCountersTestCase(TestCase):
             '   FILE: Number of bytes read=8',
         ]
 
-        with no_handlers_for_logger('mrjob.logs.step'):
-            stderr = StringIO()
-            log_to_stream('mrjob.logs.step', stderr)
+        # counter line is interpreted as group
+        self.assertEqual(_parse_indented_counters(lines), {})
 
-            # counter line is interpreted as group
-            self.assertEqual(_parse_indented_counters(lines), {})
-
-            # should complain
-            self.assertNotEqual(stderr.getvalue(), '')
+        # should complain
+        self.assertTrue(self.log.warning.called)
 
     def test_no_empty_groups(self):
         lines = [
@@ -392,7 +362,7 @@ class ParseIndentedCountersTestCase(TestCase):
         })
 
 
-class MatchEMRStepSyslogPathTestCase(TestCase):
+class MatchEMRStepSyslogPathTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_match_emr_step_syslog_path(''), None)
@@ -435,7 +405,7 @@ class MatchEMRStepSyslogPathTestCase(TestCase):
 # this is currently almost identical to MatchEMRStepSyslogPathTestCase,
 # but that could change (for example, previous versions of the code
 # ignored rotated stderr logs)
-class MatchEMRStepStderrPathTestCase(TestCase):
+class MatchEMRStepStderrPathTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_match_emr_step_stderr_path(''), None)
@@ -475,7 +445,7 @@ class MatchEMRStepStderrPathTestCase(TestCase):
             _match_emr_step_stderr_path(syslog_path), None)
 
 
-class InterpretEMRStepSyslogTestCase(PatcherTestCase):
+class InterpretEMRStepSyslogTestCase(BasicTestCase):
 
     def setUp(self):
         super(InterpretEMRStepSyslogTestCase, self).setUp()
@@ -632,7 +602,7 @@ class InterpretEMRStepSyslogTestCase(PatcherTestCase):
     maxDiff = None
 
 
-class InterpretEMRStepStderrTestCase(PatcherTestCase):
+class InterpretEMRStepStderrTestCase(BasicTestCase):
 
     def setUp(self):
         super(InterpretEMRStepStderrTestCase, self).setUp()
