@@ -1,6 +1,6 @@
 # -*- encoding: utf-8 -*-
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2017 Yelp
+# Copyright 2018 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,10 +13,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from unittest import TestCase
-
 from mrjob.logs.task import _interpret_task_logs
-from mrjob.logs.task import _interpret_spark_task_logs
 from mrjob.logs.task import _ls_spark_task_logs
 from mrjob.logs.task import _ls_task_logs
 from mrjob.logs.task import _match_task_log_path
@@ -26,10 +23,10 @@ from mrjob.logs.task import _parse_task_syslog
 from tests.py2 import call
 from tests.py2 import Mock
 from tests.py2 import patch
-from tests.sandbox import PatcherTestCase
+from tests.sandbox import BasicTestCase
 
 
-class MatchTaskLogPathTestCase(TestCase):
+class MatchTaskLogPathTestCase(BasicTestCase):
 
     PRE_YARN_STDERR_PATH = (
         '/userlogs/attempt_201512232143_0008_m_000001_3/stderr')
@@ -130,7 +127,7 @@ class MatchTaskLogPathTestCase(TestCase):
 
 # test how _ls_task_logs() and _ls_spark_task_logs() filter by
 # error_attempt_id
-class LsTaskLogsTestCase(PatcherTestCase):
+class LsTaskLogsTestCase(BasicTestCase):
 
     def setUp(self):
         super(LsTaskLogsTestCase, self).setUp()
@@ -235,7 +232,7 @@ class LsTaskLogsTestCase(PatcherTestCase):
             [self.mock_paths[2]])
 
 
-class InterpretTaskLogsTestCase(PatcherTestCase):
+class InterpretTaskLogsTestCase(BasicTestCase):
 
     maxDiff = None
 
@@ -641,374 +638,7 @@ class InterpretTaskLogsTestCase(PatcherTestCase):
         )
 
 
-class InterpretSparkTaskLogsTestCase(PatcherTestCase):
-
-    maxDiff = None
-
-    def setUp(self):
-        super(InterpretSparkTaskLogsTestCase, self).setUp()
-
-        # instead of mocking out contents of files, just mock out
-        # what _parse_task_{syslog,stderr}() should return, and have
-        # _cat_log_lines() just pass through the path
-        self.mock_paths = []
-        self.path_to_mock_result = {}
-
-        self.mock_log_callback = Mock()
-
-        def mock_cat_log_lines(fs, path):
-            if path in self.mock_paths:
-                return path
-
-        # (the actual log-parsing functions take lines from the log)
-        def mock_parse_task_syslog(path_from_mock_cat_log_lines):
-            # default is {}
-            return self.path_to_mock_result.get(
-                path_from_mock_cat_log_lines, {})
-
-        def mock_parse_task_stderr(path_from_mock_cat_log_lines):
-            # default is None
-            return self.path_to_mock_result.get(path_from_mock_cat_log_lines)
-
-        def mock_exists(path):
-            return path in self.mock_paths or path == 'MOCK_LOG_DIR'
-
-        # need to mock ls so that _ls_spark_task_logs() can work
-        def mock_ls(log_dir):
-            return self.mock_paths
-
-        self.mock_fs = Mock()
-        self.mock_fs.exists = Mock(side_effect=mock_exists)
-        self.mock_fs.ls = Mock(side_effect=mock_ls)
-
-        self.mock_cat_log_lines = self.start(
-            patch('mrjob.logs.task._cat_log_lines',
-                  side_effect=mock_cat_log_lines))
-
-        self.start(patch('mrjob.logs.task._parse_task_syslog',
-                         side_effect=mock_parse_task_syslog))
-        self.start(patch('mrjob.logs.task._parse_task_stderr',
-                         side_effect=mock_parse_task_stderr))
-
-    def mock_path_matches(self):
-        mock_log_dir_stream = [['MOCK_LOG_DIR']]  # _ls_logs() needs this
-        return _ls_spark_task_logs(self.mock_fs, mock_log_dir_stream)
-
-    def interpret_spark_task_logs(self, **kwargs):
-        return _interpret_spark_task_logs(
-            self.mock_fs, self.mock_path_matches(),
-            log_callback=self.mock_log_callback,
-            **kwargs)
-
-    def test_empty(self):
-        self.assertEqual(self.interpret_spark_task_logs(), {})
-
-    def test_stderr_with_no_error(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-
-        self.mock_paths = [stderr_path]
-
-        self.assertEqual(self.interpret_spark_task_logs(), {})
-
-    def test_stderr_with_split_only(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-
-        self.mock_paths = [stderr_path]
-
-        self.path_to_mock_result = {
-            stderr_path: dict(split=dict(path='best_input_file_ever'))
-        }
-
-        self.assertEqual(self.interpret_spark_task_logs(), {})
-
-    def test_stderr_with_executor_error(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-
-        self.mock_paths = [stderr_path]
-
-        self.path_to_mock_result = {
-            stderr_path: dict(hadoop_error=dict(message='BOOM')),
-        }
-
-        self.assertEqual(self.interpret_spark_task_logs(), dict(
-            errors=[
-                dict(
-                    container_id='container_1450486922681_0005_01_000004',
-                    hadoop_error=dict(
-                        message='BOOM',
-                        path=stderr_path,
-                    ),
-                ),
-            ],
-            partial=True,
-        ))
-
-    def test_stderr_with_executor_error_and_split(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-
-        self.mock_paths = [stderr_path]
-
-        self.path_to_mock_result = {
-            stderr_path: dict(hadoop_error=dict(message='BOOM'),
-                              split=dict(path='best_input_file_ever')),
-        }
-
-        self.assertEqual(self.interpret_spark_task_logs(), dict(
-            errors=[
-                dict(
-                    container_id='container_1450486922681_0005_01_000004',
-                    hadoop_error=dict(
-                        message='BOOM',
-                        path=stderr_path,
-                    ),
-                    split=dict(path='best_input_file_ever'),
-                ),
-            ],
-            partial=True,
-        ))
-
-    def test_stderr_with_application_exited_and_stdout(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-        stdout_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stdout')
-
-        self.mock_paths = [stderr_path, stdout_path]
-
-        self.path_to_mock_result = {
-            stderr_path: dict(
-                check_stdout=True,
-                hadoop_error=dict(message='application exited')),
-            stdout_path: dict(message='because, exploding code')
-        }
-
-        self.assertEqual(
-            self.interpret_spark_task_logs(),
-            dict(
-                errors=[
-                    dict(
-                        container_id='container_1450486922681_0005_01_000004',
-                        hadoop_error=dict(
-                            message='application exited',
-                            path=stderr_path,
-                        ),
-                        task_error=dict(
-                            message='because, exploding code',
-                            path=stdout_path,
-                        ),
-                    ),
-                ],
-                partial=True,
-            )
-        )
-
-        self.assertEqual(
-            self.mock_log_callback.call_args_list,
-            [call(stderr_path), call(stdout_path)])
-
-    def test_stderr_with_application_exited_and_empty_stdout(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-        stdout_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stdout')
-
-        self.mock_paths = [stderr_path, stdout_path]
-
-        self.path_to_mock_result = {
-            stderr_path: dict(
-                check_stdout=True,
-                hadoop_error=dict(message='application exited')),
-        }
-
-        self.assertEqual(
-            self.interpret_spark_task_logs(),
-            dict(
-                errors=[
-                    dict(
-                        container_id='container_1450486922681_0005_01_000004',
-                        hadoop_error=dict(
-                            message='application exited',
-                            path=stderr_path,
-                        ),
-                    ),
-                ],
-                partial=True,
-            )
-        )
-
-        self.assertEqual(
-            self.mock_log_callback.call_args_list,
-            [call(stderr_path), call(stdout_path)])
-
-    def test_stderr_with_application_exited_and_no_stdout(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-
-        self.mock_paths = [stderr_path]
-
-        self.path_to_mock_result = {
-            stderr_path: dict(
-                check_stdout=True,
-                hadoop_error=dict(message='application exited')),
-        }
-
-        self.assertEqual(
-            self.interpret_spark_task_logs(),
-            dict(
-                errors=[
-                    dict(
-                        container_id='container_1450486922681_0005_01_000004',
-                        hadoop_error=dict(
-                            message='application exited',
-                            path=stderr_path,
-                        ),
-                    ),
-                ],
-                partial=True,
-            )
-        )
-
-        self.assertEqual(
-            self.mock_log_callback.call_args_list,
-            [call(stderr_path)])
-
-    def test_error_in_stdout_only(self):
-        stderr_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stderr')
-        stdout_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                       '/container_1450486922681_0005_01_000004/stdout')
-
-        self.mock_paths = [stderr_path, stdout_path]
-
-        self.path_to_mock_result = {
-            stdout_path: dict(message='because, exploding code')
-        }
-
-        self.assertEqual(
-            self.interpret_spark_task_logs(),
-            {})
-
-        self.assertEqual(
-            self.mock_log_callback.call_args_list,
-            [call(stderr_path)])
-
-    # indirectly tests _ls_spark_task_logs() and its ability to sort by
-    # log type and recency
-    def test_multiple_logs(self):
-        stdout1_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000001/stdout')
-        stderr1_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000001/stderr')
-        stdout2_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000002/stdout')
-        stderr2_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000002/stderr')
-        stdout3_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000003/stdout')
-        stderr3_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000003/stderr')
-        stderr4_path = ('/log/dir/userlogs/application_1450486922681_0005'
-                        '/container_1450486922681_0005_01_000004/stderr')
-
-        self.mock_paths = [
-            stdout1_path,
-            stderr1_path,
-            stdout2_path,
-            stderr2_path,
-            stdout3_path,
-            stderr3_path,
-            stderr4_path,
-        ]
-
-        self.path_to_mock_result = {
-            stderr1_path: dict(
-                hadoop_error=dict(message='BOOM1')),
-            stderr2_path: dict(
-                check_stdout=True,
-                hadoop_error=dict(message='exited with status 2')),
-            stdout2_path: dict(message='BoomException'),
-            stderr4_path: dict(
-                check_stdout=True,
-                hadoop_error=dict(message='exited with status 4')),
-            # no errors for stdout1_path, stdout3_path, or stderr4_path
-        }
-
-        # we should yield from stderr2_path first (latest task number that
-        # has a corresponding stdout)
-        self.assertEqual(self.interpret_spark_task_logs(), dict(
-            errors=[
-                dict(
-                    container_id='container_1450486922681_0005_01_000002',
-                    hadoop_error=dict(
-                        message='exited with status 2',
-                        path=stderr2_path,
-                    ),
-                    task_error=dict(
-                        message='BoomException',
-                        path=stdout2_path,
-                    ),
-                ),
-            ],
-            partial=True,
-        ))
-
-        self.assertEqual(self.mock_log_callback.call_args_list, [
-            call(stderr3_path),
-            call(stderr2_path),
-            call(stdout2_path),
-        ])
-
-        # try again, with partial=False
-        self.mock_log_callback.reset_mock()
-
-        # paths still get sorted by _ls_logs()
-        self.assertEqual(self.interpret_spark_task_logs(partial=False), dict(
-            errors=[
-                dict(
-                    container_id='container_1450486922681_0005_01_000002',
-                    hadoop_error=dict(
-                        message='exited with status 2',
-                        path=stderr2_path,
-                    ),
-                    task_error=dict(
-                        message='BoomException',
-                        path=stdout2_path,
-                    ),
-                ),
-                dict(
-                    container_id='container_1450486922681_0005_01_000001',
-                    hadoop_error=dict(
-                        message='BOOM1',
-                        path=stderr1_path,
-                    ),
-                ),
-                dict(
-                    container_id='container_1450486922681_0005_01_000004',
-                    hadoop_error=dict(
-                        message='exited with status 4',
-                        path=stderr4_path,
-                    ),
-                ),
-            ],
-        ))
-
-        self.assertEqual(
-            self.mock_log_callback.call_args_list,
-            [
-                call(stderr3_path),
-                call(stderr2_path),
-                call(stdout2_path),
-                call(stderr1_path),
-                call(stderr4_path),
-            ]
-        )
-
-
-class ParseTaskSyslogTestCase(TestCase):
+class ParseTaskSyslogTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_parse_task_syslog([]), {})
@@ -1161,7 +791,7 @@ class ParseTaskSyslogTestCase(TestCase):
         )
 
 
-class ParseTaskStderrTestCase(TestCase):
+class ParseTaskStderrTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_parse_task_stderr([]), None)
