@@ -1,5 +1,6 @@
 # Copyright 2009-2017 Yelp and Contributors
-# Copyright 2018 Yelp
+# Copyright 2018-2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -71,6 +72,8 @@ AMI_HADOOP_VERSION_UPDATES = {
     '4.8.2': '2.7.3',
     '5.0.0': '2.7.2',
     '5.0.3': '2.7.3',
+    '5.12.0': '2.8.3',
+    '5.16.0': '2.8.4',
 }
 
 # does the AMI have a limit on the *total* number of steps?
@@ -106,6 +109,9 @@ DEFAULT_MAX_STEPS_RETURNED = 50
 # need to fill in a version for non-Hadoop applications
 DUMMY_APPLICATION_VERSION = '0.0.0'
 
+# give the runner a chance to add more steps
+DEFAULT_AUTO_TERMINATE_COUNTDOWN = 3
+
 
 # TODO: raise InvalidRequest: Missing required header for this
 # request: x-amz-content-sha256 when region name is clearly invalid
@@ -247,6 +253,7 @@ class MockEMRClient(object):
                 StateChangeReason={},
                 Timeline=dict(CreationDateTime=now),
             ),
+            StepConcurrencyLevel=1,
             Tags=[],
             TerminationProtected=False,
             VisibleToAllUsers=False,
@@ -321,6 +328,12 @@ class MockEMRClient(object):
                 'Must specify exactly one of the following:'
                 ' release label, AMI version, or Hadoop version.')
 
+        # CustomAmiId
+        if kwargs.get('CustomAmiId'):
+            if not version_gte(running_ami_version, '5.7.0'):
+                raise _error('Custom AMI is not allowed')
+            cluster['CustomAmiId'] = kwargs.pop('CustomAmiId')
+
         # Applications
         hadoop_version = map_version(
             running_ami_version, AMI_HADOOP_VERSION_UPDATES)
@@ -382,6 +395,23 @@ class MockEMRClient(object):
             _validate_param(kwargs, 'VisibleToAllUsers', bool)
             cluster['VisibleToAllUsers'] = kwargs.pop('VisibleToAllUsers')
 
+        # EbsRootVolumeSize
+        if 'EbsRootVolumeSize' in kwargs:
+            # whatever the default EbsRootVolumeSize is, the API doesn't
+            # report it if you don't set it
+            _validate_param(kwargs, 'EbsRootVolumeSize', integer_types)
+            cluster['EbsRootVolumeSize'] = kwargs.pop('EbsRootVolumeSize')
+
+        # StepConcurrencyLevel
+        if 'StepConcurrencyLevel' in kwargs:
+            _validate_param(kwargs, 'StepConcurrencyLevel', integer_types)
+            if not 1 <= kwargs['StepConcurrencyLevel'] <= 256:
+                raise _error('Invalid step concurrency level. Allowed level'
+                             ' is from 1 to 256')
+
+            cluster['StepConcurrencyLevel'] = kwargs.pop(
+                'StepConcurrencyLevel')
+
         # pass BootstrapActions off to helper
         if 'BootstrapActions' in kwargs:
             self._add_bootstrap_actions(
@@ -771,15 +801,15 @@ class MockEMRClient(object):
 
             try:
                 if not float(BidPrice) > 0:
-                        raise _error('The bid price is negative or zero.')
+                    raise _error('The bid price is negative or zero.')
             except (TypeError, ValueError):
                 raise _error(
                     'The bid price supplied for an instance fleet is'
                     ' invalid')
 
             if '.' in BidPrice and len(BidPrice.split('.', 1)[1]) > 3:
-                    raise _error('No more than 3 digits are allowed after'
-                                 ' decimal place in bid price')
+                raise _error('No more than 3 digits are allowed after'
+                             ' decimal place in bid price')
 
             spec['BidPrice'] = BidPrice
 
@@ -1113,6 +1143,14 @@ class MockEMRClient(object):
                     ['CANCEL_AND_WAIT', 'CONTINUE',
                      'TERMINATE_JOB_FLOW', 'TERMINATE_CLUSTER'])
 
+                # CANCEL_AND_WAIT makes no sense when other job's steps
+                # could be cancelled
+                if (Step['ActionOnFailure'] == 'CANCEL_AND_WAIT' and
+                        cluster.get('StepConcurrencyLevel') != 1):
+                    raise _ValidationException(
+                        operation_name,
+                        "Action on failure 'CANCEL_AND_WAIT' is invalid.")
+
                 new_step['ActionOnFailure'] = Step.pop('ActionOnFailure')
 
             # HadoopJarStep (required)
@@ -1226,6 +1264,8 @@ class MockEMRClient(object):
 
         self._add_tags('AddTags', Tags, cluster)
 
+        return {}
+
     def describe_cluster(self, ClusterId):
         _validate_param_type(ClusterId, string_types)
         cluster = self._get_mock_cluster('DescribeCluster', ClusterId)
@@ -1417,6 +1457,27 @@ class MockEMRClient(object):
 
         return [deepcopy(step) for step in results]
 
+    def remove_tags(self, ResourceId, TagKeys):
+        """Simulate successful creation of new metadata tags for the specified
+        resource id.
+        """
+        _validate_param_type(ResourceId, string_types)
+        _validate_param_type(TagKeys, (list, tuple))
+
+        cluster = self._get_mock_cluster('RemoveTags', ResourceId)
+
+        if cluster['Status']['State'].startswith('TERMINATED'):
+            raise _InvalidRequestException(
+                'RemoveTags',
+                'Tags cannot be modified on terminated clusters.')
+
+        # it's okay to remove tags that don't exist
+        TagKeys = set(TagKeys)
+        cluster['Tags'] = [
+            tag for tag in cluster['Tags'] if tag['Key'] not in TagKeys]
+
+        return {}
+
     def terminate_job_flows(self, JobFlowIds):
         _validate_param_type(JobFlowIds, (list, tuple))
 
@@ -1550,9 +1611,17 @@ class MockEMRClient(object):
         # simulate self-termination
         if cluster_id in self.mock_emr_self_termination:
             cluster['Status']['State'] = 'TERMINATING'
+
+            if (len(cluster.get('_InstanceFleets') or ()) == 1 or
+                    len(cluster.get('_InstanceGroups') or ()) == 1):
+                # single master node
+                message = 'All instances in the job flow are terminated'
+            else:
+                message = 'The master node was terminated'
+
             cluster['Status']['StateChangeReason'] = dict(
                 Code='INSTANCE_FAILURE',
-                Message='The master node was terminated. ',  # sic
+                Message=message,
             )
 
             for step in cluster['_Steps']:
@@ -1615,7 +1684,7 @@ class MockEMRClient(object):
                     add_mock_s3_data(self.mock_s3_fs, {
                         bucket_name: {key_name + 'part-%05d' % i: part}})
             elif (cluster_id, step_num) in self.mock_emr_output:
-                raise AssertionError(
+                raise ValueError(
                     "can't use output for cluster ID %s, step %d "
                     "(it doesn't output to S3)" %
                     (cluster_id, step_num))
@@ -1625,19 +1694,23 @@ class MockEMRClient(object):
             if step_num < len(cluster['_Steps']) - 1:
                 return
 
-        # no pending steps. should we wait, or shut down?
+        # no pending steps. should we shut down?
         if cluster['AutoTerminate']:
-            cluster['Status']['State'] = 'TERMINATING'
-            cluster['Status']['StateChangeReason']['Code'] = (
-                'ALL_STEPS_COMPLETED')
-            cluster['Status']['StateChangeReason']['Message'] = (
-                'Steps Completed')
-        else:
-            # just wait
-            cluster['Status']['State'] = 'WAITING'
-            cluster['Status']['StateChangeReason'] = {}
+            cluster.setdefault('_auto_terminate_countdown',
+                               DEFAULT_AUTO_TERMINATE_COUNTDOWN)
+            cluster['_auto_terminate_countdown'] -= 1
 
-        return
+            if cluster['_auto_terminate_countdown'] <= 0:
+                cluster['Status']['State'] = 'TERMINATING'
+                cluster['Status']['StateChangeReason']['Code'] = (
+                    'ALL_STEPS_COMPLETED')
+                cluster['Status']['StateChangeReason']['Message'] = (
+                    'Steps Completed')
+                return
+
+        # just wait
+        cluster['Status']['State'] = 'WAITING'
+        cluster['Status']['StateChangeReason'] = {}
 
 
 # configuration munging
@@ -1707,11 +1780,13 @@ def _validate_param_enum(value, allowed):
 
 
 def _InvalidRequestException(operation_name, message):
-    # boto3 reports this as a botocore.exceptions.InvalidRequestException,
-    # but that's not something you can actually import
-    return AttributeError(
-        'An error occurred (InvalidRequestException) when calling the'
-        ' %s operation: %s' % (operation_name, message))
+    return ClientError(
+        dict(Error=dict(
+            Code='InvalidRequestException',
+            Message=message,
+        )),
+        operation_name
+    )
 
 
 def _ValidationException(operation_name, message):
