@@ -3,6 +3,8 @@
 # Copyright 2014 Contributors
 # Copyright 2015-2016 Yelp
 # Copyright 2017 Yelp and Contributors
+# Copyright 2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -50,6 +52,34 @@ from mrjob.hadoop import _HADOOP_STREAMING_JAR_RE
 from mrjob.parse import urlparse
 from mrjob.util import shlex_split
 
+from tests.sandbox import SandboxedTestCase
+
+
+class MockHadoopTestCase(SandboxedTestCase):
+
+    def setUp(self):
+        super(MockHadoopTestCase, self).setUp()
+        # setup fake hadoop home
+        hadoop_home = self.makedirs('mock_hadoop_home')
+        os.environ['HADOOP_HOME'] = hadoop_home
+        os.environ['MOCK_HADOOP_VERSION'] = "1.2.0"
+        os.environ['MOCK_HADOOP_TMP'] = self.makedirs('mock_hadoop_tmp')
+
+        # make fake hadoop binary
+        os.mkdir(os.path.join(hadoop_home, 'bin'))
+        self.hadoop_bin = os.path.join(hadoop_home, 'bin', 'hadoop')
+        create_mock_hadoop_script(self.hadoop_bin)
+
+        # make fake streaming jar
+        os.makedirs(os.path.join(hadoop_home, 'contrib', 'streaming'))
+        streaming_jar_path = os.path.join(
+            hadoop_home, 'contrib', 'streaming', 'hadoop-0.X.Y-streaming.jar')
+        open(streaming_jar_path, 'w').close()
+
+        # make sure the fake hadoop binaries can find mrjob
+        self.add_mrjob_to_pythonpath()
+
+
 # layout of $MOCK_HADOOP_TMP:
 #   cmd.log: single file containing one line per invocation of mock hadoop
 #     binary, with lines containing space-separated args passed
@@ -112,7 +142,7 @@ def add_mock_hadoop_output(parts):
     for i, part in enumerate(parts):
         part_path = os.path.join(output_subdir, 'part-%05d' % i)
         with open(part_path, 'wb') as part_file:
-                part_file.write(part)
+            part_file.write(part)
 
 
 def get_mock_hadoop_output():
@@ -646,6 +676,35 @@ def hadoop_fs_test(stdout, stderr, environ, *args):
         return 1
 
 
+def hadoop_fs_touchz(stdout, stderr, environ, *args):
+    """Implements hadoop fs -touchz'"""
+    if len(args) < 1:
+        print('Usage: hadoop fs [generic options] -touchz <path> ...]',
+              file=stderr)
+        return -1
+
+    uri = args[0]
+    path = hdfs_uri_to_real_path(uri, environ)
+
+    if os.path.exists(path):
+        if os.path.getsize(path) == 0:
+            return 0
+        else:
+            print(
+                "touchz: `%s': Not a zero-length file" % uri, file=stderr)
+            return 1
+    else:
+        if os.path.exists(os.path.dirname(path)):
+            with open(path, 'w'):
+                pass
+            return 0
+        else:
+            print(
+                "touchz: `%s': No such file or directory: `%s'" % (
+                    uri, uri), file=stderr)
+            return 1
+
+
 def hadoop_jar(stdout, stderr, environ, *args):
     if len(args) < 1:
         print('RunJar jarFile [mainClass] args...', file=stderr)
