@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 # Copyright 2009-2017 Yelp and Contributors
 # Copyright 2018 Google Inc.
+# Copyright 2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,15 +19,22 @@
 import getpass
 import os
 import os.path
+import sys
 from contextlib import contextmanager
 from copy import deepcopy
 from io import BytesIO
 from subprocess import PIPE
-from unittest import TestCase
+from unittest import SkipTest
 
-from google.api_core.exceptions import InvalidArgument
-from google.api_core.exceptions import NotFound
-from google.api_core.exceptions import RequestRangeNotSatisfiable
+try:
+    from google.api_core.exceptions import InvalidArgument
+    from google.api_core.exceptions import NotFound
+    from google.api_core.exceptions import RequestRangeNotSatisfiable
+except ImportError:
+    if sys.version_info[:2] == (3, 4):
+        raise SkipTest('Google libraries are not supported on Python 3.4')
+    else:
+        raise
 
 import mrjob
 import mrjob.dataproc
@@ -45,10 +54,8 @@ from mrjob.fs.gcs import GCSFilesystem
 from mrjob.fs.gcs import parse_gcs_uri
 from mrjob.logs.errors import _pick_error
 from mrjob.py2 import PY2
-from mrjob.py2 import StringIO
 from mrjob.step import StepFailedException
 from mrjob.tools.emr.audit_usage import _JOB_KEY_RE
-from mrjob.util import log_to_stream
 from mrjob.util import save_current_environment
 
 from tests.mock_google import MockGoogleTestCase
@@ -59,20 +66,16 @@ from tests.mr_hadoop_format_job import MRHadoopFormatJob
 from tests.mr_jar_and_streaming import MRJarAndStreaming
 from tests.mr_just_a_jar import MRJustAJar
 from tests.mr_no_mapper import MRNoMapper
+from tests.mr_null_spark import MRNullSpark
+from tests.mr_spark_script import MRSparkScript
 from tests.mr_two_step_job import MRTwoStepJob
 from tests.mr_word_count import MRWordCount
 from tests.py2 import call
 from tests.py2 import mock
 from tests.py2 import patch
-from tests.quiet import logger_disabled
-from tests.quiet import no_handlers_for_logger
+from tests.sandbox import BasicTestCase
 from tests.sandbox import mrjob_conf_patcher
-
-# used to match command lines
-if PY2:
-    PYTHON_BIN = 'python'
-else:
-    PYTHON_BIN = 'python3'
+from tests.test_bin import PYTHON_BIN
 
 US_EAST_GCE_REGION = 'us-east1'
 EU_WEST_GCE_REGION = 'europe-west1'
@@ -163,7 +166,7 @@ class DataprocJobRunnerEndToEndTestCase(MockGoogleTestCase):
 
         mr_job = MRHadoopFormatJob(['-r', 'dataproc', '-v',
                                     '-', local_input_path, remote_input_path,
-                                    '--jobconf', 'x=y'])
+                                    '-D', 'x=y'])
         mr_job.sandbox(stdin=stdin)
 
         results = []
@@ -221,13 +224,6 @@ class DataprocJobRunnerEndToEndTestCase(MockGoogleTestCase):
             # job overrides jobconf in step 1
             self.assertIn('x=z', step_1_args)
 
-            # make sure mrjob.zip is created and uploaded as a bootstrap file
-            self.assertTrue(os.path.exists(runner._mrjob_zip_path))
-            self.assertIn(runner._mrjob_zip_path,
-                          runner._upload_mgr.path_to_uri())
-            self.assertIn(runner._mrjob_zip_path,
-                          runner._bootstrap_dir_mgr.paths())
-
             cluster_id = runner.get_cluster_id()
 
         self.assertEqual(sorted(results),
@@ -249,20 +245,20 @@ class DataprocJobRunnerEndToEndTestCase(MockGoogleTestCase):
         mr_job = MRTwoStepJob(['-r', 'dataproc', '-v'])
         mr_job.sandbox()
 
-        with no_handlers_for_logger('mrjob.dataproc'):
-            stderr = StringIO()
-            log_to_stream('mrjob.dataproc', stderr)
+        log = self.start(patch('mrjob.dataproc.log'))
 
-            self.mock_jobs_succeed = False
+        self.mock_jobs_succeed = False
 
-            with mr_job.make_runner() as runner:
-                self.assertIsInstance(runner, DataprocJobRunner)
+        with mr_job.make_runner() as runner:
+            self.assertIsInstance(runner, DataprocJobRunner)
 
-                self.assertRaises(StepFailedException, runner.run)
+            self.assertRaises(StepFailedException, runner.run)
 
-                self.assertIn(' => ERROR\n', stderr.getvalue())
+            info = ''.join(
+                [a[0] + '\n' for a, kw in log.info.call_args_list])
+            self.assertIn(' => ERROR\n', info)
 
-                cluster_id = runner.get_cluster_id()
+            cluster_id = runner.get_cluster_id()
 
         # job should get terminated
         cluster = runner._get_cluster(cluster_id)
@@ -288,7 +284,7 @@ class DataprocJobRunnerEndToEndTestCase(MockGoogleTestCase):
         # with statement finishes, cleanup runs
 
         self.assertEqual(
-            len(list(fs.client.bucket(tmp_bucket).list_blobs())),
+            len(list(fs.gcs.client.bucket(tmp_bucket).list_blobs())),
             tmp_len)
 
     def test_cleanup_all(self):
@@ -301,13 +297,13 @@ class DataprocJobRunnerEndToEndTestCase(MockGoogleTestCase):
         self._test_cloud_tmp_cleanup('CLOUD_TMP', 0)
 
     def test_cleanup_local(self):
-        self._test_cloud_tmp_cleanup('LOCAL_TMP', 4)
+        self._test_cloud_tmp_cleanup('LOCAL_TMP', 5)
 
     def test_cleanup_logs(self):
-        self._test_cloud_tmp_cleanup('LOGS', 4)
+        self._test_cloud_tmp_cleanup('LOGS', 5)
 
     def test_cleanup_none(self):
-        self._test_cloud_tmp_cleanup('NONE', 4)
+        self._test_cloud_tmp_cleanup('NONE', 5)
 
     def test_cleanup_combine(self):
         self._test_cloud_tmp_cleanup('LOGS,CLOUD_TMP', 0)
@@ -367,8 +363,7 @@ class ExistingClusterTestCase(MockGoogleTestCase):
         with mr_job.make_runner() as runner2:
             self.assertIsInstance(runner2, DataprocJobRunner)
 
-            with logger_disabled('mrjob.dataproc'):
-                self.assertRaises(StepFailedException, runner2.run)
+            self.assertRaises(StepFailedException, runner2.run)
 
             cluster2 = runner2._get_cluster(runner2._cluster_id)
             self.assertEqual(_cluster_state_name(cluster2.status.state),
@@ -387,7 +382,7 @@ class CloudAndHadoopVersionTestCase(MockGoogleTestCase):
             runner.run()
             self.assertEqual(runner.get_image_version(),
                              _DEFAULT_IMAGE_VERSION)
-            self.assertEqual(runner.get_hadoop_version(), '2.7.2')
+            self.assertEqual(runner.get_hadoop_version(), '2.9.2')
 
     def test_image_0_1(self):
         self._assert_cloud_hadoop_version('0.1', '2.7.1')
@@ -402,8 +397,12 @@ class CloudAndHadoopVersionTestCase(MockGoogleTestCase):
         # regression test for #1428
         self._assert_cloud_hadoop_version('1.0.11', '2.7.2')
 
+    def test_image_1_2(self):
+        # regression test for #1428
+        self._assert_cloud_hadoop_version('1.2', '2.8.5')
+
     def test_future_proofing(self):
-        self._assert_cloud_hadoop_version('5.0', '2.7.2')
+        self._assert_cloud_hadoop_version('5.0', '2.9.2')
 
     def _assert_cloud_hadoop_version(self, image_version, hadoop_version):
         with self.make_runner('--image-version', image_version) as runner:
@@ -412,12 +411,11 @@ class CloudAndHadoopVersionTestCase(MockGoogleTestCase):
             self.assertEqual(runner.get_hadoop_version(), hadoop_version)
 
     def test_hadoop_version_option_does_nothing(self):
-        with logger_disabled('mrjob.dataproc'):
-            with self.make_runner('--hadoop-version', '1.2.3.4') as runner:
-                runner.run()
-                self.assertEqual(runner.get_image_version(),
-                                 _DEFAULT_IMAGE_VERSION)
-                self.assertEqual(runner.get_hadoop_version(), '2.7.2')
+        with self.make_runner('--hadoop-version', '1.2.3.4') as runner:
+            runner.run()
+            self.assertEqual(runner.get_image_version(),
+                             _DEFAULT_IMAGE_VERSION)
+            self.assertEqual(runner.get_hadoop_version(), '2.9.2')
 
 
 class AvailabilityZoneConfigTestCase(MockGoogleTestCase):
@@ -505,8 +503,10 @@ class GCEClusterConfigTestCase(MockGoogleTestCase):
 
 class ClusterPropertiesTestCase(MockGoogleTestCase):
 
+    MRJOB_CONF_CONTENTS = None
+
     def _get_cluster_properties(self, *args):
-        job = MRWordCount(['-r', 'dataproc'] + list(args))
+        job = MRWordCount(['--no-conf', '-r', 'dataproc'] + list(args))
         job.sandbox()
 
         with job.make_runner() as runner:
@@ -537,9 +537,7 @@ class ClusterPropertiesTestCase(MockGoogleTestCase):
             b"      'dataproc:dataproc.allow.zero.workers': true\n"
             b"      'hdfs:dfs.namenode.handler.count': 40\n")
 
-        self.mrjob_conf_patcher.stop()
         props = self._get_cluster_properties('-c', conf_path)
-        self.mrjob_conf_patcher.start()
 
         self.assertEqual(props['dataproc:dataproc.allow.zero.workers'], 'true')
         self.assertEqual(props['hdfs:dfs.namenode.handler.count'], '40')
@@ -647,6 +645,7 @@ class RegionAndZoneOptsTestCase(MockGoogleTestCase):
 
 
 class TmpBucketTestCase(MockGoogleTestCase):
+
     def assert_new_tmp_bucket(self, location, **runner_kwargs):
         """Assert that if we create an DataprocJobRunner with the given keyword
         args, it'll create a new tmp bucket with the given location
@@ -655,20 +654,21 @@ class TmpBucketTestCase(MockGoogleTestCase):
         existing_buckets = set(self.mock_gcs_fs)
 
         runner = DataprocJobRunner(conf_paths=[], **runner_kwargs)
+        runner._upload_local_files()
 
         bucket_name, path = parse_gcs_uri(runner._cloud_tmp_dir)
-        runner._create_fs_tmp_bucket(bucket_name, location=location)
+        runner._upload_local_files()
 
         self.assertTrue(bucket_name.startswith('mrjob-'))
         self.assertNotIn(bucket_name, existing_buckets)
         self.assertEqual(path, 'tmp/')
 
-        current_bucket = runner.fs.get_bucket(bucket_name)
+        current_bucket = runner.fs.gcs.get_bucket(bucket_name)
 
         self.assertEqual(current_bucket.location, location.upper())
 
         # Verify that we setup bucket lifecycle rules of 28-day retention
-        first_lifecycle_rule = current_bucket.lifecycle_rules[0]
+        first_lifecycle_rule = list(current_bucket.lifecycle_rules)[0]
         self.assertEqual(first_lifecycle_rule['action'], dict(type='Delete'))
         self.assertEqual(first_lifecycle_rule['condition'],
                          dict(age=_DEFAULT_CLOUD_TMP_DIR_OBJECT_TTL_DAYS))
@@ -1051,7 +1051,6 @@ class MasterBootstrapScriptTestCase(MockGoogleTestCase):
         # check files get downloaded
         assertScriptDownloads(foo_py_path, 'bar.py')
         assertScriptDownloads('gs://walrus/scripts/ohnoes.sh')
-        assertScriptDownloads(runner._mrjob_zip_path)
 
         # check scripts get run
 
@@ -1071,16 +1070,6 @@ class MasterBootstrapScriptTestCase(MockGoogleTestCase):
         self.assertIn('  speedups.sh', lines)
         self.assertIn('  /tmp/s.sh', lines)
 
-        # bootstrap_mrjob
-        mrjob_zip_name = runner._bootstrap_dir_mgr.name(
-            'file', runner._mrjob_zip_path)
-        self.assertIn("  __mrjob_PYTHON_LIB=$(" + PYTHON_BIN + " -c 'from"
-                      " distutils.sysconfig import get_python_lib;"
-                      " print(get_python_lib())')", lines)
-        self.assertIn('  sudo unzip $__mrjob_PWD/' + mrjob_zip_name +
-                      ' -d $__mrjob_PYTHON_LIB', lines)
-        self.assertIn('  sudo ' + PYTHON_BIN + ' -m compileall -q -f'
-                      ' $__mrjob_PYTHON_LIB/mrjob && true', lines)
         # bootstrap_python
         if PY2:
             self.assertIn(
@@ -1098,19 +1087,6 @@ class MasterBootstrapScriptTestCase(MockGoogleTestCase):
         runner._add_bootstrap_files_for_upload()
         self.assertIsNone(runner._master_bootstrap_script_path)
 
-    def test_bootstrap_mrjob_uses_python_bin(self):
-        # use all the bootstrap options
-        runner = DataprocJobRunner(conf_paths=[],
-                                   bootstrap_mrjob=True,
-                                   python_bin=['anaconda'])
-
-        runner._add_bootstrap_files_for_upload()
-        self.assertIsNotNone(runner._master_bootstrap_script_path)
-        with open(runner._master_bootstrap_script_path, 'r') as f:
-            content = f.read()
-
-        self.assertIn('sudo anaconda -m compileall -q -f', content)
-
     def test_bootstrap_script_respects_sh_bin(self):
         runner = DataprocJobRunner(conf_paths=[])
 
@@ -1263,32 +1239,29 @@ class CleanUpJobTestCase(MockGoogleTestCase):
             self.assertFalse(m['_cleanup_job'].called)
 
     def test_kill_cluster(self):
-        with no_handlers_for_logger('mrjob.dataproc'):
-            r = self._quick_runner()
-            with patch.object(mrjob.dataproc.DataprocJobRunner,
-                              '_delete_cluster') as m:
-                r._cleanup_cluster()
-                self.assertTrue(m.called)
+        r = self._quick_runner()
+        with patch.object(mrjob.dataproc.DataprocJobRunner,
+                          '_delete_cluster') as m:
+            r._cleanup_cluster()
+            self.assertTrue(m.called)
 
     def test_kill_cluster_if_successful(self):
         # If they are setting up the cleanup to kill the cluster, mrjob should
         # kill the cluster independent of job success.
-        with no_handlers_for_logger('mrjob.dataproc'):
-            r = self._quick_runner()
-            with patch.object(mrjob.dataproc.DataprocJobRunner,
-                              '_delete_cluster') as m:
-                r._ran_job = True
-                r._cleanup_cluster()
-                self.assertTrue(m.called)
+        r = self._quick_runner()
+        with patch.object(mrjob.dataproc.DataprocJobRunner,
+                          '_delete_cluster') as m:
+            r._ran_job = True
+            r._cleanup_cluster()
+            self.assertTrue(m.called)
 
     def test_kill_persistent_cluster(self):
-        with no_handlers_for_logger('mrjob.dataproc'):
-            r = self._quick_runner()
-            with patch.object(mrjob.dataproc.DataprocJobRunner,
-                              '_delete_cluster') as m:
-                r._opts['cluster_id'] = 'j-MOCKCLUSTER0'
-                r._cleanup_cluster()
-                self.assertTrue(m.called)
+        r = self._quick_runner()
+        with patch.object(mrjob.dataproc.DataprocJobRunner,
+                          '_delete_cluster') as m:
+            r._opts['cluster_id'] = 'j-MOCKCLUSTER0'
+            r._cleanup_cluster()
+            self.assertTrue(m.called)
 
 
 class BootstrapPythonTestCase(MockGoogleTestCase):
@@ -1530,6 +1503,16 @@ class UpdateStepInterpretationTestCase(MockGoogleTestCase):
             },
         )
 
+    # read_logs=False shouldn't turn off step interpretation
+
+    def test_progress_with_no_read_logs(self):
+        self.runner._opts['read_logs'] = False
+        self.test_progress()
+
+    def test_counters_with_no_read_logs(self):
+        self.runner._opts['read_logs'] = False
+        self.test_counters()
+
 
 class ProgressAndCounterLoggingTestCase(MockGoogleTestCase):
 
@@ -1648,6 +1631,22 @@ class SetUpSSHTunnelTestCase(MockGoogleTestCase):
 
         self.assertEqual(args[:4], ['/path/to/gcloud', '-v', 'compute', 'ssh'])
 
+    def test_empty_gcloud_bin_means_default(self):
+        job = MRWordCount(
+            ['-r', 'dataproc', '--ssh-tunnel', '--gcloud-bin', ''])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            runner.run()
+
+        self.assertEqual(self.mock_Popen.call_count, 1)
+        args_tuple, kwargs = self.mock_Popen.call_args
+        args = args_tuple[0]
+
+        self.assertEqual(kwargs, dict(stdin=PIPE, stdout=PIPE, stderr=PIPE))
+
+        self.assertEqual(args[:3], ['gcloud', 'compute', 'ssh'])
+
     def test_missing_gcloud_bin(self):
         self.mock_Popen.side_effect = OSError(2, 'No such file or directory')
 
@@ -1777,7 +1776,7 @@ class FailedTaskContainerIDsTestCase(MockLogEntriesTestCase):
             [])
 
 
-class FixTracebackTestCase(TestCase):
+class FixTracebackTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(_fix_traceback(''), '')
@@ -1805,7 +1804,7 @@ class FixTracebackTestCase(TestCase):
         self.assertEqual(_fix_traceback(message), message)
 
 
-class FixJavaStackTraceTestCase(TestCase):
+class FixJavaStackTraceTestCase(BasicTestCase):
 
     STACK_TRACE = (
         'Diagnostics report from attempt_1525195653111_0001_m_000000_3:'
@@ -1987,6 +1986,46 @@ class CauseOfErrorTestCase(MockLogEntriesTestCase):
             self.assertEqual(error['hadoop_error']['message'], STACK_TRACE)
             self.assertEqual(error['task_error']['message'], TRACEBACK)
 
+    def test_no_read_logs(self):
+        # similar to end-to-end test, but does nothing
+
+        # TODO: in practice, there will be an error in the mock driver output,
+        # which *will* be returned
+
+        # use LOGGING_CLUSTER_NAME so we can generage fake logging entries
+        job = MRBoom(['-r', 'dataproc', '--cluster-id', LOGGING_CLUSTER_NAME,
+                      '--no-read-logs'])
+        job.sandbox()
+
+        self.mock_jobs_succeed = False
+
+        # feed application_id into mock driver output
+        self.get_lines.side_effect = [
+            ['15/12/11 13:32:45 INFO impl.YarnClientImpl:'
+             ' Submitted application %s' % APPLICATION_ID],
+            [],
+            [],
+            [],
+        ]
+
+        self.add_container_exit(CONTAINER_ID_1)
+        self.add_split(CONTAINER_ID_1)
+        self.add_stack_trace(CONTAINER_ID_1)
+        self.add_traceback(CONTAINER_ID_1)
+
+        with job.make_runner() as runner:
+            self.assertRaises(StepFailedException, runner.run)
+
+            self.assertEqual(len(runner._log_interpretations), 1)
+            interp = runner._log_interpretations[0]
+
+            self.assertIn('step', interp)
+            self.assertNotIn('history', interp)
+            self.assertNotIn('task', interp)
+
+            error = _pick_error(interp)
+            self.assertIsNone(error)
+
 
 class CloudPartSizeTestCase(MockGoogleTestCase):
 
@@ -2002,17 +2041,17 @@ class CloudPartSizeTestCase(MockGoogleTestCase):
     def test_default(self):
         runner = DataprocJobRunner()
 
-        self.assertEqual(runner._fs_chunk_size(), 100 * 1024 * 1024)
+        self.assertEqual(runner._upload_part_size(), 100 * 1024 * 1024)
 
     def test_float(self):
         runner = DataprocJobRunner(cloud_part_size_mb=0.25)
 
-        self.assertEqual(runner._fs_chunk_size(), 256 * 1024)
+        self.assertEqual(runner._upload_part_size(), 256 * 1024)
 
     def test_zero(self):
         runner = DataprocJobRunner(cloud_part_size_mb=0)
 
-        self.assertEqual(runner._fs_chunk_size(), None)
+        self.assertEqual(runner._upload_part_size(), None)
 
     def test_multipart_upload(self):
         job = MRWordCount(
@@ -2199,8 +2238,10 @@ class HadoopStreamingJarTestCase(MockGoogleTestCase):
 
 class NetworkAndSubnetworkTestCase(MockGoogleTestCase):
 
+    MRJOB_CONF_CONTENTS = None
+
     def _get_project_id_and_gce_config(self, *args):
-        job = MRWordCount(['-r', 'dataproc'] + list(args))
+        job = MRWordCount(['--no-conf', '-r', 'dataproc'] + list(args))
         job.sandbox()
 
         with job.make_runner() as runner:
@@ -2287,10 +2328,8 @@ class NetworkAndSubnetworkTestCase(MockGoogleTestCase):
             'mrjob.conf',
             b'runners:\n  dataproc:\n    subnet: default')
 
-        self.mrjob_conf_patcher.stop()
         project_id, gce_config = self._get_project_id_and_gce_config(
             '-c', conf_path, '--network', 'test')
-        self.mrjob_conf_patcher.start()
 
         self.assertEqual(
             gce_config.network_uri,
@@ -2303,13 +2342,29 @@ class NetworkAndSubnetworkTestCase(MockGoogleTestCase):
             'mrjob.conf',
             b'runners:\n  dataproc:\n    network: default')
 
-        self.mrjob_conf_patcher.stop()
         project_id, gce_config = self._get_project_id_and_gce_config(
             '-c', conf_path, '--subnet', 'test')
-        self.mrjob_conf_patcher.start()
 
         self.assertEqual(
             gce_config.subnetwork_uri,
             'https://www.googleapis.com/compute/v1/projects/%s'
             '/us-west1/subnetworks/test' % project_id)
         self.assertFalse(gce_config.network_uri)
+
+
+class UnsupportedStepsTestCase(MockGoogleTestCase):
+
+    def test_no_spark_steps(self):
+        # just a sanity check; _STEP_TYPES is tested in a lot of ways
+        job = MRNullSpark(['-r', 'dataproc'])
+        job.sandbox()
+
+        self.assertRaises(NotImplementedError, job.make_runner)
+
+    def test_no_jar_steps(self):
+        spark_script_path = self.makefile('chispa.py')
+
+        job = MRSparkScript(['-r', 'dataproc', '--script', spark_script_path])
+        job.sandbox()
+
+        self.assertRaises(NotImplementedError, job.make_runner)
