@@ -4,6 +4,7 @@
 # Copyright 2015-2016 Yelp
 # Copyright 2017 Yelp and Contributors
 # Copyright 2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,21 +24,23 @@ import os.path
 from io import BytesIO
 from subprocess import check_call
 from subprocess import PIPE
-from unittest import TestCase
+from unittest import skipIf
 
 import mrjob.step
 from mrjob.conf import combine_dicts
 from mrjob.fs.hadoop import HadoopFilesystem
 from mrjob.hadoop import HadoopJobRunner
 from mrjob.hadoop import fully_qualify_hdfs_path
-from mrjob.util import which
+from mrjob.hadoop import pty
+from mrjob.step import StepFailedException
 
 from tests.mockhadoop import add_mock_hadoop_counters
 from tests.mockhadoop import add_mock_hadoop_output
-from tests.mockhadoop import create_mock_hadoop_script
 from tests.mockhadoop import get_mock_hadoop_cmd_args
 from tests.mockhadoop import get_mock_hdfs_root
+from tests.mockhadoop import MockHadoopTestCase
 from tests.mr_jar_and_streaming import MRJarAndStreaming
+from tests.mr_jar_with_generic_args import MRJarWithGenericArgs
 from tests.mr_just_a_jar import MRJustAJar
 from tests.mr_null_spark import MRNullSpark
 from tests.mr_spark_jar import MRSparkJar
@@ -48,46 +51,17 @@ from tests.mr_word_count import MRWordCount
 from tests.py2 import Mock
 from tests.py2 import call
 from tests.py2 import patch
-from tests.quiet import logger_disabled
+from tests.sandbox import BasicTestCase
 from tests.sandbox import EmptyMrjobConfTestCase
 from tests.sandbox import SandboxedTestCase
 from tests.test_bin import PYTHON_BIN
-from tests.test_local import _bash_wrap
 
-# pty isn't available on Windows
-try:
-    import pty
-    pty
-except ImportError:
+if pty is None:
+    # some tests should run even if pty is missing
     pty = Mock()
 
 
-class MockHadoopTestCase(SandboxedTestCase):
-
-    def setUp(self):
-        super(MockHadoopTestCase, self).setUp()
-        # setup fake hadoop home
-        hadoop_home = self.makedirs('mock_hadoop_home')
-        os.environ['HADOOP_HOME'] = hadoop_home
-        os.environ['MOCK_HADOOP_VERSION'] = "1.2.0"
-        os.environ['MOCK_HADOOP_TMP'] = self.makedirs('mock_hadoop_tmp')
-
-        # make fake hadoop binary
-        os.mkdir(os.path.join(hadoop_home, 'bin'))
-        self.hadoop_bin = os.path.join(hadoop_home, 'bin', 'hadoop')
-        create_mock_hadoop_script(self.hadoop_bin)
-
-        # make fake streaming jar
-        os.makedirs(os.path.join(hadoop_home, 'contrib', 'streaming'))
-        streaming_jar_path = os.path.join(
-            hadoop_home, 'contrib', 'streaming', 'hadoop-0.X.Y-streaming.jar')
-        open(streaming_jar_path, 'w').close()
-
-        # make sure the fake hadoop binaries can find mrjob
-        self.add_mrjob_to_pythonpath()
-
-
-class TestFullyQualifyHDFSPath(TestCase):
+class TestFullyQualifyHDFSPath(BasicTestCase):
 
     def test_empty(self):
         with patch('getpass.getuser') as getuser:
@@ -531,6 +505,19 @@ class StreamHistoryLogDirsTestCase(StreamingLogDirsTestCase):
 
         self.assertRaises(StopIteration, next, results)
 
+    def test_no_read_logs(self):
+        self.runner._opts['read_logs'] = False
+
+        self.runner._hadoop_log_dirs.return_value = [
+            '/mnt/var/logs/hadoop', 'hdfs:///logs']
+
+        results = self.runner._stream_history_log_dirs()
+
+        self.assertRaises(StopIteration, next, results)
+
+        self.assertFalse(self.log.info.called)
+        self.assertFalse(self.runner._hadoop_log_dirs.called)
+
 
 class StreamTaskLogDirsTestCase(StreamingLogDirsTestCase):
 
@@ -600,6 +587,19 @@ class StreamTaskLogDirsTestCase(StreamingLogDirsTestCase):
 
         self.assertRaises(StopIteration, next, results)
 
+    def test_no_read_logs(self):
+        self.runner._opts['read_logs'] = False
+
+        self.runner._hadoop_log_dirs.return_value = [
+            '/mnt/var/logs/hadoop', 'hdfs:///logs']
+
+        results = self.runner._stream_task_log_dirs()
+
+        self.assertRaises(StopIteration, next, results)
+
+        self.assertFalse(self.log.info.called)
+        self.assertFalse(self.runner._hadoop_log_dirs.called)
+
 
 class GetHadoopVersionTestCase(MockHadoopTestCase):
 
@@ -639,13 +639,13 @@ class HadoopJobRunnerEndToEndTestCase(MockHadoopTestCase):
         add_mock_hadoop_output([b'1\t"qux"\n2\t"bar"\n',
                                 b'2\t"foo"\n5\tnull\n'])
 
-        # -libjar is now a supported feature. Maybe -verbose?
-
-        mr_job = MRTwoStepJob(['-r', 'hadoop', '-v',
-                               '--no-conf', '--libjar', 'containsJars.jar',
-                               '--hadoop-arg=-verbose'] + list(args)
-                              + ['-', local_input_path, remote_input_path]
-                              + ['--jobconf', 'x=y'])
+        mr_job = MRTwoStepJob([
+            '-r', 'hadoop', '-v',
+            '--no-conf', '--libjars', 'containsJars.jar',
+            '--hadoop-args=-verbose'] + list(args) +
+            ['-', local_input_path, remote_input_path] +
+            ['-D', 'x=y']
+        )
         mr_job.sandbox(stdin=stdin)
 
         local_tmp_dir = None
@@ -682,7 +682,7 @@ class HadoopJobRunnerEndToEndTestCase(MockHadoopTestCase):
             # to PYTHONPATH
             self.assertTrue(os.path.exists(runner._setup_wrapper_script_path))
             self.assertIn(runner._setup_wrapper_script_path,
-                          runner._upload_mgr.path_to_uri())
+                          runner._working_dir_mgr.paths())
             mrjob_zip_name = runner._working_dir_mgr.name(
                 'file', runner._mrjob_zip_path)
             with open(runner._setup_wrapper_script_path) as wrapper:
@@ -717,7 +717,7 @@ class HadoopJobRunnerEndToEndTestCase(MockHadoopTestCase):
             self.assertIn('-mapper', args)
             self.assertLess(args.index('-verbose'), args.index('-mapper'))
 
-        # make sure -libjar is set and comes before mapper
+        # make sure -libjars is set and comes before mapper
         for args in (step_0_args, step_1_args):
             self.assertIn('-libjars', args)
             self.assertIn('containsJars.jar', args)
@@ -877,13 +877,13 @@ class StreamingArgsTestCase(EmptyMrjobConfTestCase):
             self.runner._args_for_streaming_step(0),
             (self.BASIC_HADOOP_ARGS + self.BASIC_JOB_ARGS + [
              '-mapper',
-             "sh -ex -c 'grep anything | " + PYTHON_BIN +
+             "/bin/sh -ex -c 'grep anything | " + PYTHON_BIN +
              " my_job.py --step-num=0 --mapper'",
              '-combiner',
-             "sh -ex -c 'grep nothing | " + PYTHON_BIN +
+             "/bin/sh -ex -c 'grep nothing | " + PYTHON_BIN +
              " my_job.py --step-num=0 --combiner'",
              '-reducer',
-             "sh -ex -c 'grep something | " + PYTHON_BIN +
+             "/bin/sh -ex -c 'grep something | " + PYTHON_BIN +
              " my_job.py --step-num=0 --reducer'"]))
 
     def test_pre_filter_escaping(self):
@@ -893,7 +893,7 @@ class StreamingArgsTestCase(EmptyMrjobConfTestCase):
                 'type': 'streaming',
                 'mapper': {
                     'type': 'script',
-                    'pre_filter': _bash_wrap("grep 'anything'"),
+                    'pre_filter': "bash -c 'grep '\\''anything'\\'''",
                 },
             },
         ]
@@ -904,7 +904,7 @@ class StreamingArgsTestCase(EmptyMrjobConfTestCase):
              ['-D', 'mapreduce.job.reduces=0'] +
              self.BASIC_JOB_ARGS + [
                  '-mapper',
-                 "sh -ex -c 'bash -c '\\''grep"
+                 "/bin/sh -ex -c 'bash -c '\\''grep"
                  " '\\''\\'\\'''\\''anything'\\''\\'\\'''\\'''\\'' | " +
                  PYTHON_BIN +
                  " my_job.py --step-num=0 --mapper'"]))
@@ -940,32 +940,41 @@ class ArgsForJarStepTestCase(MockHadoopTestCase):
                 runner.get_hadoop_bin() +
                 ['jar', jar_uri])
 
-    def test_libjars(self):
+    def test_no_generic_args_by_default(self):
+        # -D and --libjars are ignored unless you use GENERIC_ARGS. See #1863
+
         fake_jar = self.makefile('fake.jar')
         fake_libjar = self.makefile('fake_lib.jar')
 
         job = MRJustAJar(
-            ['-r', 'hadoop', '--jar', fake_jar, '--libjar', fake_libjar])
+            ['-r', 'hadoop', '--jar', fake_jar,
+             '-D', 'foo=bar', '--libjars', fake_libjar])
         job.sandbox()
 
         with job.make_runner() as runner:
             self.assertEqual(
                 runner._args_for_jar_step(0),
                 runner.get_hadoop_bin() +
-                ['-libjars', fake_libjar, 'jar', fake_jar])
+                ['jar', fake_jar])
 
-    def test_jobconf(self):
+    def test_generic_args_interpolation(self):
         fake_jar = self.makefile('fake.jar')
+        fake_libjar = self.makefile('fake_lib.jar')
 
-        job = MRJustAJar(
-            ['-r', 'hadoop', '--jobconf', 'foo=bar', '--jar', fake_jar])
+        job = MRJarWithGenericArgs(
+            ['-r', 'hadoop', '--jar', fake_jar,
+             '-D', 'foo=bar', '--libjars', fake_libjar])
         job.sandbox()
 
         with job.make_runner() as runner:
             self.assertEqual(
                 runner._args_for_jar_step(0),
                 runner.get_hadoop_bin() +
-                ['-D', 'foo=bar', 'jar', fake_jar])
+                ['jar', fake_jar,
+                 'before',
+                 '-libjars', fake_libjar, '-D', 'foo=bar',
+                 'after']
+            )
 
     def test_input_output_interpolation(self):
         # TODO: rewrite this to just check the step args (see #1482)
@@ -1134,10 +1143,6 @@ class RunJobInHadoopUsesEnvTestCase(MockHadoopTestCase):
     def setUp(self):
         super(RunJobInHadoopUsesEnvTestCase, self).setUp()
 
-        self.mock_get_steps = self.start(patch(
-            'mrjob.hadoop.HadoopJobRunner._get_steps',
-            return_value=[dict(type='spark')]))
-
         self.mock_args_for_step = self.start(patch(
             'mrjob.hadoop.HadoopJobRunner._args_for_step',
             return_value=['args', 'for', 'step']))
@@ -1156,8 +1161,11 @@ class RunJobInHadoopUsesEnvTestCase(MockHadoopTestCase):
         self.mock_execvpe = self.start(patch(
             'os.execvpe', side_effect=StopIteration))
 
+        # don't actually hard-exit the child, since we're not actually forking
+        self.mock_exit = self.start(patch('os._exit'))
+
     def test_with_pty(self):
-        job = MRNullSpark(['-r', 'hadoop'])
+        job = MRWordCount(['-r', 'hadoop'])
         job.sandbox()
 
         with job.make_runner() as runner:
@@ -1171,7 +1179,7 @@ class RunJobInHadoopUsesEnvTestCase(MockHadoopTestCase):
     def test_without_pty(self):
         self.mock_pty_fork.side_effect = OSError
 
-        job = MRNullSpark(['-r', 'hadoop'])
+        job = MRWordCount(['-r', 'hadoop'])
         job.sandbox()
 
         with job.make_runner() as runner:
@@ -1184,6 +1192,51 @@ class RunJobInHadoopUsesEnvTestCase(MockHadoopTestCase):
             self.assertFalse(self.mock_execvpe.called)
 
 
+@skipIf(not (pty and hasattr(pty, 'fork')), 'no pty.fork()')
+class BadHadoopBinAfterFork(MockHadoopTestCase):
+    # test what happens if os.execvpe() fails (see #2024)
+
+    # can't just set --hadoop-bin because this would break checking
+    # Hadoop version and uploading files. Instead, patch _args_for_step()
+    def patch_args_for_step(self, runner, bad_hadoop_bin):
+        real_args_for_step = runner._args_for_step
+
+        def args_for_step(*args, **kwargs):
+            args = real_args_for_step(*args, **kwargs)
+            return [bad_hadoop_bin] + args[1:]
+
+        self.start(patch.object(runner, '_args_for_step', args_for_step))
+
+    def test_no_such_file(self):
+        missing_hadoop_bin = os.path.join(self.tmp_dir, 'no-hadoop-here')
+
+        job = MRTwoStepJob(['-r', 'hadoop'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.patch_args_for_step(runner, missing_hadoop_bin)
+            self.assertRaises(StepFailedException, runner.run)
+
+    def test_permissions_error(self):
+        nonexecutable_hadoop_bin = self.makefile('not-really-hadoop')
+
+        job = MRTwoStepJob(['-r', 'hadoop'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.patch_args_for_step(runner, nonexecutable_hadoop_bin)
+            self.assertRaises(StepFailedException, runner.run)
+
+    def test_non_oserror_exception(self):
+        self.start(patch('os.execvpe', side_effect=KeyboardInterrupt))
+
+        job = MRTwoStepJob(['-r', 'hadoop'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertRaises(StepFailedException, runner.run)
+
+
 class SparkPyFilesTestCase(MockHadoopTestCase):
 
     def test_eggs(self):
@@ -1192,22 +1245,27 @@ class SparkPyFilesTestCase(MockHadoopTestCase):
 
         job = MRNullSpark([
             '-r', 'hadoop',
-            '--py-file', egg1_path, '--py-file', egg2_path])
+            '--py-files', '%s,%s' % (egg1_path, egg2_path)])
         job.sandbox()
 
         with job.make_runner() as runner:
+            runner._create_setup_wrapper_scripts()
             runner._add_job_files_for_upload()
 
             self.assertEqual(
-                runner._spark_py_files(),
+                runner._py_files(),
                 [egg1_path, egg2_path, runner._create_mrjob_zip()]
             )
 
-            # the py_files get uploaded anyway since they appear in
-            # _upload_dir_mgr.
+            # pass the URI of the *uploaded* py_files to Spark
             self.assertIn(egg1_path, runner._upload_mgr.path_to_uri())
             self.assertIn(egg2_path, runner._upload_mgr.path_to_uri())
 
+            egg_uris = ','.join(runner._upload_mgr.uri(path)
+                                for path in runner._py_files())
+
+            self.assertIn(egg_uris, runner._spark_submit_args(0))
+
 
 class SetupLineEncodingTestCase(MockHadoopTestCase):
 
@@ -1222,13 +1280,12 @@ class SetupLineEncodingTestCase(MockHadoopTestCase):
         # that use unix line endings anyway. So monitor open() instead
         with patch(
                 'mrjob.runner.open', create=True, side_effect=open) as m_open:
-            with logger_disabled('mrjob.hadoop'):
-                with job.make_runner() as runner:
-                    runner.run()
+            with job.make_runner() as runner:
+                runner.run()
 
-                    self.assertIn(
-                        call(runner._setup_wrapper_script_path, 'wb'),
-                        m_open.mock_calls)
+                self.assertIn(
+                    call(runner._setup_wrapper_script_path, 'wb'),
+                    m_open.mock_calls)
 
 
 class PickErrorTestCase(MockHadoopTestCase):
@@ -1302,7 +1359,7 @@ class HadoopExtraArgsTestCase(MockHadoopTestCase):
 
     def test_hadoop_extra_args(self):
         # hadoop_extra_args doesn't exist in default runner
-        job = MRWordCount(['-r', self.RUNNER, '--hadoop-arg=-foo'])
+        job = MRWordCount(['-r', self.RUNNER, '--hadoop-args=-foo'])
         with job.make_runner() as runner:
             self.assertEqual(runner._hadoop_args_for_step(0), ['-foo'])
 
@@ -1310,8 +1367,8 @@ class HadoopExtraArgsTestCase(MockHadoopTestCase):
         job = MRWordCount(
             ['-r', self.RUNNER,
              '--cmdenv', 'FOO=bar',
-             '--hadoop-arg=-libjar', '--hadoop-arg', 'qux.jar',
-             '--jobconf', 'baz=qux'])
+             '--hadoop-args=-libjars qux.jar',
+             '-D', 'baz=qux'])
         job.HADOOP_INPUT_FORMAT = 'FooInputFormat'
         job.HADOOP_OUTPUT_FORMAT = 'BarOutputFormat'
 
@@ -1319,7 +1376,7 @@ class HadoopExtraArgsTestCase(MockHadoopTestCase):
             hadoop_args = runner._hadoop_args_for_step(0)
             self.assertEqual(
                 hadoop_args[:4],
-                ['-D', 'baz=qux', '-libjar', 'qux.jar'])
+                ['-D', 'baz=qux', '-libjars', 'qux.jar'])
             self.assertEqual(len(hadoop_args), 10)
 
 
@@ -1343,7 +1400,7 @@ class LibjarsTestCase(MockHadoopTestCase):
     def test_one_jar(self):
         job = MRWordCount([
             '-r', 'hadoop',
-            '--libjar', '/path/to/a.jar',
+            '--libjars', '/path/to/a.jar',
         ])
         job.sandbox()
 
@@ -1357,8 +1414,8 @@ class LibjarsTestCase(MockHadoopTestCase):
     def test_two_jars(self):
         job = MRWordCount([
             '-r', 'hadoop',
-            '--libjar', '/path/to/a.jar',
-            '--libjar', '/path/to/b.jar',
+            '--libjars', '/path/to/a.jar',
+            '--libjars', '/path/to/b.jar',
         ])
         job.sandbox()
 
@@ -1370,80 +1427,6 @@ class LibjarsTestCase(MockHadoopTestCase):
             self.assertIn('/path/to/a.jar,/path/to/b.jar', args)
 
 
-class FindSparkSubmitBinTestCase(SandboxedTestCase):
-
-    def setUp(self):
-        super(FindSparkSubmitBinTestCase, self).setUp()
-
-        # track calls to which()
-        self.which = self.start(patch('mrjob.hadoop.which', wraps=which))
-
-        # keep which() from searching in /bin, etc.
-        os.environ['PATH'] = self.tmp_dir
-
-        # create basic runner (okay to overwrite)
-        self.runner = HadoopJobRunner()
-
-    def test_do_nothing_on_init(self):
-        self.assertFalse(self.which.called)
-
-    def test_option_short_circuits_search(self):
-        self.runner = HadoopJobRunner(
-            spark_submit_bin=['/path/to/spark-submit', '-v'])
-
-        self.assertEqual(self.runner.get_spark_submit_bin(),
-                         ['/path/to/spark-submit', '-v'])
-
-        self.assertFalse(self.which.called)
-
-    def test_fallback_and_hard_coded_dirs(self):
-        # don't get caught by real spark install
-        self.which.return_value = None
-
-        os.environ['SPARK_HOME'] = '/spark/home'
-
-        self.assertEqual(self.runner.get_spark_submit_bin(), ['spark-submit'])
-
-        which_paths = [
-            kwargs.get('path') for args, kwargs in self.which.call_args_list]
-
-        self.assertEqual(which_paths, [
-            '/spark/home/bin',
-            None,
-            '/usr/lib/spark/bin',
-            '/usr/local/spark/bin',
-            '/usr/local/lib/spark/bin',
-        ])
-
-    def test_find_in_spark_home(self):
-        spark_submit_bin = self.makefile(
-            os.path.join(self.tmp_dir, 'spark', 'bin', 'spark-submit'),
-            executable=True)
-
-        os.environ['SPARK_HOME'] = os.path.join(self.tmp_dir, 'spark')
-
-        self.runner.get_spark_submit_bin()
-
-        self.assertEqual(self.runner.get_spark_submit_bin(),
-                         [spark_submit_bin])
-
-        self.assertEqual(self.which.call_count, 1)
-
-    def test_find_in_path(self):
-        spark_submit_bin = self.makefile(
-            os.path.join(self.tmp_dir, 'bin', 'spark-submit'),
-            executable=True)
-
-        os.environ['PATH'] = os.path.join(self.tmp_dir, 'bin')
-
-        # don't get caught by real $SPARK_HOME
-        if 'SPARK_HOME' in os.environ:
-            del os.environ['SPARK_HOME']
-
-        self.assertEqual(self.runner.get_spark_submit_bin(),
-                         [spark_submit_bin])
-
-
 class FindBinariesAndJARsTestCase(SandboxedTestCase):
 
     def setUp(self):
@@ -1501,78 +1484,69 @@ class FindBinariesAndJARsTestCase(SandboxedTestCase):
             self.assertTrue(self.get_spark_submit_bin.called)
 
 
-class SparkSubmitArgPrefixTestCase(MockHadoopTestCase):
+class SparkMasterAndDeployModeTestCase(MockHadoopTestCase):
 
     def test_default(self):
-        runner = HadoopJobRunner()
+        mr_job = MRNullSpark(['-r', 'hadoop'])
+        mr_job.sandbox()
 
-        self.assertEqual(
-            runner._spark_submit_arg_prefix(),
-            ['--master', 'yarn'])
+        with mr_job.make_runner() as runner:
+            runner._add_job_files_for_upload()
+            self.assertEqual(
+                runner._spark_submit_args(0)[4:8],
+                ['--master', 'yarn', '--deploy-mode', 'client']
+            )
 
-    def test_spark_master(self):
-        runner = HadoopJobRunner(spark_master='local')
+    def test_spark_master_opt(self):
+        # these are hard-coded and always the same
+        mr_job = MRNullSpark(['-r', 'hadoop', '--spark-master', 'local'])
+        mr_job.sandbox()
 
-        self.assertEqual(
-            runner._spark_submit_arg_prefix(),
-            ['--master', 'local'])
+        with mr_job.make_runner() as runner:
+            runner._add_job_files_for_upload()
+            self.assertEqual(
+                runner._spark_submit_args(0)[2:6],
+                ['--master', 'local', '--deploy-mode', 'client']
+            )
+
+    def test_spark_deploy_mode_opt(self):
+        # these are hard-coded and always the same
+        mr_job = MRNullSpark(['-r', 'hadoop',
+                              '--spark-deploy-mode', 'cluster'])
+        mr_job.sandbox()
+
+        with mr_job.make_runner() as runner:
+            runner._add_job_files_for_upload()
+
+            self.assertEqual(
+                runner._spark_submit_args(0)[4:8],
+                ['--master', 'yarn', '--deploy-mode', 'cluster']
+            )
 
 
-class WarnAboutSparkArchivesTestCase(MockHadoopTestCase):
+class GetHadoopBinTestCase(MockHadoopTestCase):
+    # mostly exists to test --hadoop-bin ''
 
-    def setUp(self):
-        super(WarnAboutSparkArchivesTestCase, self).setUp()
-
-        self.log = self.start(patch('mrjob.hadoop.log'))
-
-    def test_warning(self):
-        fake_archive = self.makefile('fake.tar.gz')
-
-        job = MRNullSpark(['-r', 'hadoop',
-                           '--spark-master', 'local',
-                           '--archive', fake_archive])
+    def test_default(self):
+        job = MRTwoStepJob(['-r', 'hadoop'])
         job.sandbox()
 
         with job.make_runner() as runner:
-            runner._warn_about_spark_archives(runner._get_step(0))
+            self.assertEqual(runner.get_hadoop_bin(),
+                             [self.hadoop_bin])
 
-        self.assertTrue(self.log.warning.called)
-
-    def test_requires_spark(self):
-        fake_archive = self.makefile('fake.tar.gz')
-
-        job = MRTwoStepJob(['-r', 'hadoop',
-                            '--spark-master', 'local',
-                            '--archive', fake_archive])
+    def test_custom_hadoop_bin(self):
+        job = MRTwoStepJob(['-r', 'hadoop', '--hadoop-bin', 'hadoop -v'])
         job.sandbox()
 
         with job.make_runner() as runner:
-            runner._warn_about_spark_archives(runner._get_step(0))
+            self.assertEqual(runner.get_hadoop_bin(),
+                             ['hadoop', '-v'])
 
-        self.assertFalse(self.log.warning.called)
-
-    def test_requires_non_yarn_spark_master(self):
-        fake_archive = self.makefile('fake.tar.gz')
-
-        job = MRNullSpark(['-r', 'hadoop',
-                           '--spark-master', 'yarn',
-                           '--archive', fake_archive])
+    def test_empty_hadoop_bin_means_find_hadoop_bin(self):
+        job = MRTwoStepJob(['-r', 'hadoop', '--hadoop-bin', ''])
         job.sandbox()
 
         with job.make_runner() as runner:
-            runner._warn_about_spark_archives(runner._get_step(0))
-
-        self.assertFalse(self.log.warning.called)
-
-    def test_requires_archives(self):
-        fake_file = self.makefile('fake.txt')
-
-        job = MRNullSpark(['-r', 'hadoop',
-                           '--spark-master', 'local',
-                           '--file', fake_file])
-        job.sandbox()
-
-        with job.make_runner() as runner:
-            runner._warn_about_spark_archives(runner._get_step(0))
-
-        self.assertFalse(self.log.warning.called)
+            self.assertEqual(runner.get_hadoop_bin(),
+                             [self.hadoop_bin])
