@@ -2,6 +2,8 @@
 # Copyright 2013 David Marin and Lyft
 # Copyright 2015-2017 Yelp
 # Copyright 2018 Yelp and Contributors
+# Copyright 2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,32 +19,38 @@
 """Tests for LocalMRJobRunner"""
 import gzip
 import os
-import shutil
 import stat
 import sys
-import tempfile
 from io import BytesIO
 from multiprocessing import Pool
 from multiprocessing import cpu_count
 from os.path import exists
 from os.path import join
+from shutil import make_archive
 from subprocess import check_call
-from unittest import TestCase
 from unittest import skipIf
 
 from warcio.warcwriter import WARCWriter
 
+try:
+    import pyspark
+except ImportError:
+    pyspark = None
+
 import mrjob
 from mrjob.cat import decompress
 from mrjob.examples.mr_phone_to_url import MRPhoneToURL
-from mrjob.launch import MRJobLauncher
+from mrjob.examples.mr_spark_wordcount import MRSparkWordcount
+from mrjob.examples.mr_spark_wordcount_script import MRSparkScriptWordcount
+from mrjob.examples.mr_sparkaboom import MRSparKaboom
 from mrjob.local import LocalMRJobRunner
 from mrjob.local import _sort_lines_in_memory
+from mrjob.parse import is_uri
 from mrjob.step import StepFailedException
 from mrjob.util import cmd_line
+from mrjob.util import safeeval
 from mrjob.util import to_lines
 
-import tests.sr_wc
 from tests.examples.test_mr_phone_to_url import write_conversion_record
 from tests.job import run_job
 from tests.mr_cmd_job import MRCmdJob
@@ -51,13 +59,15 @@ from tests.mr_exit_42_job import MRExit42Job
 from tests.mr_filter_job import MRFilterJob
 from tests.mr_group import MRGroup
 from tests.mr_job_where_are_you import MRJobWhereAreYou
+from tests.mr_just_a_jar import MRJustAJar
 from tests.mr_sort_and_group import MRSortAndGroup
+from tests.mr_spark_os_walk import MRSparkOSWalk
+from tests.mr_stdin_only import MRStdinOnly
 from tests.mr_two_step_job import MRTwoStepJob
 from tests.mr_word_count import MRWordCount
 from tests.py2 import call
 from tests.py2 import patch
-from tests.quiet import logger_disabled
-from tests.quiet import no_handlers_for_logger
+from tests.sandbox import BasicTestCase
 from tests.sandbox import EmptyMrjobConfTestCase
 from tests.sandbox import SandboxedTestCase
 from tests.sandbox import mrjob_conf_patcher
@@ -68,19 +78,6 @@ from tests.test_sim import SimRunnerNoMapperTestCase
 from tests.test_sim import SortValuesTestCase
 
 
-def _bash_wrap(cmd_str):
-    """Escape single quotes in a shell command string and wrap it with ``bash
-    -c '<string>'``.
-
-    This low-tech replacement works because we control the surrounding string
-    and single quotes are the only character in a single-quote string that
-    needs escaping.
-
-    .. deprecated:: 0.5.8
-    """
-    return "bash -c '%s'" % cmd_str.replace("'", "'\\''")
-
-
 class LocalMRJobRunnerEndToEndTestCase(SandboxedTestCase):
 
     def test_end_to_end(self):
@@ -172,8 +169,8 @@ class TestsToPort:
         input_gz.close()
 
         mr_job = MRTwoStepJob(['-r', 'local',
-                               '--jobconf=mapred.map.tasks=2',
-                               '--jobconf=mapred.reduce.tasks=2',
+                               '-D=mapred.map.tasks=2',
+                               '-D=mapred.reduce.tasks=2',
                                '-', input_path, input_gz_path])
         mr_job.sandbox(stdin=stdin)
 
@@ -374,7 +371,7 @@ class TimeoutException(Exception):
     pass
 
 
-class ExitWithoutExceptionTestCase(TestCase):
+class ExitWithoutExceptionTestCase(BasicTestCase):
 
     def test_exit_42_job(self):
         mr_job = MRExit42Job(['--no-conf', '--runner=local'])
@@ -392,8 +389,7 @@ class PythonBinTestCase(EmptyMrjobConfTestCase):
         # "echo" is a pretty poor substitute for Python, but it
         # should be available on most systems
         mr_job = MRTwoStepJob(
-            ['--python-bin', 'echo', '--steps-python-bin', sys.executable,
-             '--no-conf', '-r', 'local'])
+            ['--python-bin', 'echo', '--no-conf', '-r', 'local'])
         mr_job.sandbox()
 
         with mr_job.make_runner() as runner:
@@ -415,74 +411,28 @@ class PythonBinTestCase(EmptyMrjobConfTestCase):
                                '-r', 'local'])
         mr_job.sandbox(stdin=[b'bar\n'])
 
-        with no_handlers_for_logger():
-            with mr_job.make_runner() as runner:
-                runner.run()
-
-                # expect python -v crud in stderr
-
-                with open(runner._task_stderr_path('mapper', 0, 0)) as lines:
-                    self.assertTrue(any(
-                        'import mrjob' in line or  # Python 2
-                        "import 'mrjob'" in line
-                        for line in lines))
-
-                with open(runner._task_stderr_path('mapper', 0, 0)) as lines:
-                    self.assertTrue(any(
-                        '#' in line for line in lines))
-
-                # should still get expected results
-                self.assertEqual(
-                    sorted(to_lines(runner.cat_output())),
-                    sorted([b'1\tnull\n', b'1\t"bar"\n']))
-
-
-class StepsPythonBinTestCase(TestCase):
-
-    def test_echo_as_steps_python_bin(self):
-        mr_job = MRTwoStepJob(
-            ['--steps', '--steps-python-bin', 'echo', '--no-conf',
-             '-r', 'local'])
-        mr_job.sandbox()
-
         with mr_job.make_runner() as runner:
-            assert isinstance(runner, LocalMRJobRunner)
-            # MRTwoStepJob populates _steps in the runner, so un-populate
-            # it here so that the runner actually tries to get the steps
-            # via subprocess
-            runner._steps = None
-            self.assertRaises(ValueError, runner._get_steps)
+            runner.run()
 
-    def test_echo_as_steps_interpreter(self):
-        import logging
-        logging.basicConfig()
-        mr_job = MRTwoStepJob(
-            ['--steps', '--steps-interpreter', 'echo', '--no-conf', '-r',
-             'local'])
-        mr_job.sandbox()
+            # expect python -v crud in stderr
 
-        with mr_job.make_runner() as runner:
-            assert isinstance(runner, LocalMRJobRunner)
-            # MRTwoStepJob populates _steps in the runner, so un-populate
-            # it here so that the runner actually tries to get the steps
-            # via subprocess
-            runner._steps = None
-            self.assertRaises(ValueError, runner._get_steps)
+            with open(runner._task_stderr_path('mapper', 0, 0)) as lines:
+                self.assertTrue(any(
+                    'import mrjob' in line or  # Python 2
+                    "import 'mrjob'" in line
+                    for line in lines))
+
+            with open(runner._task_stderr_path('mapper', 0, 0)) as lines:
+                self.assertTrue(any(
+                    '#' in line for line in lines))
+
+            # should still get expected results
+            self.assertEqual(
+                sorted(to_lines(runner.cat_output())),
+                sorted([b'1\tnull\n', b'1\t"bar"\n']))
 
 
-class LocalBootstrapMrjobTestCase(TestCase):
-
-    def setUp(self):
-        self.make_tmp_dir()
-
-    def tearDown(self):
-        self.rm_tmp_dir()
-
-    def make_tmp_dir(self):
-        self.tmp_dir = tempfile.mkdtemp()
-
-    def rm_tmp_dir(self):
-        shutil.rmtree(self.tmp_dir)
+class LocalBootstrapMrjobTestCase(BasicTestCase):
 
     def test_loading_bootstrapped_mrjob_library(self):
         # track the dir we're loading mrjob from rather than the full path
@@ -501,11 +451,11 @@ class LocalBootstrapMrjobTestCase(TestCase):
 
                 runner.run()
 
-                output = list(to_lines(runner.cat_output()))
+                output = list(mr_job.parse_output((runner.cat_output())))
                 self.assertEqual(len(output), 1)
 
                 # script should load mrjob from its working dir
-                _, script_mrjob_dir = mr_job.parse_output_line(output[0])
+                _, script_mrjob_dir = output[0]
 
                 self.assertNotEqual(our_mrjob_dir, script_mrjob_dir)
                 self.assertTrue(script_mrjob_dir.startswith(local_tmp_dir))
@@ -522,8 +472,7 @@ class LocalBootstrapMrjobTestCase(TestCase):
                 self.assertEqual(runner._opts['bootstrap_mrjob'], False)
                 local_tmp_dir = os.path.realpath(runner._get_local_tmp_dir())
                 try:
-                    with no_handlers_for_logger():
-                        runner.run()
+                    runner.run()
                 except StepFailedException:
                     # this is what happens when mrjob isn't installed elsewhere
                     return
@@ -657,7 +606,7 @@ class CommandSubstepTestCase(SandboxedTestCase):
         with gzip.open(x_gz_path, 'wb') as x_gz:
             x_gz.write(b'x\nx\nx\nx\nx\nx\n')
 
-        reducer_cmd = _bash_wrap('wc -l | tr -Cd "[:digit:]"')
+        reducer_cmd = '/bin/sh -c \'wc -l | tr -Cd "[:digit:]"\''
         job = MRCmdJob([
             '--runner', 'local',
             '--mapper-cmd', 'cat -e',
@@ -689,7 +638,9 @@ class CommandSubstepTestCase(SandboxedTestCase):
         job.sandbox(stdin=BytesIO(data))
         with job.make_runner() as r:
             r.run()
-            self.assertEqual(sum(int(l) for l in to_lines(r.cat_output())), 3)
+            self.assertEqual(
+                sum(int(line) for line in to_lines(r.cat_output())),
+                3)
 
 
 class FilterTestCase(SandboxedTestCase):
@@ -801,7 +752,7 @@ class FilterTestCase(SandboxedTestCase):
             self.assertEqual(sorted(lines), [b'x$', b'y$', b'z$'])
 
 
-class SetupLineEncodingTestCase(TestCase):
+class SetupLineEncodingTestCase(BasicTestCase):
 
     def test_setup_wrapper_script_uses_local_line_endings(self):
         job = MRTwoStepJob(['-r', 'local', '--setup', 'true'])
@@ -811,13 +762,12 @@ class SetupLineEncodingTestCase(TestCase):
         # that use unix line endings anyway. So monitor open() instead
         with patch(
                 'mrjob.sim.open', create=True, side_effect=open) as m_open:
-            with logger_disabled('mrjob.local'):
-                with job.make_runner() as runner:
-                    runner.run()
+            with job.make_runner() as runner:
+                runner.run()
 
-                    self.assertIn(
-                        call(runner._setup_wrapper_script_path, 'w'),
-                        m_open.mock_calls)
+                self.assertIn(
+                    call(runner._setup_wrapper_script_path, 'w'),
+                    m_open.mock_calls)
 
 
 class LocalModeSortValuesTestCase(SortValuesTestCase):
@@ -917,6 +867,25 @@ class SortBinTestCase(SandboxedTestCase):
 
         self.assertEqual(sort_args[:2], ['sort', '-r'])
 
+    def test_empty_sort_bin_means_default(self):
+        job = MRGroup(['-r', 'local', '--sort-bin', ''])
+        job.sandbox(stdin=BytesIO(
+            b'apples\nbuffaloes\nbears'))
+
+        with job.make_runner() as runner:
+            runner.run()
+
+            self.assertEqual(
+                sorted(job.parse_output(runner.cat_output())),
+                [('a', ['apples']), ('b', ['buffaloes', 'bears'])])
+
+        self.assertTrue(self.check_call.called)
+        self.assertFalse(self._sort_lines_in_memory.called)
+
+        sort_args = self.check_call.call_args[0][0]
+        self.assertEqual(sort_args[:6],
+                         ['sort', '-t', '\t', '-k', '1,1', '-s'])
+
     def test_sort_in_memory_on_windows(self):
         self.start(patch('platform.system', return_value='Windows'))
 
@@ -996,26 +965,6 @@ class SortBinTestCase(SandboxedTestCase):
         self._test_environment_variables('--local-tmp-dir', tmp_dir)
 
 
-class InputFileArgsTestCase(SandboxedTestCase):
-    # test for #567: ensure that local runner doesn't need to pass
-    # file args to jobs
-
-    def test_no_file_args_required(self):
-        words1 = self.makefile('words1', b'kit and caboodle\n')
-        words2 = self.makefile('words2', b'baubles\nbangles and beads\n')
-
-        job = MRJobLauncher(
-            args=['-r', 'local', tests.sr_wc.__file__, words1, words2])
-        job.sandbox()
-
-        with job.make_runner() as runner:
-            runner.run()
-
-            lines = list(to_lines(runner.cat_output()))
-            self.assertEqual(len(lines), 1)
-            self.assertEqual(int(lines[0]), 7)
-
-
 class LocalInputManifestTestCase(InlineInputManifestTestCase):
 
     RUNNER = 'local'
@@ -1043,3 +992,201 @@ class LocalInputManifestTestCase(InlineInputManifestTestCase):
             self.EXPECTED_OUTPUT)
 
         self.assertTrue(exists(touched_path))
+
+
+class UnsupportedStepsTestCase(SandboxedTestCase):
+
+    def test_no_jar_steps(self):
+        jar_path = self.makefile('dora.jar')
+
+        job = MRJustAJar(['-r', 'local', '--jar', jar_path])
+        job.sandbox()
+
+        self.assertRaises(NotImplementedError, job.make_runner)
+
+
+class SparkMasterTestCase(SandboxedTestCase):
+
+    def test_default_spark_master(self):
+        runner = LocalMRJobRunner()
+
+        self.assertEqual(runner._spark_master(),
+                         'local-cluster[%d,1,1024]' % cpu_count())
+
+    def test_num_cores(self):
+        runner = LocalMRJobRunner(num_cores=3)
+
+        self.assertEqual(runner._spark_master(),
+                         'local-cluster[3,1,1024]')
+
+    def _test_spark_executor_memory(self, conf_value, megs):
+        runner = LocalMRJobRunner(
+            jobconf={'spark.executor.memory': conf_value})
+
+        self.assertEqual(runner._spark_master(),
+                         'local-cluster[%d,1,%d]' % (
+                             cpu_count(), megs))
+
+    def test_spark_exector_memory_2g(self):
+        self._test_spark_executor_memory('2g', 2048)
+
+    def test_spark_exector_memory_512m(self):
+        self._test_spark_executor_memory('512m', 512)
+
+    def test_spark_exector_memory_512M(self):
+        # test case-insensitivity
+        self._test_spark_executor_memory('512M', 512)
+
+    def test_spark_exector_memory_1t(self):
+        self._test_spark_executor_memory('1t', 1024 * 1024)
+
+    def test_spark_exector_memory_1_billion(self):
+        # should round up
+        self._test_spark_executor_memory('1000000000', 954)
+
+    def test_spark_exector_memory_640000k(self):
+        self._test_spark_executor_memory('640000k', 625)
+
+
+@skipIf(pyspark is None, 'no pyspark module')
+class LocalRunnerSparkTestCase(SandboxedTestCase):
+    # these tests are slow (~30s) because they run on
+    # actual Spark, in local-cluster mode
+
+    def test_spark_mrjob(self):
+        text = b'one fish\ntwo fish\nred fish\nblue fish\n'
+
+        job = MRSparkWordcount(['-r', 'local'])
+        job.sandbox(stdin=BytesIO(text))
+
+        counts = {}
+
+        with job.make_runner() as runner:
+            runner.run()
+
+            for line in to_lines(runner.cat_output()):
+                k, v = safeeval(line)
+                counts[k] = v
+
+        self.assertEqual(counts, dict(
+            blue=1, fish=4, one=1, red=1, two=1))
+
+    def test_spark_job_failure(self):
+        job = MRSparKaboom(['-r', 'local'])
+        job.sandbox(stdin=BytesIO(b'line\n'))
+
+        with job.make_runner() as runner:
+            self.assertRaises(StepFailedException, runner.run)
+
+    def test_spark_script_mrjob(self):
+        text = b'one fish\ntwo fish\nred fish\nblue fish\n'
+
+        job = MRSparkScriptWordcount(['-r', 'local'])
+        job.sandbox(stdin=BytesIO(text))
+
+        counts = {}
+
+        with job.make_runner() as runner:
+            runner.run()
+
+            for line in to_lines(runner.cat_output()):
+                k, v = safeeval(line)
+                counts[k] = v
+
+        self.assertEqual(counts, dict(
+            blue=1, fish=4, one=1, red=1, two=1))
+
+    def test_upload_files_with_rename(self):
+        fish_path = self.makefile('fish', b'salmon')
+        fowl_path = self.makefile('fowl', b'goose')
+
+        job = MRSparkOSWalk(['-r', 'local',
+                             '--files',
+                             '%s#ghoti,%s' % (fish_path, fowl_path)])
+        job.sandbox()
+
+        file_sizes = {}
+
+        with job.make_runner() as runner:
+            runner.run()
+
+            # check working dir mirror
+            wd_mirror = runner._wd_mirror()
+            self.assertIsNotNone(wd_mirror)
+            self.assertFalse(is_uri(wd_mirror))
+
+            self.assertTrue(os.path.exists(wd_mirror))
+            # only files which needed to be renamed should be in wd_mirror
+            self.assertTrue(os.path.exists(os.path.join(wd_mirror, 'ghoti')))
+            self.assertFalse(os.path.exists(os.path.join(wd_mirror, 'fish')))
+            self.assertFalse(os.path.exists(os.path.join(wd_mirror, 'fowl')))
+
+            for line in to_lines(runner.cat_output()):
+                path, size = safeeval(line)
+                file_sizes[path] = size
+
+        # check that files were uploaded to working dir
+        self.assertIn('fowl', file_sizes)
+        self.assertEqual(file_sizes['fowl'], 5)
+
+        self.assertIn('ghoti', file_sizes)
+        self.assertEqual(file_sizes['ghoti'], 6)
+
+        # fish was uploaded as "ghoti"
+        self.assertNotIn('fish', file_sizes)
+
+    def test_archive_emulation(self):
+        f_dir = self.makedirs('f')
+        self.makefile(join(f_dir, 'fish'), b'salmon')
+        self.makefile(join(f_dir, 'fowl'), b'goose')
+
+        f_tar_gz = make_archive(join(self.tmp_dir, 'f'), 'gztar', f_dir)
+
+        job = MRSparkOSWalk(['-r', 'local',
+                             '--archives', '%s#f-unpacked' % f_tar_gz,
+                             '--dirs', f_dir])
+        job.sandbox()
+
+        file_sizes = {}
+
+        with job.make_runner() as runner:
+            runner.run()
+
+            for line in to_lines(runner.cat_output()):
+                path, size = safeeval(line)
+                file_sizes[path] = size
+
+        self.assertIn('f/fish', file_sizes)
+        self.assertEqual(file_sizes['f/fish'], 6)
+        self.assertIn('f/fowl', file_sizes)
+        self.assertEqual(file_sizes['f/fowl'], 5)
+
+        self.assertIn('f-unpacked/fish', file_sizes)
+        self.assertEqual(file_sizes['f-unpacked/fish'], 6)
+        self.assertIn('f-unpacked/fowl', file_sizes)
+        self.assertEqual(file_sizes['f-unpacked/fowl'], 5)
+
+        # archives should have been uploaded as files
+        self.assertIn('f.tar.gz.file', file_sizes)
+        self.assertIn('f-1.tar.gz.file', file_sizes)
+
+    # TODO: add a Spark JAR to the repo, so we can test it
+
+
+class InputFileArgsTestCase(SandboxedTestCase):
+    # test for #567: ensure that local runner doesn't need to pass
+    # file args to jobs
+
+    def test_stdin_only(self):
+        input1 = self.makefile('input1', contents='cat cat cat cat cat')
+        input2 = self.makefile('input2', contents='dog dog dog')
+
+        job = MRStdinOnly(['-r', 'local', input1, input2])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            runner.run()
+
+            output = dict(job.parse_output(runner.cat_output()))
+
+            self.assertEqual(output, dict(cat=5, dog=3))
