@@ -1,6 +1,7 @@
 # Copyright 2009-2013 Yelp and Contributors
 # Copyright 2015 Yelp
 # Copyright 2017 Yelp
+# Copyright 2018 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +17,7 @@
 
 """Make sure all of our protocols work as advertised."""
 import unittest
-from unittest import TestCase
+from tests.sandbox import BasicTestCase
 from unittest import skipIf
 
 from mrjob.protocol import BytesProtocol
@@ -90,7 +91,7 @@ PICKLE_KEYS_AND_VALUES = REPR_KEYS_AND_VALUES + [
 ]
 
 
-class ProtocolTestCase(TestCase):
+class ProtocolTestCase(BasicTestCase):
 
     def assertRoundTripOK(self, protocol, key, value):
         """Assert that we can encode and decode the given key and value,
@@ -112,7 +113,7 @@ class ProtocolTestCase(TestCase):
         self.assertRaises(Exception, protocol.read, data)
 
 
-class JSONProtocolAliasesTestCase(TestCase):
+class JSONProtocolAliasesTestCase(BasicTestCase):
 
     def test_use_ujson_or_simplejson_if_installed(self):
         # these aliases are determined at compile time, so there
@@ -343,7 +344,7 @@ class PickleValueProtocolTestCase(ProtocolTestCase):
     # no tests of what encoded data looks like; pickle is an opaque protocol
 
 
-class RawProtocolAliasesTestCase(TestCase):
+class RawProtocolAliasesTestCase(BasicTestCase):
 
     def test_raw_protocol_aliases(self):
         if PY2:
