@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2015 Yelp
 # Copyright 2017 Yelp
+# Copyright 2018 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,12 +14,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from unittest import TestCase
-
 from mrjob.py2 import to_unicode
 
+from tests.sandbox import BasicTestCase
 
-class ToUnicodeTestCase(TestCase):
+
+class ToUnicodeTestCase(BasicTestCase):
 
     def test_None(self):
         self.assertRaises(TypeError, to_unicode, None)
