@@ -1,5 +1,5 @@
-# Copyright 2009-2017 Yelp
-# Copyright 2018 Yelp
+# Copyright 2009-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,10 +17,10 @@ import datetime
 import getpass
 import os
 import os.path
-import shutil
 import tarfile
+import tempfile
+from io import BytesIO
 from time import sleep
-from unittest import TestCase
 from unittest import skipIf
 
 import mrjob.conf
@@ -29,41 +29,43 @@ from mrjob.conf import dump_mrjob_conf
 from mrjob.emr import EMRJobRunner
 from mrjob.examples.mr_phone_to_url import MRPhoneToURL
 from mrjob.inline import InlineMRJobRunner
-from mrjob.py2 import StringIO
+from mrjob.job import MRJob
+from mrjob.local import LocalMRJobRunner
+from mrjob.parse import to_uri
+from mrjob.runner import MRJobRunner
+from mrjob.step import INPUT
+from mrjob.step import MRStep
+from mrjob.step import OUTPUT
 from mrjob.tools.emr.audit_usage import _JOB_KEY_RE
-from mrjob.util import log_to_stream
 from mrjob.util import to_lines
 
 from tests.mock_boto3 import MockBoto3TestCase
+from tests.mr_cmd_job import MRCmdJob
 from tests.mr_counting_job import MRCountingJob
+from tests.mr_just_a_jar import MRJustAJar
+from tests.mr_null_spark import MRNullSpark
+from tests.mr_spark_jar import MRSparkJar
+from tests.mr_spark_script import MRSparkScript
 from tests.mr_two_step_job import MRTwoStepJob
 from tests.mr_word_count import MRWordCount
 from tests.py2 import patch
-from tests.quiet import logger_disabled
-from tests.quiet import no_handlers_for_logger
+from tests.sandbox import BasicTestCase
 from tests.sandbox import EmptyMrjobConfTestCase
 from tests.sandbox import SandboxedTestCase
 from tests.sandbox import mrjob_conf_patcher
 
 
-class WithStatementTestCase(TestCase):
-
-    def setUp(self):
-        self.local_tmp_dir = None
-
-    def tearDown(self):
-        if self.local_tmp_dir:
-            shutil.rmtree(self.local_tmp_dir)
-            self.local_tmp_dir = None
+class WithStatementTestCase(BasicTestCase):
 
     def _test_cleanup_after_with_statement(self, mode, should_exist):
-        with InlineMRJobRunner(cleanup=mode, conf_paths=[]) as runner:
-            self.local_tmp_dir = runner._get_local_tmp_dir()
-            self.assertTrue(os.path.exists(self.local_tmp_dir))
+        local_tmp_dir = None
 
-        self.assertEqual(os.path.exists(self.local_tmp_dir), should_exist)
-        if not should_exist:
-            self.local_tmp_dir = None
+        with InlineMRJobRunner(cleanup=mode, conf_paths=[]) as runner:
+            local_tmp_dir = runner._get_local_tmp_dir()
+            self.assertTrue(os.path.exists(local_tmp_dir))
+
+        # leaving the with: block activates cleanup
+        self.assertEqual(os.path.exists(local_tmp_dir), should_exist)
 
     def test_cleanup_all(self):
         self._test_cleanup_after_with_statement(['ALL'], False)
@@ -90,9 +92,11 @@ class WithStatementTestCase(TestCase):
         self._test_cleanup_after_with_statement(['NONE', 'NONE'], True)
 
 
-class TestJobName(TestCase):
+class TestJobName(BasicTestCase):
 
     def setUp(self):
+        super(TestJobName, self).setUp()
+
         self.blank_out_environment()
         self.monkey_patch_getuser()
 
@@ -100,6 +104,8 @@ class TestJobName(TestCase):
         self.restore_getuser()
         self.restore_environment()
 
+        super(TestJobName, self).tearDown()
+
     def blank_out_environment(self):
         self._old_environ = os.environ.copy()
         # don't do os.environ = {}! This won't actually set environment
@@ -191,54 +197,88 @@ class TestJobName(TestCase):
 
 class TestCatOutput(SandboxedTestCase):
 
-    # Test regression for #269
-    def test_cat_output(self):
-        a_dir_path = os.path.join(self.tmp_dir, 'a')
-        b_dir_path = os.path.join(self.tmp_dir, 'b')
-        l_dir_path = os.path.join(self.tmp_dir, '_logs')
-        os.mkdir(a_dir_path)
-        os.mkdir(b_dir_path)
-        os.mkdir(l_dir_path)
+    def setUp(self):
+        super(TestCatOutput, self).setUp()
 
-        a_file_path = os.path.join(a_dir_path, 'part-00000')
-        b_file_path = os.path.join(b_dir_path, 'part-00001')
-        c_file_path = os.path.join(self.tmp_dir, 'part-00002')
-        x_file_path = os.path.join(l_dir_path, 'log.xml')
-        y_file_path = os.path.join(self.tmp_dir, '_SUCCESS')
+        self.output_dir = os.path.join(self.tmp_dir, 'job_output')
+        os.mkdir(self.output_dir)
 
-        with open(a_file_path, 'w') as f:
-            f.write('A')
+        self.runner = InlineMRJobRunner(
+            conf_paths=[], output_dir=self.output_dir)
 
-        with open(b_file_path, 'w') as f:
-            f.write('B')
+    def test_empty(self):
+        self.assertEqual(list(self.runner.cat_output()), [])
 
-        with open(c_file_path, 'w') as f:
-            f.write('C')
+    def test_typical_output(self):
+        # actual output
+        self.makefile(os.path.join(self.output_dir, 'part-00000'),
+                      b'line0\n')
+        self.makefile(os.path.join(self.output_dir, 'part-00001'),
+                      b'line1\n')
 
-        with open(x_file_path, 'w') as f:
-            f.write('<XML XML XML/>')
+        # hidden .crc file
+        self.makefile(os.path.join(self.output_dir, '.crc.part-00000'),
+                      b'42\n')
 
-        with open(y_file_path, 'w') as f:
-            f.write('I win')
+        # hidden _SUCCESS file (ignore)
+        self.makefile(os.path.join(self.output_dir, '_SUCCESS'),
+                      b'such a relief!\n')
+
+        # hidden _logs dir
+        self.makefile(os.path.join(self.output_dir, '_logs', 'log.xml'),
+                      b'pretty much the usual\n')
+
+        self.assertEqual(sorted(to_lines(self.runner.cat_output())),
+                         [b'line0\n', b'line1\n'])
+
+    def test_output_in_subdirs(self):
+        # test for output being placed in subdirs, for example with nicknack
+        self.makefile(os.path.join(self.output_dir, 'a', 'part-00000'),
+                      b'line-a0\n')
+        self.makefile(os.path.join(self.output_dir, 'a', 'part-00001'),
+                      b'line-a1\n')
+
+        self.makefile(os.path.join(self.output_dir, 'b', 'part-00000'),
+                      b'line-b0\n')
+
+        self.makefile(os.path.join(self.output_dir, 'b', '.crc.part-00000'),
+                      b'42\n')
+
+        self.assertEqual(sorted(to_lines(self.runner.cat_output())),
+                         [b'line-a0\n', b'line-a1\n', b'line-b0\n'])
+
+    def test_read_all_non_hidden_files(self):
+        self.makefile(os.path.join(self.output_dir, 'baz'),
+                      b'qux\n')
+
+        self.makefile(os.path.join(self.output_dir, 'foo', 'bar'),
+                      b'baz\n')
+
+        self.assertEqual(sorted(to_lines(self.runner.cat_output())),
+                         [b'baz\n', b'qux\n'])
+
+    def test_empty_string_between_files(self):
+        self.makefile(os.path.join(self.output_dir, 'part-00000'), b'A')
+        self.makefile(os.path.join(self.output_dir, 'part-00001'), b'\n')
+        self.makefile(os.path.join(self.output_dir, 'part-00002'), b'C')
+
+        # order isn't guaranteed, but there should be 3 chunks separated
+        # by two empty strings
+        chunks = list(self.runner.cat_output())
+        self.assertEqual(len(chunks), 5)
+        self.assertEqual(chunks[1], b'')
+        self.assertEqual(chunks[3], b'')
+
+    def test_output_dir_not_considered_hidden(self):
+        output_dir = os.path.join(self.tmp_dir, '_hidden', '_output_dir')
+
+        self.makefile(os.path.join(output_dir, 'part-00000'),
+                      b'cats\n')
+
+        runner = InlineMRJobRunner(conf_paths=[], output_dir=output_dir)
 
-        runner = InlineMRJobRunner(conf_paths=[], output_dir=self.tmp_dir)
         self.assertEqual(sorted(to_lines(runner.cat_output())),
-                         [b'A', b'B', b'C'])
-
-    def test_deprecated_stream_output(self):
-        self.makefile('part-00000', contents=b'1\n2')
-        self.makefile('part-00001', contents=b'3\n4\n')
-
-        runner = InlineMRJobRunner(conf_paths=[], output_dir=self.tmp_dir)
-
-        log = self.start(patch('mrjob.runner.log'))
-
-        # should group output into lines, but not join across files
-        self.assertEqual(sorted(runner.stream_output()),
-                         [b'1\n', b'2', b'3\n', b'4\n'])
-
-        # should issue deprecation warning
-        self.assertEqual(log.warning.call_count, 1)
+                         [b'cats\n'])
 
 
 class CheckInputPathsTestCase(SandboxedTestCase):
@@ -269,7 +309,7 @@ class CheckInputPathsTestCase(SandboxedTestCase):
             self.assertRaises(StopIteration, runner.run)
 
     def test_stdin_is_fine(self):
-        job = MRWordCount()
+        job = MRWordCount([])
         job.sandbox()
 
         with job.make_runner() as runner:
@@ -283,7 +323,7 @@ class CheckInputPathsTestCase(SandboxedTestCase):
             runner.run()
 
     def test_check_input_paths_enabled_by_default(self):
-        job = MRWordCount()
+        job = MRWordCount([])
         with job.make_runner() as runner:
             self.assertTrue(runner._opts['check_input_paths'])
 
@@ -293,7 +333,7 @@ class CheckInputPathsTestCase(SandboxedTestCase):
             self.assertFalse(runner._opts['check_input_paths'])
 
     def test_can_disable_check_input_paths_in_config(self):
-        job = MRWordCount()
+        job = MRWordCount([])
         with mrjob_conf_patcher(
                 {'runners': {'inline': {'check_input_paths': False}}}):
             with job.make_runner() as runner:
@@ -303,7 +343,7 @@ class CheckInputPathsTestCase(SandboxedTestCase):
 class ClosedRunnerTestCase(EmptyMrjobConfTestCase):
 
     def test_job_closed_on_cleanup(self):
-        job = MRWordCount()
+        job = MRWordCount([])
         with job.make_runner() as runner:
             # do nothing
             self.assertFalse(runner._closed)
@@ -378,6 +418,47 @@ class StepInputAndOutputURIsTestCase(SandboxedTestCase):
             output_uri_2 = runner._step_output_uri(2)
             self.assertEqual(output_uri_2, 'hdfs:///tmp/output')
 
+    def test_local_output_dir_and_step_output_dir(self):
+        input1_path = self.makefile('input1')
+        input2_path = self.makefile('input2')
+
+        output_dir = self.makedirs('output')
+        step_output_dir = self.makedirs('step_output')
+
+        # this has three steps, which lets us test step numbering
+        job = MRCountingJob([
+            '-r', 'local',
+            '--output-dir', output_dir,
+            '--step-output-dir', step_output_dir,
+            input1_path, input2_path])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(runner._num_steps(), 3)
+
+            input_uris_0 = runner._step_input_uris(0)
+            self.assertEqual([os.path.basename(uri) for uri in input_uris_0],
+                             ['input1', 'input2'])
+            self.assertEqual([uri[:8] for uri in input_uris_0],
+                             ['file:///', 'file:///'])
+
+            output_uri_0 = runner._step_output_uri(0)
+            self.assertEqual(output_uri_0,
+                             to_uri(os.path.join(step_output_dir, '0000')))
+
+            input_uris_1 = runner._step_input_uris(1)
+            self.assertEqual(input_uris_1, [output_uri_0])
+
+            output_uri_1 = runner._step_output_uri(1)
+            self.assertEqual(output_uri_1,
+                             to_uri(os.path.join(step_output_dir, '0001')))
+
+            input_uris_2 = runner._step_input_uris(2)
+            self.assertEqual(input_uris_2, [output_uri_1])
+
+            output_uri_2 = runner._step_output_uri(2)
+            self.assertEqual(output_uri_2, to_uri(output_dir))
+
 
 class DirArchivePathTestCase(SandboxedTestCase):
 
@@ -544,7 +625,7 @@ class RemoteCreateDirArchiveTestCase(MockBoto3TestCase):
 
 class ConfigFilesTestCase(SandboxedTestCase):
 
-    MRJOB_CONF_CONTENTS = None  # don't patch load_opts_from_mrjob_confsq
+    MRJOB_CONF_CONTENTS = None  # don't patch load_opts_from_mrjob_confs
 
     def save_conf(self, name, conf):
         conf_path = os.path.join(self.tmp_dir, name)
@@ -655,17 +736,18 @@ class MultipleConfigFilesValuesTestCase(ConfigFilesTestCase):
 
 class MultipleConfigFilesMachineryTestCase(ConfigFilesTestCase):
 
+    def setUp(self):
+        super(MultipleConfigFilesMachineryTestCase, self).setUp()
+        self.log = self.start(patch('mrjob.conf.log'))
+
     def test_empty_runner_error(self):
         conf = dict(runner=dict(local=dict(local_tmp_dir='/tmp')))
         path = self.save_conf('basic', conf)
 
-        stderr = StringIO()
-        with no_handlers_for_logger():
-            log_to_stream('mrjob.conf', stderr)
-            InlineMRJobRunner(conf_paths=[path])
-            self.assertEqual(
-                "No configs specified for inline runner\n",
-                stderr.getvalue())
+        InlineMRJobRunner(conf_paths=[path])
+
+        self.log.warning.assert_called_once_with(
+            'No configs specified for inline runner')
 
     def test_conf_contain_only_include_file(self):
         """If a config file only include other configuration files
@@ -701,13 +783,8 @@ class MultipleConfigFilesMachineryTestCase(ConfigFilesTestCase):
         }
         path = self.save_conf('twoincludefiles', conf)
 
-        stderr = StringIO()
-        with no_handlers_for_logger():
-            log_to_stream('mrjob.conf', stderr)
-            InlineMRJobRunner(conf_paths=[path])
-            self.assertEqual(
-                "",
-                stderr.getvalue())
+        InlineMRJobRunner(conf_paths=[path])
+        self.assertFalse(self.log.called)
 
 
 class MultipleMultipleConfigFilesTestCase(ConfigFilesTestCase):
@@ -715,7 +792,7 @@ class MultipleMultipleConfigFilesTestCase(ConfigFilesTestCase):
     BASE_CONFIG_LEFT = {
         'runners': {
             'inline': {
-                'jobconf': dict(from_left=1, from_both=1),
+                'jobconf': dict(from_left='one', from_both='one'),
                 'label': 'i_dont_like_to_be_labelled',
             }
         }
@@ -724,7 +801,7 @@ class MultipleMultipleConfigFilesTestCase(ConfigFilesTestCase):
     BASE_CONFIG_RIGHT = {
         'runners': {
             'inline': {
-                'jobconf': dict(from_right=2, from_both=2),
+                'jobconf': dict(from_right='two', from_both='two'),
                 'owner': 'ownership_is_against_my_principles'
             }
         }
@@ -736,12 +813,15 @@ class MultipleMultipleConfigFilesTestCase(ConfigFilesTestCase):
         opts_both = self.opts_for_conf('both.conf',
                                        dict(include=[path_left, path_right]))
 
-        self.assertEqual(opts_both['jobconf'],
-                         dict(from_left=1, from_both=2, from_right=2))
-        self.assertEqual(opts_both['label'],
-                         'i_dont_like_to_be_labelled')
-        self.assertEqual(opts_both['owner'],
-                         'ownership_is_against_my_principles')
+        self.assertEqual(
+            opts_both['jobconf'],
+            dict(from_left='one', from_both='two', from_right='two'))
+        self.assertEqual(
+            opts_both['label'],
+            'i_dont_like_to_be_labelled')
+        self.assertEqual(
+            opts_both['owner'],
+            'ownership_is_against_my_principles')
 
     def test_multiple_configs_via_runner_args(self):
         path_left = self.save_conf('left.conf', self.BASE_CONFIG_LEFT)
@@ -749,8 +829,9 @@ class MultipleMultipleConfigFilesTestCase(ConfigFilesTestCase):
 
         runner = InlineMRJobRunner(conf_paths=[path_left, path_right])
 
-        self.assertEqual(runner._opts['jobconf'],
-                         dict(from_left=1, from_both=2, from_right=2))
+        self.assertEqual(
+            runner._opts['jobconf'],
+            dict(from_left='one', from_both='two', from_right='two'))
 
 
 @skipIf(mrjob.conf.yaml is None, 'no yaml module')
@@ -849,58 +930,32 @@ class TestExtraKwargs(ConfigFilesTestCase):
         self.path = self.save_conf('config', self.CONFIG)
 
     def test_extra_kwargs_in_mrjob_conf_okay(self):
-        with logger_disabled('mrjob.runner'):
-            runner = InlineMRJobRunner(conf_paths=[self.path])
-            self.assertEqual(runner._opts['setup'], ['echo foo'])
-            self.assertNotIn('qux', runner._opts)
+        runner = InlineMRJobRunner(conf_paths=[self.path])
+        self.assertEqual(runner._opts['setup'], ['echo foo'])
+        self.assertNotIn('qux', runner._opts)
 
     def test_extra_kwargs_passed_in_directly_okay(self):
-        with logger_disabled('mrjob.runner'):
-            runner = InlineMRJobRunner(
-                foo='bar',
-                local_tmp_dir='/var/tmp',
-                conf_paths=[],
-            )
+        runner = InlineMRJobRunner(
+            foo='bar',
+            local_tmp_dir='/var/tmp',
+            conf_paths=[],
+        )
 
-            self.assertEqual(runner._opts['local_tmp_dir'], '/var/tmp')
-            self.assertNotIn('bar', runner._opts)
+        self.assertEqual(runner._opts['local_tmp_dir'], '/var/tmp')
+        self.assertNotIn('bar', runner._opts)
 
 
 class OptDebugPrintoutTestCase(ConfigFilesTestCase):
 
     def test_option_debug_printout(self):
-        stderr = StringIO()
+        log = self.start(patch('mrjob.runner.log'))
 
-        with no_handlers_for_logger():
-            log_to_stream('mrjob.runner', stderr, debug=True)
+        InlineMRJobRunner(owner='dave')
 
-            InlineMRJobRunner(owner='dave')
+        debug = ''.join(a[0] + '\n' for a, kw in log.debug.call_args_list)
 
-        self.assertIn("'owner'", stderr.getvalue())
-        self.assertIn("'dave'", stderr.getvalue())
-
-
-class DeprecatedFileUploadArgsTestCase(SandboxedTestCase):
-
-    def setUp(self):
-        super(DeprecatedFileUploadArgsTestCase, self).setUp()
-
-        self.log = self.start(patch('mrjob.runner.log'))
-
-    def test_deprecated_file_upload_args(self):
-        old_runner = InlineMRJobRunner(
-            file_upload_args=[('--foo-config', '/tmp/.fooconf#dot-fooconf')])
-
-        new_runner = InlineMRJobRunner(
-            extra_args=['--foo-config', dict(
-                path='/tmp/.fooconf', name='dot-fooconf', type='file')])
-
-        self.assertEqual(old_runner._extra_args, new_runner._extra_args)
-        self.assertEqual(old_runner._spark_files, new_runner._spark_files)
-        self.assertEqual(old_runner._working_dir_mgr._name_to_typed_path,
-                         new_runner._working_dir_mgr._name_to_typed_path)
-
-        self.assertTrue(self.log.warning.called)
+        self.assertIn("'owner'", debug)
+        self.assertIn("'dave'", debug)
 
 
 # job that improperly uses mapper_raw on a step other than the first
@@ -914,5 +969,352 @@ class InputManifestTestCase(SandboxedTestCase):
     def test_only_first_step_can_use_mapper_raw(self):
         job = MRPhoneToURLToPhoneToURL('')
 
+        self.assertRaises(ValueError, job.make_runner)
+
+
+class LocalTmpDirTestCase(SandboxedTestCase):
+
+    def make_runner(self, *args):
+        mr_job = MRWordCount(args)
+        mr_job.sandbox()
+        return mr_job.make_runner()
+
+    def assert_local_tmp_in(self, runner, path):
+        self.assertEqual(
+            runner._get_local_tmp_dir(),
+            os.path.join(path, runner._job_key))
+
+    def test_default(self):
+        with self.make_runner() as runner:
+            self.assert_local_tmp_in(runner, tempfile.gettempdir())
+
+    def test_mrjob_conf(self):
+        self.start(mrjob_conf_patcher(
+            dict(runners=dict(inline=dict(
+                local_tmp_dir=self.tmp_dir)))))
+
+        with self.make_runner() as runner:
+            self.assert_local_tmp_in(runner, self.tmp_dir)
+
+    def test_blank_local_tmp_dir_means_default(self):
+        self.start(mrjob_conf_patcher(
+            dict(runners=dict(inline=dict(
+                local_tmp_dir='')))))
+
+        with self.make_runner() as runner:
+            self.assert_local_tmp_in(runner, tempfile.gettempdir())
+
+    def test_command_line_switch(self):
+        with self.make_runner(
+                '--local-tmp-dir', self.tmp_dir) as runner:
+            self.assert_local_tmp_in(runner, self.tmp_dir)
+
+    def test_command_line_can_blank_out_conf(self):
+        self.start(mrjob_conf_patcher(
+            dict(runners=dict(inline=dict(
+                local_tmp_dir=self.tmp_dir)))))
+
+        with self.make_runner('--local-tmp-dir', '') as runner:
+            self.assert_local_tmp_in(runner, tempfile.gettempdir())
+
+
+class MRCatsJob(MRJob):
+    # used below to test when passthru args are not needed
+
+    def configure_args(self):
+        super(MRCatsJob, self).configure_args()
+
+        self.arg_parser.add_argument(
+            '--num-cats', dest='num_cats', type=int, default=1)
+
+    def steps(self):
+        return [MRStep(mapper_cmd='cat')] * self.options.num_cats
+
+
+class PassStepsToRunnerTestCase(BasicTestCase):
+
+    def setUp(self):
+        super(PassStepsToRunnerTestCase, self).setUp()
+        self.log = self.start(patch('mrjob.runner.log'))
+
+    def test_job_passes_in_steps(self):
+        job = MRWordCount([])
+        job.sandbox()
+
         with job.make_runner() as runner:
-            self.assertRaises(ValueError, runner._get_steps)
+            self.assertTrue(runner._steps)
+
+            runner.run()
+
+            self.assertFalse(self.log.warning.called)
+
+    def test_command_steps(self):
+        job = MRCatsJob(['-r', 'local', '--num-cats', '3'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(len(runner._steps), 3)
+
+            runner.run()
+
+            self.assertFalse(self.log.warning.called)
+
+    def test_no_steps(self):
+        job = MRJob([])
+        job.sandbox()
+
+        # it's possible to make a runner with the base MRJob, but it has
+        # no steps
+        with job.make_runner() as runner:
+            self.assertEqual(runner._steps, [])
+
+            self.assertRaises(ValueError, runner.run)
+
+    def test_no_script_and_no_steps(self):
+        runner = InlineMRJobRunner()
+
+        self.assertEqual(runner._script_path, None)
+        self.assertEqual(runner._steps, [])
+
+        self.assertRaises(ValueError, runner.run)
+
+        self.assertFalse(self.log.warning.called)
+
+
+class TestStepsWithoutMRJobScript(MockBoto3TestCase):
+
+    def test_classic_streaming_step_without_mr_job_script(self):
+        # classic MRJob mappers and reducers require a MRJob script
+        steps = MRWordCount([])._steps_desc()
+
+        self.assertRaises(ValueError,
+                          LocalMRJobRunner,
+                          steps=steps, stdin=BytesIO(b'one\ntwo\n'))
+
+    def test_command_streaming_step_without_mr_job_script(self):
+        # you don't need a script to run commands
+        steps = MRCmdJob(['--mapper-cmd', 'cat'])._steps_desc()
+
+        runner = LocalMRJobRunner(steps=steps, stdin=BytesIO(b'dog\n'))
+
+        runner.run()
+        runner.cleanup()
+
+    def test_jar_step_without_mr_job_script(self):
+        jar_path = self.makefile('dora.jar')
+        steps = MRJustAJar(['--jar', jar_path])._steps_desc()
+
+        runner = EMRJobRunner(steps=steps, stdin=BytesIO(b'backpack'))
+
+        runner.run()
+        runner.cleanup()
+
+    def test_spark_step_without_mr_job_script(self):
+        steps = MRNullSpark([])._steps_desc()
+
+        # need to be able to call the script's spark() method
+        self.assertRaises(ValueError,
+                          EMRJobRunner, steps=steps, stdin=BytesIO())
+
+    def test_spark_jar_step_without_mr_job_script(self):
+        spark_jar_path = self.makefile('fireflies.jar')
+        steps = MRSparkJar(['--jar', spark_jar_path])._steps_desc()
+
+        runner = EMRJobRunner(steps=steps, stdin=BytesIO())
+
+        runner.run()
+        runner.cleanup()
+
+    def test_spark_script_step_without_mr_job_script(self):
+        spark_script_path = self.makefile('a_spark_script.py')
+        steps = MRSparkScript(['--script', spark_script_path])._steps_desc()
+
+        runner = EMRJobRunner(steps=steps, stdin=BytesIO())
+
+        runner.run()
+        runner.cleanup()
+
+
+class UnsupportedStepsTestCase(MockBoto3TestCase):
+
+    def test_base_classes_cant_have_steps(self):
+        steps = MRTwoStepJob([])._steps_desc()
+
+        self.assertRaises(NotImplementedError, MRJobRunner, steps=steps)
+
+    def test_unknown_step_type(self):
+        steps = [dict(type='cameloop')]  # great name for your ML startup?
+
+        self.assertRaises(NotImplementedError, EMRJobRunner, steps=steps)
+
+    def test_malformed_step(self):
+        steps = [dict(foo='bar')]
+
+        self.assertRaises(NotImplementedError, EMRJobRunner, steps=steps)
+
+
+class UnexpectedOptsWarningTestCase(SandboxedTestCase):
+
+    MRJOB_CONF_CONTENTS = None  # don't patch load_opts_from_mrjob_confs
+
+    def setUp(self):
+        super(UnexpectedOptsWarningTestCase, self).setUp()
+
+        self.log = self.start(patch('mrjob.runner.log'))
+
+    def test_no_warning_by_default(self):
+        job = MRTwoStepJob(['-r', 'local', '--no-conf'])
+        job.sandbox()
+
+        with job.make_runner():
+            self.assertFalse(self.log.warning.called)
+
+    def test_unexpected_opt_from_mrjob_conf(self):
+        conf_path = self.makefile('mrjob.custom.conf')
+
+        with open(conf_path, 'w') as f:
+            dump_mrjob_conf(
+                dict(runners=dict(local=dict(land='useless_swamp'))), f)
+
+        job = MRTwoStepJob(['-r', 'local', '-c', conf_path])
+        job.sandbox()
+
+        with job.make_runner():
+            self.assertTrue(self.log.warning.called)
+            warnings = '\n'.join(
+                arg[0][0] for arg in self.log.warning.call_args_list)
+
+            self.assertIn('Unexpected option', warnings)
+            self.assertIn('land', warnings)
+            self.assertIn(conf_path, warnings)
+
+    def test_unexpected_opt_from_command_line(self):
+        # regression test for #1898. local runner doesn't support *zone*
+        job = MRTwoStepJob(['-r', 'local', '--no-conf', '--zone', 'DANGER'])
+        job.sandbox()
+
+        with job.make_runner():
+            self.assertTrue(self.log.warning.called)
+            warnings = '\n'.join(
+                arg[0][0] for arg in self.log.warning.call_args_list)
+
+            self.assertIn('Unexpected option', warnings)
+            self.assertIn('zone', warnings)
+            self.assertIn('command line', warnings)
+
+
+class SparkScriptArgsTestCase(SandboxedTestCase):
+
+    def setUp(self):
+        super(SparkScriptArgsTestCase, self).setUp()
+
+        # don't bother with actual input/output URIs, which
+        # are tested elsewhere
+        def mock_interpolate_step_args(args, step_num):
+            def interpolate(arg):
+                if arg == INPUT:
+                    return '<step %d input>' % step_num
+                elif arg == OUTPUT:
+                    return '<step %d output>' % step_num
+                else:
+                    return arg
+
+            return [interpolate(arg) for arg in args]
+
+        self.start(patch(
+            'mrjob.bin.MRJobRunner._interpolate_step_args',
+            side_effect=mock_interpolate_step_args))
+
+        self.start(patch(
+            'mrjob.inline.InlineMRJobRunner._STEP_TYPES',
+            {'spark', 'spark_jar', 'spark_script', 'streaming'}))
+
+    def test_spark_mr_job(self):
+        job = MRNullSpark([])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['--step-num=0',
+                 '--spark',
+                 '<step 0 input>',
+                 '<step 0 output>'])
+
+    def test_spark_passthrough_arg(self):
+        job = MRNullSpark(['--extra-spark-arg=--verbose'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['--step-num=0',
+                 '--spark',
+                 '--extra-spark-arg=--verbose',
+                 '<step 0 input>',
+                 '<step 0 output>'])
+
+    def test_spark_file_arg(self):
+        foo_path = self.makefile('foo')
+
+        job = MRNullSpark(['--extra-file', foo_path])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['--step-num=0',
+                 '--spark',
+                 '--extra-file',
+                 'foo',
+                 '<step 0 input>',
+                 '<step 0 output>'])
+
+            name_to_path = runner._working_dir_mgr.name_to_path('file')
+            self.assertIn('foo', name_to_path)
+            self.assertEqual(name_to_path['foo'], foo_path)
+
+    def test_spark_jar(self):
+        job = MRSparkJar(['--jar-arg', 'foo', '--jar-arg', 'bar'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['foo', 'bar'])
+
+    def test_spark_jar_interpolation(self):
+        job = MRSparkJar(['--jar-arg', OUTPUT, '--jar-arg', INPUT])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['<step 0 output>', '<step 0 input>'])
+
+    def test_spark_script(self):
+        job = MRSparkScript(['--script-arg', 'foo', '--script-arg', 'bar'])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['foo', 'bar'])
+
+    def test_spark_script_interpolation(self):
+        job = MRSparkScript(['--script-arg', OUTPUT, '--script-arg', INPUT])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertEqual(
+                runner._spark_script_args(0),
+                ['<step 0 output>', '<step 0 input>'])
+
+    def test_streaming_step_not_okay(self):
+        job = MRTwoStepJob([])
+        job.sandbox()
+
+        with job.make_runner() as runner:
+            self.assertRaises(
+                TypeError,
+                runner._spark_script_args, 0)
