@@ -1,7 +1,7 @@
 # Copyright 2012 Yelp
 # Copyright 2013 David Marin
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import os
-from unittest import TestCase
 
 from mrjob.setup import UploadDirManager
 from mrjob.setup import WorkingDirManager
@@ -24,9 +23,10 @@ from mrjob.setup import parse_legacy_hash_path
 from mrjob.setup import parse_setup_cmd
 
 from tests.py2 import patch
+from tests.sandbox import BasicTestCase
 
 
-class ParseSetupCmdTestCase(TestCase):
+class ParseSetupCmdTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(parse_setup_cmd(''), [])
@@ -188,7 +188,7 @@ class ParseSetupCmdTestCase(TestCase):
             ValueError, parse_setup_cmd, 'foo\\')
 
 
-class ParseLegacyHashPathTestCase(TestCase):
+class ParseLegacyHashPathTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertRaises(ValueError, parse_legacy_hash_path, 'archive', '')
@@ -259,7 +259,7 @@ class ParseLegacyHashPathTestCase(TestCase):
             parse_legacy_hash_path, 'file', 'foo#bar/baz')
 
 
-class NameUniquelyTestCase(TestCase):
+class NameUniquelyTestCase(BasicTestCase):
 
     def test_no_empty_names(self):
         self.assertEqual(name_uniquely(''), '1')
@@ -354,7 +354,7 @@ class NameUniquelyTestCase(TestCase):
             name_uniquely(os.path.join('foo', 'bar', '')), 'bar')
 
 
-class UploadDirManagerTestCase(TestCase):
+class UploadDirManagerTestCase(BasicTestCase):
 
     def test_empty(self):
         sd = UploadDirManager('hdfs:///')
@@ -440,25 +440,40 @@ class UploadDirManagerTestCase(TestCase):
                           '._foo': 'hdfs:///foo'})
 
 
-class WorkingDirManagerTestCase(TestCase):
+class WorkingDirManagerTestCase(BasicTestCase):
 
     def test_empty(self):
         wd = WorkingDirManager()
         self.assertEqual(wd.name_to_path('archive'), {})
         self.assertEqual(wd.name_to_path('file'), {})
+        self.assertEqual(wd.name_to_path(), {})
         self.assertEqual(wd.paths(), set())
+        self.assertEqual(wd.paths('archive'), set())
+        self.assertEqual(wd.paths('file'), set())
 
-    def test_simple(self):
+    def test_basic(self):
         wd = WorkingDirManager()
         wd.add('archive', 's3://bucket/path/to/baz.tar.gz')
         wd.add('file', 'foo/bar.py')
         self.assertEqual(wd.name_to_path('file'),
                          {'bar.py': 'foo/bar.py'})
         self.assertEqual(wd.name_to_path('archive'),
+                         {'baz': 's3://bucket/path/to/baz.tar.gz'})
+        self.assertEqual(wd.name_to_path('archive_file'),
                          {'baz.tar.gz': 's3://bucket/path/to/baz.tar.gz'})
+        self.assertEqual(wd.name_to_path(),
+                         {'bar.py': 'foo/bar.py',
+                          'baz': 's3://bucket/path/to/baz.tar.gz',
+                          'baz.tar.gz': 's3://bucket/path/to/baz.tar.gz'})
         self.assertEqual(
             wd.paths(),
-            set(['foo/bar.py', 's3://bucket/path/to/baz.tar.gz']))
+            {'foo/bar.py', 's3://bucket/path/to/baz.tar.gz'})
+        self.assertEqual(
+            wd.paths('archive'), {'s3://bucket/path/to/baz.tar.gz'})
+        self.assertEqual(
+            wd.paths('archive_file'), {'s3://bucket/path/to/baz.tar.gz'})
+        self.assertEqual(
+            wd.paths('file'), {'foo/bar.py'})
 
     def test_explicit_name_collision(self):
         wd = WorkingDirManager()
@@ -479,7 +494,7 @@ class WorkingDirManagerTestCase(TestCase):
         self.assertEqual(wd.name_to_path('file'),
                          {'qux.py': 'foo/bar.py',
                           'bar.py': 'foo/bar.py'})
-        self.assertEqual(wd.paths(), set(['foo/bar.py']))
+        self.assertEqual(wd.paths(), {'foo/bar.py'})
 
     def test_cant_give_same_path_different_types(self):
         wd = WorkingDirManager()
@@ -524,5 +539,15 @@ class WorkingDirManagerTestCase(TestCase):
         wd.add('archive', '_foo.tar.gz')
         wd.add('file', '.bazrc')
 
-        self.assertEqual(wd.name('archive', '_foo.tar.gz'), '_foo.tar.gz')
+        self.assertEqual(wd.name('archive', '_foo.tar.gz'), '_foo')
         self.assertEqual(wd.name('file', '.bazrc'), '.bazrc')
+
+    def test_archive_file_suffix(self):
+        wd = WorkingDirManager(archive_file_suffix='.file')
+
+        wd.add('archive', 'foo.tar.gz')
+
+        self.assertEqual(
+            wd.name('archive', 'foo.tar.gz'), 'foo')
+        self.assertEqual(
+            wd.name('archive_file', 'foo.tar.gz'), 'foo.tar.gz.file')
