@@ -1,5 +1,6 @@
 # Copyright 2009-2015 Yelp and Contributors
-# Copyright 2016-2017 Yelp
+# Copyright 2016-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,9 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Tests of all the amazing utilities in mrjob.util"""
-import bz2
-import gzip
-import optparse
 import os
 import shutil
 import sys
@@ -23,27 +21,26 @@ import tempfile
 from io import BytesIO
 from subprocess import PIPE
 from subprocess import Popen
-from unittest import TestCase
 
 from mrjob.py2 import PY2
 from mrjob.util import cmd_line
 from mrjob.util import file_ext
-from mrjob.util import parse_and_save_options
 from mrjob.util import random_identifier
-from mrjob.util import read_file
-from mrjob.util import read_input
 from mrjob.util import safeeval
+from mrjob.util import save_sys_std
 from mrjob.util import to_lines
 from mrjob.util import unarchive
 from mrjob.util import unique
 from mrjob.util import which
 
+from tests.py2 import Mock
 from tests.py2 import patch
+from tests.sandbox import BasicTestCase
 from tests.sandbox import SandboxedTestCase
 from tests.sandbox import random_seed
 
 
-class ToLinesTestCase(TestCase):
+class ToLinesTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(
@@ -90,7 +87,7 @@ class ToLinesTestCase(TestCase):
             [b'a' * 10000 + b'\n', b'b' * 1000 + b'\n', b'last\n'])
 
 
-class CmdLineTestCase(TestCase):
+class CmdLineTestCase(BasicTestCase):
 
     def test_cmd_line(self):
         self.assertEqual(cmd_line(['cut', '-f', 2, '-d', ' ']),
@@ -103,7 +100,7 @@ class CmdLineTestCase(TestCase):
 # expand_path() is tested by tests.test_conf.CombineAndExpandPathsTestCase
 
 
-class FileExtTestCase(TestCase):
+class FileExtTestCase(BasicTestCase):
 
     def test_file_ext(self):
         self.assertEqual(file_ext('foo.zip'), '.zip')
@@ -113,148 +110,13 @@ class FileExtTestCase(TestCase):
         self.assertEqual(file_ext('README,v'), '')
         self.assertEqual(file_ext('README.txt,v'), '.txt,v')
 
-
-class ParseAndSaveOptionsTestCase(TestCase):
-
-    def setUp(self):
-        self.setup_options()
-
-    def setup_options(self):
-        self.original_parser = optparse.OptionParser(
-            usage="don't", description='go away')
-        self.original_group = optparse.OptionGroup(self.original_parser, '?')
-        self.original_parser.add_option_group(self.original_group)
-
-        self.original_parser.add_option(
-            '-b', '--no-a', dest='a', action='store_false')
-        self.original_parser.add_option(
-            '-a', '--yes-a', dest='a', action='store_true', default=False)
-        self.original_group.add_option('-x', '--xx', dest='x', action='store')
-        self.original_group.add_option(
-            '-y', '--yy', dest='y', action='store', nargs=2)
-
-        self.new_parser = optparse.OptionParser()
-        self.new_group_1 = optparse.OptionGroup(self.new_parser, '?')
-        self.new_group_2 = optparse.OptionGroup(self.new_parser, '?')
-        self.new_parser.add_option_group(self.new_group_1)
-        self.new_parser.add_option_group(self.new_group_2)
-
-    def test_parse_and_save_simple(self):
-        args = ['x.py', '-b', '-a', '--no-a',
-                '-x', 'x', '-y', 'y', 'ynot', '-x', 'z']
-
-        self.assertEqual(
-            dict(parse_and_save_options(self.original_parser, args)),
-            {
-                'a': ['-b', '-a', '--no-a'],
-                'x': ['-x', 'x', '-x', 'z'],
-                'y': ['-y', 'y', 'ynot']
-            })
-
-    def test_parse_and_save_with_dashes(self):
-        args = ['x.py', '-b', '-a', '--no-a',
-                '-x', 'x', '-y', 'y', 'ynot', '-x', 'z',
-                '--', 'ignore', 'these', 'args']
-        self.assertEqual(
-            dict(parse_and_save_options(self.original_parser, args)),
-            {
-                'a': ['-b', '-a', '--no-a'],
-                'x': ['-x', 'x', '-x', 'z'],
-                'y': ['-y', 'y', 'ynot']
-            })
+    def ignore_initial_dots(self):
+        self.assertEqual(file_ext('.emacs'), '')
+        self.assertEqual(file_ext('.mrjob.conf'), '.conf')
+        self.assertEqual(file_ext('...dots.txt'), '.txt')
 
 
-class DeprecatedReadInputTestCase(SandboxedTestCase):
-
-    def setUp(self):
-        super(DeprecatedReadInputTestCase, self).setUp()
-        self.start(patch('mrjob.util.log'))
-
-    @classmethod
-    def setUpClass(cls):
-        cls.setup_tmpdir_with_beaver_data()
-
-    @classmethod
-    def tearDownClass(cls):
-        cls.delete_tmpdir()
-
-    # we're going to put the same data in every file, so we don't
-    # have to worry about ordering
-    BEAVER_DATA = b'Beavers mate for life.\n'
-
-    @classmethod
-    def setup_tmpdir_with_beaver_data(self):
-        self.tmpdir = tempfile.mkdtemp()
-
-        def write_beaver_data_and_close(f):
-            f.write(self.BEAVER_DATA)
-            f.close()
-
-        write_beaver_data_and_close(
-            open(os.path.join(self.tmpdir, 'beavers.txt'), 'wb'))
-        write_beaver_data_and_close(
-            gzip.GzipFile(os.path.join(self.tmpdir, 'beavers.gz'), 'wb'))
-        write_beaver_data_and_close(
-            bz2.BZ2File(os.path.join(self.tmpdir, 'beavers.bz2'), 'wb'))
-
-        os.mkdir(os.path.join(self.tmpdir, 'beavers'))
-        write_beaver_data_and_close(
-            open(os.path.join(self.tmpdir, 'beavers/README.txt'), 'wb'))
-
-    @classmethod
-    def delete_tmpdir(self):
-        shutil.rmtree(self.tmpdir)
-
-    def test_stdin(self):
-        lines = read_input('-', stdin=BytesIO(self.BEAVER_DATA))
-        self.assertEqual(list(lines), [self.BEAVER_DATA])
-
-    def test_stdin_can_be_iterator(self):
-        lines = read_input('-', stdin=[self.BEAVER_DATA] * 5)
-        self.assertEqual(list(lines), [self.BEAVER_DATA] * 5)
-
-    def test_normal_file(self):
-        lines = read_input(os.path.join(self.tmpdir, 'beavers'))
-        self.assertEqual(list(lines), [self.BEAVER_DATA])
-
-    def test_gz_file(self):
-        lines = read_input(os.path.join(self.tmpdir, 'beavers.gz'))
-        self.assertEqual(list(lines), [self.BEAVER_DATA])
-
-    def test_bz2_file(self):
-        lines = read_input(os.path.join(self.tmpdir, 'beavers.bz2'))
-        self.assertEqual(list(lines), [self.BEAVER_DATA])
-
-    def test_glob(self):
-        lines = read_input(os.path.join(self.tmpdir, 'beavers.*'))
-        self.assertEqual(list(lines), [self.BEAVER_DATA] * 3)
-
-    def test_dir(self):
-        lines = read_input(os.path.join(self.tmpdir, 'beavers/'))
-        self.assertEqual(list(lines), [self.BEAVER_DATA])
-
-    def test_dir_recursion(self):
-        lines = read_input(self.tmpdir)
-        self.assertEqual(list(lines), [self.BEAVER_DATA] * 4)
-
-    def test_glob_including_dir(self):
-        lines = read_input(os.path.join(self.tmpdir, 'beavers*'))
-        self.assertEqual(list(lines), [self.BEAVER_DATA] * 4)
-
-    def test_bad_path(self):
-        # read_input is a generator, so we won't get an error
-        # until we try to read from it
-        self.assertRaises(IOError, list,
-                          read_input(os.path.join(self.tmpdir, 'lions')))
-
-    def test_bad_glob(self):
-        # read_input is a generator, so we won't get an error
-        # until we try to read from it
-        self.assertRaises(IOError, list,
-                          read_input(os.path.join(self.tmpdir, 'lions*')))
-
-
-class SafeEvalTestCase(TestCase):
+class SafeEvalTestCase(BasicTestCase):
 
     def test_simple_data_structures(self):
         # try unrepr-ing a bunch of simple data structures
@@ -289,7 +151,7 @@ class SafeEvalTestCase(TestCase):
                              range_type(3))
 
 
-class ArchiveTestCase(TestCase):
+class ArchiveTestCase(BasicTestCase):
 
     def setUp(self):
         self.setup_tmp_dir()
@@ -431,110 +293,7 @@ class OnlyReadWrapper(object):
         return self.fp.read(*args, **kwargs)
 
 
-class DeprecatedReadFileTestCase(SandboxedTestCase):
-
-    def setUp(self):
-        super(DeprecatedReadFileTestCase, self).setUp()
-        self.start(patch('mrjob.util.log'))
-
-    def test_read_uncompressed_file(self):
-        input_path = os.path.join(self.tmp_dir, 'input')
-        with open(input_path, 'wb') as input_file:
-            input_file.write(b'bar\nfoo\n')
-
-        output = []
-        for line in read_file(input_path):
-            output.append(line)
-
-        self.assertEqual(output, [b'bar\n', b'foo\n'])
-
-    def test_read_uncompressed_file_from_fileobj(self):
-        input_path = os.path.join(self.tmp_dir, 'input')
-        with open(input_path, 'wb') as input_file:
-            input_file.write(b'bar\nfoo\n')
-
-        output = []
-        with open(input_path, 'rb') as f:
-            for line in read_file(input_path, fileobj=f):
-                output.append(line)
-
-        self.assertEqual(output, [b'bar\n', b'foo\n'])
-
-    def test_read_gz_file(self):
-        input_gz_path = os.path.join(self.tmp_dir, 'input.gz')
-        input_gz = gzip.GzipFile(input_gz_path, 'wb')
-        input_gz.write(b'foo\nbar\n')
-        input_gz.close()
-
-        output = []
-        for line in read_file(input_gz_path):
-            output.append(line)
-
-        self.assertEqual(output, [b'foo\n', b'bar\n'])
-
-    def test_read_bz2_file(self):
-        input_bz2_path = os.path.join(self.tmp_dir, 'input.bz2')
-        input_bz2 = bz2.BZ2File(input_bz2_path, 'wb')
-        input_bz2.write(b'bar\nbar\nfoo\n')
-        input_bz2.close()
-
-        output = []
-        for line in read_file(input_bz2_path):
-            output.append(line)
-
-        self.assertEqual(output, [b'bar\n', b'bar\n', b'foo\n'])
-
-    def test_read_large_bz2_file(self):
-        # catch incorrect use of bz2 library (Issue #814)
-
-        input_bz2_path = os.path.join(self.tmp_dir, 'input.bz2')
-        input_bz2 = bz2.BZ2File(input_bz2_path, 'wb')
-
-        # can't just repeat same value, because we need the file to be
-        # compressed! 50000 lines is too few to catch the bug.
-        with random_seed(0):
-            for _ in range(100000):
-                input_bz2.write((random_identifier() + '\n').encode('ascii'))
-            input_bz2.close()
-
-        # now expect to read back the same bytes
-        with random_seed(0):
-            num_lines = 0
-            for line in read_file(input_bz2_path):
-                self.assertEqual(line,
-                                 (random_identifier() + '\n').encode('ascii'))
-                num_lines += 1
-
-            self.assertEqual(num_lines, 100000)
-
-    def test_read_gz_file_from_fileobj(self):
-        input_gz_path = os.path.join(self.tmp_dir, 'input.gz')
-        input_gz = gzip.GzipFile(input_gz_path, 'wb')
-        input_gz.write(b'foo\nbar\n')
-        input_gz.close()
-
-        output = []
-        with open(input_gz_path, 'rb') as f:
-            for line in read_file(input_gz_path, fileobj=OnlyReadWrapper(f)):
-                output.append(line)
-
-        self.assertEqual(output, [b'foo\n', b'bar\n'])
-
-    def test_read_bz2_file_from_fileobj(self):
-        input_bz2_path = os.path.join(self.tmp_dir, 'input.bz2')
-        input_bz2 = bz2.BZ2File(input_bz2_path, 'wb')
-        input_bz2.write(b'bar\nbar\nfoo\n')
-        input_bz2.close()
-
-        output = []
-        with open(input_bz2_path, 'rb') as f:
-            for line in read_file(input_bz2_path, fileobj=OnlyReadWrapper(f)):
-                output.append(line)
-
-        self.assertEqual(output, [b'bar\n', b'bar\n', b'foo\n'])
-
-
-class RandomIdentifierTestCase(TestCase):
+class RandomIdentifierTestCase(BasicTestCase):
 
     def test_format(self):
         with random_seed(0):
@@ -548,7 +307,73 @@ class RandomIdentifierTestCase(TestCase):
             self.assertNotEqual(random_identifier(), random_identifier())
 
 
-class UniqueTestCase(TestCase):
+class SaveSysStdTestCase(BasicTestCase):
+
+    def setUp(self):
+        # if save_sys_std() *doesn't* work, don't mess up other tests
+        super(SaveSysStdTestCase, self).setUp()
+
+        self.stdin = self.start(patch('sys.stdin'))
+        self.stdout = self.start(patch('sys.stdout'))
+        self.stderr = self.start(patch('sys.stderr'))
+
+    def test_basic(self):
+        fake_stdin = BytesIO(b'HI')
+        fake_stdout = BytesIO()
+        fake_stderr = BytesIO()
+
+        with save_sys_std():
+            sys.stdin = fake_stdin
+            self.assertEqual(sys.stdin.read(), b'HI')
+
+            sys.stdout = fake_stdout
+            sys.stdout.write(b'Hello!\n')
+
+            sys.stderr = fake_stderr
+            sys.stderr.write(b'!!!')
+
+        self.assertEqual(sys.stdin, self.stdin)
+        self.assertEqual(sys.stdout, self.stdout)
+        self.assertEqual(sys.stderr, self.stderr)
+
+        self.assertFalse(self.stdin.read.called)
+        self.assertFalse(self.stdout.write.called)
+        self.assertFalse(self.stderr.write.called)
+
+        self.assertEqual(fake_stdout.getvalue(), b'Hello!\n')
+        self.assertEqual(fake_stderr.getvalue(), b'!!!')
+
+    def test_flushing(self):
+        fake_stderr = Mock()
+
+        with save_sys_std():
+            sys.stderr = fake_stderr
+            sys.stderr.write(b'Hello!\n')
+
+        self.assertEqual(self.stderr.flush.call_count, 1)
+        self.assertEqual(fake_stderr.flush.call_count, 1)
+
+        # stdout was never patched, so it gets flushed twice
+        self.assertEqual(self.stdout.flush.call_count, 2)
+
+        # we don't flush stdin
+        self.assertFalse(self.stdin.flush.called)
+
+    def test_bad_flush(self):
+        fake_stdout = "LOOK AT ME I'M STDOUT"
+        self.assertFalse(hasattr(fake_stdout, 'flush'))
+
+        with save_sys_std():
+            sys.stdout = fake_stdout
+
+        self.assertEqual(sys.stdout, self.stdout)
+        self.assertEqual(self.stdout.flush.call_count, 1)
+
+        # sys.stderr, which was not patched, should be flushed twice
+        self.assertEqual(self.stderr.flush.call_count, 2)
+
+
+class UniqueTestCase(BasicTestCase):
 
     def test_empty(self):
         self.assertEqual(list(unique([])), [])
