@@ -1,7 +1,7 @@
 # Copyright 2011 Yelp
 # Copyright 2012 Yelp and Contributors
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2018 Yelp
+# Copyright 2019 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ import sys
 from datetime import date
 from datetime import datetime
 from datetime import timedelta
-from unittest import TestCase
 
 import boto3
 from dateutil.parser import parse
@@ -32,6 +31,7 @@ from mrjob.tools.emr.audit_usage import _subdivide_interval_by_hour
 from mrjob.tools.emr.audit_usage import main
 
 from tests.py2 import patch
+from tests.sandbox import BasicTestCase
 from tests.tools.emr import ToolTestCase
 
 
@@ -77,7 +77,7 @@ class AuditUsageTestCase(ToolTestCase):
         self.assertEqual(self.describe_cluster_sleep.call_count, 1)
 
 
-class ClusterToFullSummaryTestCase(TestCase):
+class ClusterToFullSummaryTestCase(BasicTestCase):
 
     maxDiff = None  # show whole diff when tests fail
 
@@ -663,25 +663,7 @@ class ClusterToFullSummaryTestCase(TestCase):
             }],
         })
 
-    def test_pooled_cluster(self):
-        self._test_new_or_legacy_pooled_cluster(Tags=[
-            dict(Key='__mrjob_pool_hash',
-                 Value='0123456789abcdef0123456789abcdef'),
-            dict(Key='__mrjob_pool_name',
-                 Value='reflecting'),
-        ])
-
-    def test_legacy_pooled_cluster(self):
-        # audit clusters from previous versions of mrjob
-        self._test_new_or_legacy_pooled_cluster(
-            BootstrapActions=[
-                dict(Args=[], Name='empty'),
-                dict(Args=['pool-0123456789abcdef0123456789abcdef',
-                           'reflecting'], Name='master'),
-            ],
-        )
-
-    def _test_new_or_legacy_pooled_cluster(self, **kwargs):
+    def test_pooled_cluster(self, **kwargs):
         # same as test case above with different job keys
         cluster = dict(
             Id='j-ISFORJOB',
@@ -725,7 +707,12 @@ class ClusterToFullSummaryTestCase(TestCase):
                     ),
                 ),
             ],
-            **kwargs
+            Tags=[
+                dict(Key='__mrjob_pool_hash',
+                     Value='0123456789abcdef0123456789abcdef'),
+                dict(Key='__mrjob_pool_name',
+                     Value='reflecting'),
+            ],
         )
 
         summary = _cluster_to_full_summary(cluster)
@@ -823,7 +810,7 @@ class ClusterToFullSummaryTestCase(TestCase):
         })
 
 
-class SubdivideIntervalByDateTestCase(TestCase):
+class SubdivideIntervalByDateTestCase(BasicTestCase):
 
     def test_zero_interval(self):
         self.assertEqual(
@@ -886,7 +873,7 @@ class SubdivideIntervalByDateTestCase(TestCase):
         )
 
 
-class SubdivideIntervalByHourTestCase(TestCase):
+class SubdivideIntervalByHourTestCase(BasicTestCase):
 
     def test_zero_interval(self):
         self.assertEqual(
@@ -947,7 +934,7 @@ class SubdivideIntervalByHourTestCase(TestCase):
         )
 
 
-class PercentTestCase(TestCase):
+class PercentTestCase(BasicTestCase):
 
     def test_basic(self):
         self.assertEqual(62.5, _percent(5, 8))
