@@ -1,8 +1,8 @@
 # Copyright 2009-2012 Yelp
 # Copyright 2013 Lyft
 # Copyright 2014 Marc Abramowitz
-# Copyright 2015-2016 Yelp
-# Copyright 2017 Yelp
+# Copyright 2015-2019 Yelp
+# Copyright 2020 Affirm, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,10 +20,8 @@ import sys
 from datetime import timedelta
 
 from mrjob.aws import _boto3_now
-from mrjob.fs.s3 import S3Filesystem
-from mrjob.pool import _pool_hash_and_name
+from mrjob.pool import _pool_name
 from mrjob.py2 import StringIO
-from mrjob.tools.emr.terminate_idle_clusters import main
 from mrjob.tools.emr.terminate_idle_clusters import _maybe_terminate_clusters
 from mrjob.tools.emr.terminate_idle_clusters import _is_cluster_bootstrapping
 from mrjob.tools.emr.terminate_idle_clusters import _is_cluster_done
@@ -46,6 +44,10 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
 
     def setUp(self):
         super(ClusterTerminationTestCase, self).setUp()
+
+        # don't delete locks, so we can test they were created
+        self.start(patch('mrjob.fs.s3.S3Filesystem.rm'))
+
         self.create_fake_clusters()
 
     def create_fake_clusters(self):
@@ -93,6 +95,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         self.add_mock_emr_cluster(
             dict(
                 Id='j-EMPTY',
+                TerminationProtected=False,
                 Status=dict(
                     State='STARTING',
                     Timeline=dict(
@@ -105,6 +108,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # job that's bootstrapping
         self.add_mock_emr_cluster(dict(
             Id='j-BOOTSTRAPPING',
+            TerminationProtected=False,
             Status=dict(
                 State='BOOTSTRAPPING',
                 Timeline=dict(
@@ -118,6 +122,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         self.add_mock_emr_cluster(
             dict(
                 Id='j-CURRENTLY_RUNNING',
+                TerminationProtected=False,
                 Status=dict(
                     State='RUNNING',
                     Timeline=dict(
@@ -132,6 +137,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # finished cluster
         self.add_mock_emr_cluster(dict(
             Id='j-DONE',
+            TerminationProtected=False,
             Status=dict(
                 State='TERMINATED',
                 Timeline=dict(
@@ -146,6 +152,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # idle cluster
         self.add_mock_emr_cluster(dict(
             Id='j-DONE_AND_IDLE',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -160,6 +167,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # recognizable as a streaming step
         self.add_mock_emr_cluster(dict(
             Id='j-DONE_AND_IDLE_4_X',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -172,45 +180,10 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
                          args=['hadoop-streaming'] + self._DEFAULT_STEP_ARGS)],
         ))
 
-        # idle cluster with an active lock
-        self.add_mock_emr_cluster(dict(
-            Id='j-IDLE_AND_LOCKED',
-            Status=dict(
-                State='WAITING',
-                Timeline=dict(
-                    CreationDateTime=ago(hours=6),
-                    ReadyDateTime=ago(hours=5, minutes=5),
-                ),
-            ),
-            _Steps=[step(started=ago(hours=4), ended=ago(hours=2))],
-        ))
-        self.add_mock_s3_data({
-            'my_bucket': {
-                'locks/j-IDLE_AND_LOCKED/2': b'not_you',
-            },
-        })
-
-        # idle cluster with an expired lock
-        self.add_mock_emr_cluster(dict(
-            Id='j-IDLE_AND_EXPIRED',
-            Status=dict(
-                State='WAITING',
-                Timeline=dict(
-                    CreationDateTime=ago(hours=6),
-                    ReadyDateTime=ago(hours=5, minutes=5),
-                ),
-            ),
-            _Steps=[step(started=ago(hours=4), ended=ago(hours=2))],
-        ))
-        self.add_mock_s3_data({
-            'my_bucket': {
-                'locks/j-IDLE_AND_EXPIRED/2': b'not_you',
-            },
-        }, age=timedelta(minutes=5))
-
         # idle cluster with an expired lock
         self.add_mock_emr_cluster(dict(
             Id='j-IDLE_BUT_INCOMPLETE_STEPS',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -224,6 +197,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # custom hadoop streaming jar
         self.add_mock_emr_cluster(dict(
             Id='j-CUSTOM_DONE_AND_IDLE',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -240,9 +214,24 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
             )],
         ))
 
+        # idle cluster, termination protected
+        self.add_mock_emr_cluster(dict(
+            Id='j-IDLE_AND_PROTECTED',
+            TerminationProtected=True,
+            Status=dict(
+                State='WAITING',
+                Timeline=dict(
+                    CreationDateTime=ago(hours=6),
+                    ReadyDateTime=ago(hours=5, minutes=5),
+                ),
+            ),
+            _Steps=[step(started=ago(hours=4), ended=ago(hours=2))],
+        ))
+
         # hadoop debugging without any other steps
         self.add_mock_emr_cluster(dict(
             Id='j-DEBUG_ONLY',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -262,6 +251,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # hadoop debugging + actual job
         self.add_mock_emr_cluster(dict(
             Id='j-HADOOP_DEBUGGING',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -282,6 +272,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # should skip cancelled steps
         self.add_mock_emr_cluster(dict(
             Id='j-IDLE_AND_FAILED',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -306,6 +297,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
                 ),
             ],
             Id='j-POOLED',
+            TerminationProtected=False,
             Status=dict(
                 State='WAITING',
                 Timeline=dict(
@@ -324,6 +316,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # cluster that has had pending jobs but hasn't run them
         self.add_mock_emr_cluster(dict(
             Id='j-PENDING_BUT_IDLE',
+            TerminationProtected=False,
             Status=dict(
                 State='RUNNING',
                 Timeline=dict(
@@ -378,7 +371,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
             done=False,
             has_pending_steps=False,
             idle_for=timedelta(0),
-            pool_hash=None,
             pool_name=None,
             running=False):
 
@@ -392,37 +384,11 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
                          _cluster_has_pending_steps(mock_cluster['_Steps']))
         self.assertEqual(idle_for,
                          self.time_mock_cluster_idle(mock_cluster))
-        self.assertEqual((pool_hash, pool_name),
-                         _pool_hash_and_name(mock_cluster))
+        self.assertEqual(pool_name,
+                         _pool_name(mock_cluster))
         self.assertEqual(running,
                          _is_cluster_running(mock_cluster['_Steps']))
 
-    def _lock_contents(self, mock_cluster, steps_ahead=0):
-        fs = S3Filesystem()
-
-        contents = b''.join(fs.cat('s3://my_bucket/locks/%s/%d' % (
-            mock_cluster['Id'], len(mock_cluster['_Steps']) + steps_ahead)))
-
-        return contents or None
-
-    def assert_locked_by_terminate(self, mock_cluster, steps_ahead=1):
-        contents = self._lock_contents(mock_cluster, steps_ahead=steps_ahead)
-        self.assertIsNotNone(contents)
-        self.assertIn(b'terminate', contents)
-
-    def assert_locked_by_something_else(self, mock_cluster, steps_ahead=1):
-        contents = self._lock_contents(mock_cluster, steps_ahead=steps_ahead)
-        self.assertIsNotNone(contents)
-        self.assertNotIn(b'terminate', contents)
-
-    def assert_not_locked(self, mock_cluster, steps_ahead=1):
-        self.assertIsNone(
-            self._lock_contents(mock_cluster, steps_ahead=steps_ahead))
-
-    def assert_terminated_clusters_locked_by_terminate(self):
-        for cluster_id in self.ids_of_terminated_clusters():
-            self.assert_locked_by_terminate(self.mock_emr_clusters[cluster_id])
-
     def test_empty(self):
         self.assert_mock_cluster_is(
             self.mock_emr_clusters['j-EMPTY'],
@@ -459,12 +425,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
             idle_for=timedelta(hours=2),
         )
 
-    def test_idle_and_expired(self):
-        self.assert_mock_cluster_is(
-            self.mock_emr_clusters['j-IDLE_AND_EXPIRED'],
-            idle_for=timedelta(hours=2),
-        )
-
     def test_hadoop_debugging_cluster(self):
         self.assert_mock_cluster_is(
             self.mock_emr_clusters['j-HADOOP_DEBUGGING'],
@@ -481,7 +441,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         self.assert_mock_cluster_is(
             self.mock_emr_clusters['j-POOLED'],
             idle_for=timedelta(minutes=50),
-            pool_hash='0123456789abcdef0123456789abcdef',
             pool_name='reflecting',
         )
 
@@ -495,24 +454,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
     def test_dry_run_does_nothing(self):
         self.maybe_terminate_quietly(max_mins_idle=0.6, dry_run=True)
 
-        unlocked_ids = [
-            'j-BOOTSTRAPPING',
-            'j-CURRENTLY_RUNNING',
-            'j-CUSTOM_DONE_AND_IDLE',
-            'j-DEBUG_ONLY',
-            'j-DONE',
-            'j-DONE_AND_IDLE',
-            'j-DONE_AND_IDLE_4_X',
-            'j-EMPTY',
-            'j-HADOOP_DEBUGGING',
-            'j-IDLE_AND_FAILED',
-            'j-IDLE_BUT_INCOMPLETE_STEPS',
-            'j-PENDING_BUT_IDLE',
-            'j-POOLED'
-        ]
-        for cluster_id in unlocked_ids:
-            self.assert_not_locked(self.mock_emr_clusters[cluster_id])
-
         self.assertEqual(self.ids_of_terminated_clusters(), [])
 
     def test_increasing_idle_time(self):
@@ -542,12 +483,11 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
 
         self.maybe_terminate_quietly(max_mins_idle=60)
 
-        self.assert_terminated_clusters_locked_by_terminate()
         self.assertEqual(self.ids_of_terminated_clusters(),
                          ['j-CUSTOM_DONE_AND_IDLE',
                           'j-DEBUG_ONLY',
                           'j-DONE_AND_IDLE', 'j-DONE_AND_IDLE_4_X',
-                          'j-HADOOP_DEBUGGING', 'j-IDLE_AND_EXPIRED',
+                          'j-HADOOP_DEBUGGING',
                           'j-IDLE_AND_FAILED', 'j-PENDING_BUT_IDLE'])
 
     def test_one_hour_is_the_default(self):
@@ -555,12 +495,11 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
 
         self.maybe_terminate_quietly()
 
-        self.assert_terminated_clusters_locked_by_terminate()
         self.assertEqual(self.ids_of_terminated_clusters(),
                          ['j-CUSTOM_DONE_AND_IDLE',
                           'j-DEBUG_ONLY',
                           'j-DONE_AND_IDLE', 'j-DONE_AND_IDLE_4_X',
-                          'j-HADOOP_DEBUGGING', 'j-IDLE_AND_EXPIRED',
+                          'j-HADOOP_DEBUGGING',
                           'j-IDLE_AND_FAILED', 'j-PENDING_BUT_IDLE'])
 
     def test_zero_idle_time(self):
@@ -568,12 +507,11 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
 
         self.maybe_terminate_quietly(max_mins_idle=0)
 
-        self.assert_terminated_clusters_locked_by_terminate()
         self.assertEqual(self.ids_of_terminated_clusters(),
                          ['j-CUSTOM_DONE_AND_IDLE',
                           'j-DEBUG_ONLY',
                           'j-DONE_AND_IDLE', 'j-DONE_AND_IDLE_4_X',
-                          'j-HADOOP_DEBUGGING', 'j-IDLE_AND_EXPIRED',
+                          'j-HADOOP_DEBUGGING',
                           'j-IDLE_AND_FAILED', 'j-PENDING_BUT_IDLE',
                           'j-POOLED'])
 
@@ -582,8 +520,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
 
         self.maybe_terminate_quietly(pooled_only=True)
 
-        self.assert_terminated_clusters_locked_by_terminate()
-
         # pooled job was not idle for an hour (the default)
         self.assertEqual(self.ids_of_terminated_clusters(), [])
 
@@ -596,13 +532,11 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
 
         self.maybe_terminate_quietly(unpooled_only=True)
 
-        self.assert_terminated_clusters_locked_by_terminate()
-
         self.assertEqual(self.ids_of_terminated_clusters(),
                          ['j-CUSTOM_DONE_AND_IDLE',
                           'j-DEBUG_ONLY',
                           'j-DONE_AND_IDLE', 'j-DONE_AND_IDLE_4_X',
-                          'j-HADOOP_DEBUGGING', 'j-IDLE_AND_EXPIRED',
+                          'j-HADOOP_DEBUGGING',
                           'j-IDLE_AND_FAILED', 'j-PENDING_BUT_IDLE'])
 
         self.maybe_terminate_quietly(unpooled_only=True, max_mins_idle=0.6)
@@ -611,7 +545,7 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
                          ['j-CUSTOM_DONE_AND_IDLE',
                           'j-DEBUG_ONLY',
                           'j-DONE_AND_IDLE', 'j-DONE_AND_IDLE_4_X',
-                          'j-HADOOP_DEBUGGING', 'j-IDLE_AND_EXPIRED',
+                          'j-HADOOP_DEBUGGING',
                           'j-IDLE_AND_FAILED', 'j-PENDING_BUT_IDLE'])
 
     def test_terminate_by_pool_name(self):
@@ -625,8 +559,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         # right pool name
         self.maybe_terminate_quietly(pool_name='reflecting', max_mins_idle=0.6)
 
-        self.assert_terminated_clusters_locked_by_terminate()
-
         self.assertEqual(self.ids_of_terminated_clusters(), ['j-POOLED'])
 
     def test_its_quiet_too_quiet(self):
@@ -646,8 +578,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         ' was idle for 2:00:00',
         'Terminated cluster j-DONE_AND_IDLE_4_X (DONE_AND_IDLE_4_X);'
         ' was idle for 2:00:00',
-        'Terminated cluster j-IDLE_AND_EXPIRED (IDLE_AND_EXPIRED);'
-        ' was idle for 2:00:00',
         'Terminated cluster j-IDLE_AND_FAILED (IDLE_AND_FAILED);'
         ' was idle for 3:00:00',
         'Terminated cluster j-HADOOP_DEBUGGING (HADOOP_DEBUGGING);'
@@ -670,7 +600,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
             'j-DONE_AND_IDLE',
             'j-DONE_AND_IDLE_4_X',
             'j-HADOOP_DEBUGGING',
-            'j-IDLE_AND_EXPIRED',
             'j-IDLE_AND_FAILED',
             'j-PENDING_BUT_IDLE',
             'j-POOLED',
@@ -681,14 +610,6 @@ class ClusterTerminationTestCase(MockBoto3TestCase):
         self.maybe_terminate_quietly(
             stdout=stdout, max_mins_idle=0.6, dry_run=True)
 
-        # dry_run doesn't actually try to lock
-        expected_stdout_lines = self.EXPECTED_STDOUT_LINES + [
-            'Terminated cluster j-IDLE_AND_LOCKED (IDLE_AND_LOCKED);'
-            ' was idle for 2:00:00']
-
-        self.assertEqual(set(stdout.getvalue().splitlines()),
-                         set(expected_stdout_lines))
-
         # shouldn't *actually* terminate clusters
         self.assertEqual(self.ids_of_terminated_clusters(), [])
 
@@ -704,21 +625,3 @@ class DeprecatedSwitchesTestCase(SandboxedTestCase):
 
         self.log = self.start(
             patch('mrjob.tools.emr.terminate_idle_clusters.log'))
-
-    def test_deprecated_max_hours_idle(self):
-        main(['--max-hours-idle', '2'])
-
-        self.assertEqual(
-            self._maybe_terminate_clusters.call_args[1]['max_mins_idle'],
-            120)
-
-        self.assertTrue(self.log.warning.called)
-
-    def test_deprecated_mins_to_end_of_hour(self):
-        main(['--mins-to-end-of-hour', '5'])
-
-        self.assertNotIn(
-            'mins_to_end_of_hour',
-            self._maybe_terminate_clusters.call_args[1])
-
-        self.assertTrue(self.log.warning.called)
